import * as token from '../token';
import { BlockStatement } from './statement';
import { Identifier } from './literal';
import { Node } from '.';
import { Transpiler } from '../transpiler';
export interface Expression extends Node {
    expressionNode(): any;
}
export declare class PrefixExpression implements Expression {
    Token: token.Token;
    Operator: string;
    Right?: Expression;
    NodeName: string;
    constructor(Token: token.Token, // The prefix token, e.g. !
    Operator: string, Right?: Expression, NodeName?: string);
    expressionNode(): void;
    TokenLiteral(): string;
    String(): string;
    UnParse(ctx: Transpiler): string;
}
export declare class InfixExpression implements Expression {
    Token: token.Token;
    Left: Expression;
    Operator: string;
    Right?: Expression;
    NodeName: string;
    constructor(Token: token.Token, // The operator token, e.g. +
    Left: Expression, Operator: string, Right?: Expression, NodeName?: string);
    expressionNode(): void;
    TokenLiteral(): string;
    String(): string;
    UnParse(ctx: Transpiler): string;
}
export declare class CallExpression implements Expression {
    Token: token.Token;
    Function: Expression;
    Arguments: Expression[];
    NodeName: string;
    constructor(Token: token.Token, // The '(' token
    Function: Expression, // Identifier or FunctionLiteral
    Arguments?: Expression[], NodeName?: string);
    expressionNode(): void;
    TokenLiteral(): string;
    String(): string;
    UnParse(ctx: Transpiler): string;
}
export declare class NewExpression implements Expression {
    Token: token.Token;
    Name?: Identifier;
    NodeName: string;
    constructor(Token: token.Token, // the token.NEW token
    Name?: Identifier, NodeName?: string);
    expressionNode(): void;
    TokenLiteral(): string;
    String(): string;
    UnParse(ctx: Transpiler): string;
}
export declare class ExecExpression implements Expression {
    Token: token.Token;
    Name?: Expression;
    Callback?: BlockStatement;
    NodeName: string;
    constructor(Token: token.Token, // the token.NEW token
    Name?: Expression, Callback?: BlockStatement, NodeName?: string);
    expressionNode(): void;
    TokenLiteral(): string;
    String(): string;
    UnParse(ctx: Transpiler): string;
}
export declare class StreamExpression implements Expression {
    Token: token.Token;
    Direction: token.STREAM_DIRECTION;
    Source?: Expression;
    Sink?: Expression;
    Transforms: Expression[];
    NodeName: string;
    constructor(Token: token.Token, // The '->' or '<- tokens
    Direction: token.STREAM_DIRECTION, Source?: Expression, Sink?: Expression, Transforms?: Expression[], NodeName?: string);
    expressionNode(): void;
    TokenLiteral(): string;
    String(): string;
    UnParse(ctx: Transpiler): string;
}
export declare class IndexExpression implements Expression {
    Token: token.Token;
    Left: Expression;
    Index?: Expression;
    NodeName: string;
    constructor(Token: token.Token, // The [ token
    Left: Expression, Index?: Expression, NodeName?: string);
    expressionNode(): void;
    TokenLiteral(): string;
    String(): string;
    UnParse(ctx: Transpiler): string;
}
export declare class IndexAssignmentExpression implements Expression {
    Token: token.Token;
    Left: Expression;
    Index?: Expression;
    Assignment?: Expression;
    NodeName: string;
    constructor(Token: token.Token, // The [ token
    Left: Expression, Index?: Expression, Assignment?: Expression, NodeName?: string);
    expressionNode(): void;
    TokenLiteral(): string;
    String(): string;
    UnParse(ctx: Transpiler): string;
}
