import * as token from '../token';
import { BlockStatement } from './statement';
import { Expression } from './expression';
import { Transpiler } from '../transpiler';
export declare class Identifier {
    Token: token.Token;
    Value?: string;
    NodeName: string;
    constructor(Token: token.Token, // the token.IDENT token
    Value?: string, NodeName?: string);
    expressionNode(): void;
    TokenLiteral(): string;
    String(): string;
    UnParse(ctx: Transpiler): string;
}
export declare class BooleanLiteral implements Expression {
    Token: token.Token;
    Value?: boolean;
    NodeName: string;
    constructor(Token: token.Token, Value?: boolean, NodeName?: string);
    expressionNode(): void;
    TokenLiteral(): string;
    String(): string;
    UnParse(ctx: Transpiler): string;
}
export declare class IntegerLiteral implements Expression {
    Token: token.Token;
    Value?: number;
    NodeName: string;
    constructor(Token: token.Token, Value?: number, NodeName?: string);
    expressionNode(): void;
    TokenLiteral(): string;
    String(): string;
    UnParse(ctx: Transpiler): string;
}
export declare class FloatLiteral implements Expression {
    Token: token.Token;
    Value?: number;
    NodeName: string;
    constructor(Token: token.Token, Value?: number, NodeName?: string);
    expressionNode(): void;
    TokenLiteral(): string;
    String(): string;
    UnParse(ctx: Transpiler): string;
}
export declare class StringLiteral implements Expression {
    Token: token.Token;
    Value?: string;
    NodeName: string;
    modifiers?: number[];
    constructor(Token: token.Token, Value?: string, NodeName?: string, modifiers?: number[]);
    expressionNode(): void;
    TokenLiteral(): string;
    String(): string;
    UnParse(ctx: Transpiler): string;
}
export declare class FunctionLiteral implements Expression {
    Token: token.Token;
    Parameters: Identifier[];
    Body?: BlockStatement;
    ObjectContext?: any;
    ParameterTypes?: string[];
    ReturnType?: string;
    FunctionName?: string;
    NodeName: string;
    constructor(Token: token.Token, // The 'fn' token
    Parameters?: Identifier[], Body?: BlockStatement, ObjectContext?: any, ParameterTypes?: string[], ReturnType?: string, FunctionName?: string, NodeName?: string);
    expressionNode(): void;
    TokenLiteral(): string;
    String(): string;
    UnParse(ctx: Transpiler): string;
}
export declare class PureFunctionLiteral implements Expression {
    Token: token.Token;
    Parameters: Identifier[];
    Body?: BlockStatement;
    ReturnType?: string;
    ParameterTypes?: string[];
    FunctionName?: string;
    NodeName: string;
    constructor(Token: token.Token, Parameters?: Identifier[], Body?: BlockStatement, ReturnType?: string, ParameterTypes?: string[], FunctionName?: string, NodeName?: string);
    expressionNode(): void;
    TokenLiteral(): string;
    String(): string;
    UnParse(ctx: Transpiler): string;
}
export declare class ArrayLiteral implements Expression {
    Token: token.Token;
    Elements: Expression[];
    NodeName: string;
    constructor(Token: token.Token, // the '[' token
    Elements?: Expression[], NodeName?: string);
    expressionNode(): void;
    TokenLiteral(): string;
    String(): string;
    UnParse(ctx: Transpiler): string;
}
export declare class HashLiteral implements Expression {
    Token: token.Token;
    Pairs?: {
        [index: string]: Expression;
    };
    Modifiers: {
        [pairName: string]: number[];
    };
    NodeName: string;
    constructor(Token: token.Token, // the '{' token
    Pairs?: {
        [index: string]: Expression;
    }, Modifiers?: {
        [pairName: string]: number[];
    }, NodeName?: string);
    expressionNode(): void;
    TokenLiteral(): string;
    String(): string;
    UnParse(ctx: Transpiler): string;
}
