"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Identifier = /** @class */ (function () {
    function Identifier(Token, // the token.IDENT token
    Value, NodeName) {
        if (NodeName === void 0) { NodeName = "Identifier"; }
        this.Token = Token;
        this.Value = Value;
        this.NodeName = NodeName;
    }
    Identifier.prototype.expressionNode = function () { };
    Identifier.prototype.TokenLiteral = function () {
        return this.Token.Literal;
    };
    Identifier.prototype.String = function () { return this.Value; };
    Identifier.prototype.UnParse = function (ctx) {
        return ctx.replaceBuiltinIdentifiers(this.Token.Literal);
    };
    return Identifier;
}());
exports.Identifier = Identifier;
var BooleanLiteral = /** @class */ (function () {
    function BooleanLiteral(Token, Value, NodeName) {
        if (NodeName === void 0) { NodeName = "Boolean"; }
        this.Token = Token;
        this.Value = Value;
        this.NodeName = NodeName;
    }
    BooleanLiteral.prototype.expressionNode = function () { };
    BooleanLiteral.prototype.TokenLiteral = function () { return this.Token.Literal; };
    BooleanLiteral.prototype.String = function () { return this.Token.Literal; };
    BooleanLiteral.prototype.UnParse = function (ctx) {
        return this.String();
    };
    return BooleanLiteral;
}());
exports.BooleanLiteral = BooleanLiteral;
var IntegerLiteral = /** @class */ (function () {
    function IntegerLiteral(Token, Value, NodeName) {
        if (NodeName === void 0) { NodeName = "IntegerLiteral"; }
        this.Token = Token;
        this.Value = Value;
        this.NodeName = NodeName;
    }
    IntegerLiteral.prototype.expressionNode = function () { };
    IntegerLiteral.prototype.TokenLiteral = function () { return this.Token.Literal; };
    IntegerLiteral.prototype.String = function () { return this.Token.Literal; };
    IntegerLiteral.prototype.UnParse = function (ctx) {
        return this.String();
    };
    return IntegerLiteral;
}());
exports.IntegerLiteral = IntegerLiteral;
var FloatLiteral = /** @class */ (function () {
    function FloatLiteral(Token, Value, NodeName) {
        if (NodeName === void 0) { NodeName = "FloatLiteral"; }
        this.Token = Token;
        this.Value = Value;
        this.NodeName = NodeName;
    }
    FloatLiteral.prototype.expressionNode = function () { };
    FloatLiteral.prototype.TokenLiteral = function () { return this.Token.Literal; };
    FloatLiteral.prototype.String = function () { return this.Token.Literal; };
    FloatLiteral.prototype.UnParse = function (ctx) {
        return this.String();
    };
    return FloatLiteral;
}());
exports.FloatLiteral = FloatLiteral;
var StringLiteral = /** @class */ (function () {
    function StringLiteral(Token, Value, NodeName, modifiers) {
        if (NodeName === void 0) { NodeName = "StringLiteral"; }
        this.Token = Token;
        this.Value = Value;
        this.NodeName = NodeName;
        this.modifiers = modifiers;
    }
    StringLiteral.prototype.expressionNode = function () { };
    StringLiteral.prototype.TokenLiteral = function () { return this.Token.Literal; };
    StringLiteral.prototype.String = function () { return this.Token.Literal; };
    StringLiteral.prototype.UnParse = function (ctx) {
        return "\"" + this.String() + "\"";
    };
    return StringLiteral;
}());
exports.StringLiteral = StringLiteral;
var FunctionLiteral = /** @class */ (function () {
    function FunctionLiteral(Token, // The 'fn' token
    Parameters, Body, ObjectContext, ParameterTypes, ReturnType, FunctionName, NodeName) {
        if (Parameters === void 0) { Parameters = []; }
        if (NodeName === void 0) { NodeName = "FunctionLiteral"; }
        this.Token = Token;
        this.Parameters = Parameters;
        this.Body = Body;
        this.ObjectContext = ObjectContext;
        this.ParameterTypes = ParameterTypes;
        this.ReturnType = ReturnType;
        this.FunctionName = FunctionName;
        this.NodeName = NodeName;
    }
    FunctionLiteral.prototype.expressionNode = function () { };
    FunctionLiteral.prototype.TokenLiteral = function () { return this.Token.Literal; };
    FunctionLiteral.prototype.String = function () {
        var out = "", params = [];
        this.Parameters.forEach(function (p) {
            params.push(p.String());
        });
        out += this.TokenLiteral();
        out += "(" + params.join(", ") + ") ";
        out += this.Body.String();
        return out;
    };
    FunctionLiteral.prototype.UnParse = function (ctx) {
        return ctx.transpile(this);
    };
    return FunctionLiteral;
}());
exports.FunctionLiteral = FunctionLiteral;
var PureFunctionLiteral = /** @class */ (function () {
    function PureFunctionLiteral(Token, Parameters, Body, ReturnType, ParameterTypes, FunctionName, NodeName) {
        if (Parameters === void 0) { Parameters = []; }
        if (NodeName === void 0) { NodeName = "PureFunctionLiteral"; }
        this.Token = Token;
        this.Parameters = Parameters;
        this.Body = Body;
        this.ReturnType = ReturnType;
        this.ParameterTypes = ParameterTypes;
        this.FunctionName = FunctionName;
        this.NodeName = NodeName;
    }
    PureFunctionLiteral.prototype.expressionNode = function () { };
    PureFunctionLiteral.prototype.TokenLiteral = function () { return this.Token.Literal; };
    PureFunctionLiteral.prototype.String = function () {
        var out = "", params = [];
        this.Parameters.forEach(function (p) {
            params.push(p.String());
        });
        out += this.TokenLiteral();
        out += "(" + params.join(", ") + ") ";
        out += this.Body.String();
        return out;
    };
    PureFunctionLiteral.prototype.UnParse = function (ctx) {
        return ctx.transpile(this);
    };
    return PureFunctionLiteral;
}());
exports.PureFunctionLiteral = PureFunctionLiteral;
var ArrayLiteral = /** @class */ (function () {
    function ArrayLiteral(Token, // the '[' token
    Elements, NodeName) {
        if (Elements === void 0) { Elements = []; }
        if (NodeName === void 0) { NodeName = "ArrayLiteral"; }
        this.Token = Token;
        this.Elements = Elements;
        this.NodeName = NodeName;
    }
    ArrayLiteral.prototype.expressionNode = function () { };
    ArrayLiteral.prototype.TokenLiteral = function () { return this.Token.Literal; };
    ArrayLiteral.prototype.String = function () {
        var out = "", elements = [];
        this.Elements.forEach(function (el) {
            elements.push(el.String());
        });
        out += "[";
        out += elements.join(", ");
        out += "]";
        return out;
    };
    ArrayLiteral.prototype.UnParse = function (ctx) {
        return ctx.transpile(this);
    };
    return ArrayLiteral;
}());
exports.ArrayLiteral = ArrayLiteral;
var HashLiteral = /** @class */ (function () {
    function HashLiteral(Token, // the '{' token
    Pairs, Modifiers, NodeName) {
        if (Modifiers === void 0) { Modifiers = {}; }
        if (NodeName === void 0) { NodeName = "HashLiteral"; }
        this.Token = Token;
        this.Pairs = Pairs;
        this.Modifiers = Modifiers;
        this.NodeName = NodeName;
    }
    HashLiteral.prototype.expressionNode = function () { };
    HashLiteral.prototype.TokenLiteral = function () { return this.Token.Literal; };
    HashLiteral.prototype.String = function () {
        var out = "", pairs = [];
        // this.Pairs.forEach( (value, key) =>{
        // 	pairs.push( key.String()+":"+value.String() )
        // })
        out += "{";
        out += pairs.join(", ");
        out += "}";
        return out;
    };
    HashLiteral.prototype.UnParse = function (ctx) {
        return ctx.transpile(this);
    };
    return HashLiteral;
}());
exports.HashLiteral = HashLiteral;
