import * as token from '../token';
import { Identifier, HashLiteral } from './literal';
import { Expression } from './expression';
import { Transpiler } from '../transpiler';
import { Statement, IBlockStatement } from '.';
export declare class AssignmentStatement implements Statement {
    Name?: Identifier;
    Value?: Expression;
    NodeName: string;
    constructor(Name?: Identifier, Value?: Expression, NodeName?: string);
    statementNode(): void;
    TokenLiteral(): string;
    String(): string;
    UnParse(ctx: Transpiler): string;
}
export declare class LetStatement implements Statement {
    Token: token.Token;
    Name?: Identifier;
    Value?: Expression;
    Vars?: [Identifier, Expression][];
    Types: string[];
    NodeName: string;
    dataType: string;
    constructor(Token: token.Token, // the token.LET token
    Name?: Identifier, Value?: Expression, Vars?: [Identifier, Expression][], Types?: string[], NodeName?: string);
    statementNode(): void;
    TokenLiteral(): string;
    String(): string;
    UnParse(ctx: Transpiler): string;
}
export declare class ClassStatement implements Statement {
    Token: token.Token;
    Name?: Identifier;
    Value?: HashLiteral;
    NodeName: string;
    constructor(Token: token.Token, // the token.CLASS token
    Name?: Identifier, Value?: HashLiteral, NodeName?: string);
    statementNode(): void;
    TokenLiteral(): string;
    String(): string;
    UnParse(ctx: Transpiler): string;
}
export declare class ReturnStatement implements Statement {
    Token: token.Token;
    ReturnValue?: Expression;
    NodeName: string;
    constructor(Token: token.Token, // the 'return' token
    ReturnValue?: Expression, NodeName?: string);
    statementNode(): void;
    TokenLiteral(): string;
    String(): string;
    UnParse(ctx: Transpiler): string;
}
export declare class ExpressionStatement implements Statement {
    Token: token.Token;
    Expression?: Expression;
    NodeName: string;
    constructor(Token: token.Token, // the first token of the expression
    Expression?: Expression, NodeName?: string);
    statementNode(): void;
    TokenLiteral(): string;
    String(): string;
    UnParse(ctx: Transpiler): string;
}
export declare class BlockStatement implements IBlockStatement {
    Token: token.Token;
    Statements: Statement[];
    NodeName: string;
    constructor(Token: token.Token, // the { token
    Statements?: Statement[], NodeName?: string);
    statementNode(): void;
    TokenLiteral(): string;
    String(): string;
    UnParse(ctx: Transpiler): string;
}
export declare class IfStatement implements Statement {
    Token: token.Token;
    Condition?: Expression;
    Consequence?: BlockStatement;
    Alternative?: BlockStatement;
    NodeName: string;
    constructor(Token: token.Token, // The 'if' token
    Condition?: Expression, Consequence?: BlockStatement, Alternative?: BlockStatement, NodeName?: string);
    statementNode(): void;
    TokenLiteral(): string;
    String(): string;
    UnParse(ctx: Transpiler): string;
}
export declare class ForStatement implements Statement {
    Token: token.Token;
    Element?: Identifier;
    Range?: Expression;
    Consequence?: BlockStatement;
    NodeName: string;
    constructor(Token: token.Token, // The 'for' token
    Element?: Identifier, Range?: Expression, Consequence?: BlockStatement, NodeName?: string);
    statementNode(): void;
    TokenLiteral(): string;
    String(): string;
    UnParse(ctx: Transpiler): string;
}
export declare class WhileStatement implements Statement {
    Token: token.Token;
    Condition?: Expression;
    Consequence?: BlockStatement;
    NodeName: string;
    constructor(Token: token.Token, // The 'while' token
    Condition?: Expression, Consequence?: BlockStatement, NodeName?: string);
    statementNode(): void;
    TokenLiteral(): string;
    String(): string;
    UnParse(ctx: Transpiler): string;
}
export declare class SleepStatement implements Statement {
    Token: token.Token;
    Duration?: Expression;
    Consequence?: BlockStatement;
    NodeName: string;
    constructor(Token: token.Token, // The 'while' token
    Duration?: Expression, Consequence?: BlockStatement, NodeName?: string);
    statementNode(): void;
    TokenLiteral(): string;
    String(): string;
    UnParse(ctx: Transpiler): string;
}
/**
 * Here's a concept.
 * A concept is a relationship between classes or other concepts.
 * The following types of relationships can exist:
 * - Composition // Item in ListView
 *  // <class | concept> is encapsulated by <class | concept>
 * - Connection  // RadioWave connects(Transmitter, Antenna) {
 *  //              ..and how
 *  //              }
 *  // <concept> is the connection from <class | concept> to <class | concept>
 * -//
 *
 */
export declare class ConceptStatement implements Statement {
    Token: token.Token;
    ConceptType: string;
    Name?: Identifier;
    Value?: HashLiteral;
    NodeName: string;
    constructor(Token: token.Token, // the token.CONCEPT token
    ConceptType: string, // "control" | "data"
    Name?: Identifier, Value?: HashLiteral, // 
    NodeName?: string);
    statementNode(): void;
    TokenLiteral(): string;
    String(): string;
    UnParse(ctx: Transpiler): string;
}
