"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var AssignmentStatement = /** @class */ (function () {
    function AssignmentStatement(Name, Value, NodeName) {
        if (NodeName === void 0) { NodeName = "AssignmentStatement"; }
        this.Name = Name;
        this.Value = Value;
        this.NodeName = NodeName;
    }
    AssignmentStatement.prototype.statementNode = function () { };
    AssignmentStatement.prototype.TokenLiteral = function () {
        return this.Name.Value;
    };
    AssignmentStatement.prototype.String = function () {
        var out = "";
        out += this.Name.String();
        out += " = ";
        if (this.Value != null) {
            out += this.Value.String();
        }
        out += ";";
        return out;
    };
    AssignmentStatement.prototype.UnParse = function (ctx) {
        return ctx.transpile(this.Name) + " = " + ctx.transpile(this.Value);
    };
    return AssignmentStatement;
}());
exports.AssignmentStatement = AssignmentStatement;
var LetStatement = /** @class */ (function () {
    function LetStatement(Token, // the token.LET token
    Name, Value, Vars, Types, NodeName) {
        if (Types === void 0) { Types = []; }
        if (NodeName === void 0) { NodeName = "LetStatement"; }
        this.Token = Token;
        this.Name = Name;
        this.Value = Value;
        this.Vars = Vars;
        this.Types = Types;
        this.NodeName = NodeName;
        this.dataType = "";
    }
    LetStatement.prototype.statementNode = function () { };
    LetStatement.prototype.TokenLiteral = function () {
        return this.Token.Literal;
    };
    LetStatement.prototype.String = function () {
        var out;
        out += this.TokenLiteral() + " ";
        out += this.Name.String();
        out += " = ";
        if (this.Value != null) {
            out += this.Value.String();
        }
        out += ";";
        return out;
    };
    LetStatement.prototype.UnParse = function (ctx) {
        return ctx.transpile(this);
    };
    return LetStatement;
}());
exports.LetStatement = LetStatement;
var ClassStatement = /** @class */ (function () {
    function ClassStatement(Token, // the token.CLASS token
    Name, Value, NodeName) {
        if (NodeName === void 0) { NodeName = "ClassStatement"; }
        this.Token = Token;
        this.Name = Name;
        this.Value = Value;
        this.NodeName = NodeName;
    }
    ClassStatement.prototype.statementNode = function () { };
    ClassStatement.prototype.TokenLiteral = function () {
        return this.Token.Literal;
    };
    ClassStatement.prototype.String = function () {
        var out;
        out += this.TokenLiteral() + " ";
        out += this.Name.String();
        out += " = "; // is this line wrong?
        if (this.Value != null) {
            out += this.Value.String();
        }
        out += ";";
        return out;
    };
    ClassStatement.prototype.UnParse = function (ctx) {
        return ctx.transpile(this); //
    };
    return ClassStatement;
}());
exports.ClassStatement = ClassStatement;
var ReturnStatement = /** @class */ (function () {
    function ReturnStatement(Token, // the 'return' token
    ReturnValue, NodeName) {
        if (NodeName === void 0) { NodeName = "ReturnStatement"; }
        this.Token = Token;
        this.ReturnValue = ReturnValue;
        this.NodeName = NodeName;
    }
    ReturnStatement.prototype.statementNode = function () { };
    ReturnStatement.prototype.TokenLiteral = function () { return this.Token.Literal; };
    ReturnStatement.prototype.String = function () {
        var out = "";
        out += this.TokenLiteral() + " ";
        if (this.ReturnValue != null) {
            out += this.ReturnValue.String();
        }
        out += ";";
        return out;
    };
    ReturnStatement.prototype.UnParse = function (ctx) {
        return "return " + ctx.transpile(this.ReturnValue);
    };
    return ReturnStatement;
}());
exports.ReturnStatement = ReturnStatement;
var ExpressionStatement = /** @class */ (function () {
    function ExpressionStatement(Token, // the first token of the expression
    Expression, NodeName) {
        if (NodeName === void 0) { NodeName = "ExpressionStatement"; }
        this.Token = Token;
        this.Expression = Expression;
        this.NodeName = NodeName;
    }
    ExpressionStatement.prototype.statementNode = function () { };
    ExpressionStatement.prototype.TokenLiteral = function () { return this.Token.Literal; };
    ExpressionStatement.prototype.String = function () {
        if (this.Expression != null) {
            return this.Expression.String();
        }
        return "";
    };
    ExpressionStatement.prototype.UnParse = function (ctx) {
        return this.Expression && this.Expression.UnParse ? this.Expression.UnParse(ctx) : "";
    };
    return ExpressionStatement;
}());
exports.ExpressionStatement = ExpressionStatement;
var BlockStatement = /** @class */ (function () {
    function BlockStatement(Token, // the { token
    Statements, NodeName) {
        if (Statements === void 0) { Statements = []; }
        if (NodeName === void 0) { NodeName = "BlockStatement"; }
        this.Token = Token;
        this.Statements = Statements;
        this.NodeName = NodeName;
    }
    BlockStatement.prototype.statementNode = function () { };
    BlockStatement.prototype.TokenLiteral = function () { return this.Token.Literal; };
    BlockStatement.prototype.String = function () {
        var out = "";
        this.Statements.forEach(function (s) {
            out += s.String();
        });
        return out;
    };
    BlockStatement.prototype.UnParse = function (ctx) {
        var strStatements, statements = this.Statements;
        for (var bs = 0, l = statements.length; bs < l; bs++) {
            var stmt = statements[bs];
            strStatements += stmt.UnParse(ctx) + "\n";
        }
        return strStatements;
    };
    return BlockStatement;
}());
exports.BlockStatement = BlockStatement;
var IfStatement = /** @class */ (function () {
    function IfStatement(Token, // The 'if' token
    Condition, Consequence, Alternative, NodeName) {
        if (NodeName === void 0) { NodeName = "IfStatement"; }
        this.Token = Token;
        this.Condition = Condition;
        this.Consequence = Consequence;
        this.Alternative = Alternative;
        this.NodeName = NodeName;
    }
    IfStatement.prototype.statementNode = function () { };
    IfStatement.prototype.TokenLiteral = function () { return this.Token.Literal; };
    IfStatement.prototype.String = function () {
        var out = "";
        out += "if(";
        out += this.Condition.String();
        out += ") {";
        out += this.Consequence.String();
        out += "}";
        if (this.Alternative != null) {
            out += "else ";
            out += this.Alternative.String();
        }
        return out;
    };
    IfStatement.prototype.UnParse = function (ctx) {
        return ctx.transpile(this);
    };
    return IfStatement;
}());
exports.IfStatement = IfStatement;
var ForStatement = /** @class */ (function () {
    function ForStatement(Token, // The 'for' token
    Element, Range, Consequence, NodeName) {
        if (NodeName === void 0) { NodeName = "ForStatement"; }
        this.Token = Token;
        this.Element = Element;
        this.Range = Range;
        this.Consequence = Consequence;
        this.NodeName = NodeName;
    }
    ForStatement.prototype.statementNode = function () { };
    ForStatement.prototype.TokenLiteral = function () { return this.Token.Literal; };
    ForStatement.prototype.String = function () {
        var out = "";
        out += "for ( " + this.Element.String() + ", " + this.Range.String() + " ) {";
        out += this.Consequence.String();
        out += "}";
        return out;
    };
    ForStatement.prototype.UnParse = function (ctx) {
        return ctx.transpile(this);
    };
    return ForStatement;
}());
exports.ForStatement = ForStatement;
var WhileStatement = /** @class */ (function () {
    function WhileStatement(Token, // The 'while' token
    Condition, Consequence, NodeName) {
        if (NodeName === void 0) { NodeName = "WhileStatement"; }
        this.Token = Token;
        this.Condition = Condition;
        this.Consequence = Consequence;
        this.NodeName = NodeName;
    }
    WhileStatement.prototype.statementNode = function () { };
    WhileStatement.prototype.TokenLiteral = function () { return this.Token.Literal; };
    WhileStatement.prototype.String = function () {
        var out = "";
        out += "while (" + this.Condition.String() + ") {";
        out += this.Consequence.String();
        out += "}";
        return out;
    };
    WhileStatement.prototype.UnParse = function (ctx) {
        return ctx.transpile(this);
    };
    return WhileStatement;
}());
exports.WhileStatement = WhileStatement;
var SleepStatement = /** @class */ (function () {
    function SleepStatement(Token, // The 'while' token
    Duration, Consequence, NodeName) {
        if (NodeName === void 0) { NodeName = "SleepStatement"; }
        this.Token = Token;
        this.Duration = Duration;
        this.Consequence = Consequence;
        this.NodeName = NodeName;
    }
    SleepStatement.prototype.statementNode = function () { };
    SleepStatement.prototype.TokenLiteral = function () { return this.Token.Literal; };
    SleepStatement.prototype.String = function () {
        var out = "";
        out += "sleep (" + this.Duration.String() + ") {";
        out += this.Consequence.String();
        out += "}";
        return out;
    };
    SleepStatement.prototype.UnParse = function (ctx) {
        return ctx.transpile(this);
    };
    return SleepStatement;
}());
exports.SleepStatement = SleepStatement;
/**
 * Here's a concept.
 * A concept is a relationship between classes or other concepts.
 * The following types of relationships can exist:
 * - Composition // Item in ListView
 *  // <class | concept> is encapsulated by <class | concept>
 * - Connection  // RadioWave connects(Transmitter, Antenna) {
 *  //              ..and how
 *  //              }
 *  // <concept> is the connection from <class | concept> to <class | concept>
 * -//
 *
 */
var ConceptStatement = /** @class */ (function () {
    function ConceptStatement(Token, // the token.CONCEPT token
    ConceptType, // "control" | "data"
    Name, Value, // 
    NodeName) {
        if (NodeName === void 0) { NodeName = "ConceptStatement"; }
        this.Token = Token;
        this.ConceptType = ConceptType;
        this.Name = Name;
        this.Value = Value;
        this.NodeName = NodeName;
    }
    ConceptStatement.prototype.statementNode = function () { };
    ConceptStatement.prototype.TokenLiteral = function () {
        return this.Token.Literal;
    };
    ConceptStatement.prototype.String = function () {
        var out;
        out += this.TokenLiteral() + " ";
        out += this.Name.String();
        out += " = ";
        if (this.Value != null) {
            out += this.Value.String();
        }
        out += ";";
        return out;
    };
    ConceptStatement.prototype.UnParse = function (ctx) {
        return ctx.transpile(this); //
    };
    return ConceptStatement;
}());
exports.ConceptStatement = ConceptStatement;
