"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var object_1 = require("../object/object");
var error_1 = require("../object/error");
var argsError = "wrong number of arguments. got=%d, want=%d  ";
var posErr = "argument %d ";
function assertBuiltinArgs(args, min, max, name, types) {
    var numArgs = args.length;
    if (min && (numArgs < min || !args[min - 1])) {
        return error_1.newError(argsError, numArgs, min);
    }
    if (max && numArgs > max) {
        return error_1.newError(argsError + " to %d arguments", numArgs, min, max);
    }
    if (types) {
        for (var i = 0, l = types.length; i < l; i++) {
            var got = args[i].Type(), want = types[i];
            var fail_1 = false;
            if (typeof want === "string") {
                fail_1 = got != want;
            }
            else {
                fail_1 = want.indexOf(got) === -1;
            }
            if (fail_1) {
                return error_1.newError(posErr + (name ? " to " + name : "") + " wants %s except it got %s", i, want, got);
            }
        }
    }
}
exports.assertBuiltinArgs = assertBuiltinArgs;
function numericArgumentsError(methodName, theArgs, n) {
    if (theArgs.length < n) {
        return error_1.newError("Not enough arguments to " + methodName + ". Needs this many: %d, got %s  ", n, theArgs.length - 2);
    }
    for (var x = 0; x < n; x++) {
        var theArg = theArgs[x];
        if (theArg == null || typeof theArg != "number") {
            return error_1.newError("argument " + x + " of " + methodName + " must be float or int, got %s", theArg ? typeof theArg : "null");
        }
    }
    return false;
}
exports.numericArgumentsError = numericArgumentsError;
exports.makeBuiltinClass = function (className, fields, modifiers) {
    var instance = makeBuiltinInterface(fields, modifiers);
    instance.Constructor = (instance.Pairs[className].Value);
    instance.className = className;
    instance.Pairs.builtin = { Key: object_1.strBuiltin, Value: new object_1.BooleanObject(true) };
    return instance;
};
function makeBuiltinInterface(methods, modifiers) {
    var pairs = {};
    for (var mIdx in methods) {
        var m = methods[mIdx];
        pairs[m[0]] = {
            Key: new object_1.StringObject(m[0]),
            Value: m[1],
            modifiers: (modifiers && modifiers[m[0]]) ? modifiers[m[0]] : null
        };
    }
    return new object_1.Hash(pairs);
}
exports.makeBuiltinInterface = makeBuiltinInterface;
function makeBuiltinStream(dataType, direction, transforms, source, sink) {
    return new object_1.Stream(direction, transforms, source, sink, dataType);
}
exports.makeBuiltinStream = makeBuiltinStream;
exports.makeAPIFromMapOfStringLists = function (map, mapper) {
    return Object.keys(map)
        .map(function (v) { return [v, map[v]]; })
        .filter(function (v) { return v[1].length > 0; })
        .map(function (eventTarget) { return [
        eventTarget[0],
        eventTarget[1].map(function (event) { return mapper(eventTarget[0], event); })
    ]; })
        .map(function (v) {
        var pairs = {}, members = v[1];
        for (var x in members) {
            pairs[members[x][0]] = {
                Key: new object_1.StringObject(members[x][0]),
                Value: members[x][1]
            };
        }
        return [v[0], new object_1.Hash(pairs)];
    });
};
exports.makeBuiltinEnum = function (names) {
    return makeBuiltinInterface(names.map(function (v, k) { return [v, new object_1.Integer(k)]; }));
};
