"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var object = require("../../object/object");
var obj_util_1 = require("../../object/obj-util");
var component_util_1 = require("./component-util");
var actions_1 = require("./actions");
var error_1 = require("../../object/error");
var builtin_util_1 = require("../builtin-util");
// update component telemetry
var setPosition = new object.Builtin("setPosition", [], function (context, _2, jsScope, position) {
    component_util_1.vectorCallError(position, "setPosition", 3);
    jsScope.position = position;
    context.postMessage("native-call", { method: "component.setPosition", args: [position] });
}, "component");
var setRotation = new object.Builtin("setRotation", [], function (context, _2, _3, rotation) {
    component_util_1.vectorCallError(rotation, "setRotation", 3);
    context.postMessage("native-call", { method: "component.setRotation", args: [rotation] });
}, "component");
var setQuaternion = new object.Builtin("setQuaternion", [], function (context, _2, jsScope, quaternion) {
    component_util_1.vectorCallError(quaternion, "setQuaternion", 4);
    jsScope.quaternion = quaternion;
    context.postMessage("native-call", { method: "component.setQuaternion", args: [quaternion] });
}, "component");
// set data in the component
var setAttrs = new object.Builtin("setAttrs", [], function (context, _2, jsScope, attributes) {
    context.postMessage("native-call", { method: "component.setAttrs", args: [attributes] });
}, "component");
var setProps = new object.Builtin("setProps", [], function (context, _2, jsScope, props) {
    context.postMessage("native-call", { method: "component.setProps", args: [props] });
}, "component");
var setState = new object.Builtin("setState", [], function (context, scope, jsScope, state) {
    context.postMessage("native-call", { method: "component.setState", args: [state] });
}, "component");
var callMethod = new object.Builtin("callMethod", [], function (context, _2, _3, systemName, methodName, argumentList) {
    if (!systemName || !methodName || !argumentList) {
        return error_1.newError("wrong number of arguments to component.callMethod. got=%d, want=3", (!!systemName ? 1 : 0) + (!!methodName ? 1 : 0) + (!!argumentList ? 1 : 0));
    }
    if (typeof systemName != "string") {
        return error_1.newError("first argument to `Component.callMethod` must be string, got %s", typeof systemName);
    }
    if (typeof methodName != "string") {
        return error_1.newError("second argument to `Component.callMethod` must be string, got %s", typeof methodName);
    }
    if (!argumentList || isNaN(argumentList.length)) {
        return error_1.newError("third argument to `Component.callMethod` must be array, got %s", typeof argumentList);
    }
    context.postMessage("native-call", { method: "component.callMethod", args: [systemName, methodName, argumentList] });
}, "component");
// methods for child components
var addComponent = new object.Builtin("addComponent", [], function (context, scope, jsScope, component) {
    context.postMessage("native-call", { method: "component.addComponent", args: [component] });
});
var removeComponent = new object.Builtin("removeComponent", [], function (context, scope, jsScope, componentIndex) {
    context.postMessage("native-call", { method: "component.removeComponent", args: [componentIndex] });
}, "component");
var updateComponent = new object.Builtin("updateComponent", [], function (context, scope, jsScope) {
    var args = [];
    for (var _i = 3; _i < arguments.length; _i++) {
        args[_i - 3] = arguments[_i];
    }
    context.postMessage("native-call", { method: "component.updateComponent", args: [args[0]] });
}, "component");
var getComponent = function (components, args) {
    if (typeof args[0] == "number") {
        return components[args[0]];
    }
    else if (typeof args[0] == "string") {
        return component_util_1.getComponentByName(components, args[0]);
    }
    else if (!!args[0] && args[0].length) {
        // not implemented
    }
    else {
        return error_1.newError("first argument to `Component.getComponent` must be int, Sring or List, got %s", typeof args[0]);
    }
};
function ComponentState(dto) {
    this.id = dto.id;
    this.className = dto.class;
    this.name = dto.name;
    this.position = dto.position;
    this.quaternion = dto.quaternion;
    this.attrs = dto.attrs;
    this.props = dto.props;
    this.state = dto.state;
}
var componentConstructor = new object.Builtin("Component", [], function (context, scope, jsScope) {
    var args = [];
    for (var _i = 3; _i < arguments.length; _i++) {
        args[_i - 3] = arguments[_i];
    }
    var components = (context && context.data) ? context.data.components : [];
    if (args.length == 0) { // return current component
        Object.assign(scope.builtins, (context && context.data.component) ? context.data.component : {});
    }
    else if (args.length == 1) {
        var comp = void 0;
        if (typeof args[0] == "string" || typeof args[0] == "number") {
            comp = getComponent(components, args);
            if (comp) {
                scope.builtins = new ComponentState(comp || {});
            }
            else {
                return error_1.newError("Component not found: " + args[0]);
            }
        }
        else {
            scope.builtins = new ComponentState({ attrs: args[0] || {} });
        }
    }
    else {
        return error_1.newError("wrong number of arguments. got=%d, want=0 or 1", args.length);
    }
}, "component");
exports.Component = builtin_util_1.makeBuiltinClass("Component", [
    ["Component", componentConstructor],
    ["events", new object.Builtin("events", [], function () {
            return builtin_util_1.makeBuiltinInterface(builtin_util_1.makeAPIFromMapOfStringLists(actions_1.events, obj_util_1.makeEventSource));
        })],
    ["actions", new object.Builtin("actions", [], function () {
            return builtin_util_1.makeBuiltinInterface(builtin_util_1.makeAPIFromMapOfStringLists(actions_1.events, obj_util_1.makeEventSink));
        })],
    ["setPosition", setPosition],
    ["setRotation", setRotation],
    ["setQuaternion", setQuaternion],
    ["addComponent", addComponent],
    ["removeComponent", removeComponent],
    ["updateComponent", updateComponent],
    ["setAttrs", setAttrs],
    ["setProps", setProps],
    ["setState", setState],
    ["callMethod", callMethod],
], {
    "events": [1],
    "actions": [1]
});
