"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var object = require("../../object/object");
var object_1 = require("../../object/object");
var component_util_1 = require("./component-util");
var error_1 = require("../../object/error");
var builtin_util_1 = require("../builtin-util");
// update component telemetry
var setPosition = new object.Builtin("setPosition", [object.ARRAY_OBJ], function (context, _2, jsScope, position) {
    component_util_1.vectorCallError(position, "Entity.setPosition", 3);
    jsScope.position = position;
    context.postMessage("native-call", { method: "entity.setPosition", args: [position] });
    return object_1.NULL;
}, "entity");
var setRotation = new object.Builtin("setRotation", [object.ARRAY_OBJ], function (context, _2, _3, rotation) {
    component_util_1.vectorCallError(rotation, "Entity.setRotation", 3);
    context.postMessage("native-call", { method: "entity.setRotation", args: [rotation] });
    return object_1.NULL;
}, "entity");
var setQuaternion = new object.Builtin("setQuaternion", [object.ARRAY_OBJ], function (context, _2, jsScope, quaternion) {
    component_util_1.vectorCallError(quaternion, "Entity.setQuaternion", 4);
    jsScope.quaternion = quaternion;
    context.postMessage("native-call", { method: "entity.setQuaternion", args: [quaternion] });
    return object_1.NULL;
}, "entity");
// methods for child components
var addComponent = new object.Builtin("addComponent", [object.ARRAY_OBJ], function (context, _2, _3) {
    var args = [];
    for (var _i = 3; _i < arguments.length; _i++) {
        args[_i - 3] = arguments[_i];
    }
    context.postMessage("native-call", { method: "entity.addComponent", args: [args[0]] });
    return object_1.NULL;
});
var removeComponent = new object.Builtin("removeComponent", [], function (context, _2, _3) {
    var args = [];
    for (var _i = 3; _i < arguments.length; _i++) {
        args[_i - 3] = arguments[_i];
    }
    context.postMessage("native-call", { method: "entity.setAttrs", args: [args[0]] });
    return object_1.NULL;
}, "entity");
var updateComponent = new object.Builtin("updateComponent", [], function (context, _2, _3) {
    var args = [];
    for (var _i = 3; _i < arguments.length; _i++) {
        args[_i - 3] = arguments[_i];
    }
    context.postMessage("native-call", { method: "entity.updateComponent", args: [args[0]] });
    return object_1.NULL;
}, "entity");
function EntityState(dto) {
    this.id = dto.id || -1;
    this.name = dto.name;
    this.position = dto.position;
    this.quaternion = dto.quaternion;
    this.boundingRadius = dto.boundingRadius;
    this.tags = dto.tags || [];
    this.components = dto.components || [];
}
var entityConstructor = new object.Builtin("Entity", [], function (context, scope, jsScope, dto) {
    var entities = (context && context.data) ? context.data.entities : [];
    if (!dto) { // return current component
        scope.builtins = Object.assign(scope.builtins || {}, new EntityState((context && context.data.entity) ? context.data.entity : {}));
    }
    else if (dto) {
        if (typeof dto == "object") {
            scope.builtins = Object.assign(scope.builtins || {}, new EntityState(dto));
        }
        else {
            return error_1.newError("Not implemented: Entity( " + typeof dto + " )");
        }
    }
}, "entity");
exports.Entity = builtin_util_1.makeBuiltinClass("Entity", [
    ["Entity", entityConstructor],
    ["setPosition", setPosition],
    ["setRotation", setRotation],
    ["setQuaternion", setQuaternion],
    ["addComponent", addComponent],
    ["removeComponent", removeComponent],
    ["updateComponent", updateComponent],
]);
