"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var token_1 = require("../token");
var docs = new Map();
docs.set("keywords", [
    "Keywords: |                                                                                                ",
    "fn        | fn(?param1, ?param2...) { ?<statement>; ?<statement>... ?return <expression>; }                ",
    "let       | let identifierName = <expression>;                                                             ",
    "class     | class ClassName { \"ClassName\": fn(){ }, <HashPair>... }                                        ",
    "new       | new ClassName(?param1, ?param2...)                                                             ",
    "if        | if ( <boolean expression> ) { <statement>; <statement>... } else { <statement>; <statement>... } ",
    "else      | if (...) { ... } else { <statement>; <statement>... }                                          ",
    "for       | for ( <identifier>, <number | string | array | hashmap> ) { }                                    ",
    "while     | while (<boolean expression>) { <statement>; <statement>... }                                   ",
    "sleep     | sleep ( <integer expression> ) { <statement>; <statement>... }                                   ",
    "return    | return <expression>;                                                                           ",
    "this      | this                                                                                           ",
    "true      | true                                                                                           ",
    "false     | false                                                                                          ",
    "exec      | exec \"other-script.ecs\"                                                                        "
]);
docs.set("delimiters", [
    "Delimiters: |",
    token_1.COMMA + "           | Comma",
    token_1.SEMICOLON + "           | Semicolon",
    token_1.COLON + "           | Colon",
    token_1.LPAREN + "           | Left Parenthesis",
    token_1.RPAREN + "           | Right Parenthesis",
    token_1.LBRACE + "           | Left Brace",
    token_1.RBRACE + "           | Right Brace",
    token_1.LBRACKET + "           | Left Bracket",
    token_1.RBRACKET + "           | Right Bracket"
]);
docs.set("operators", [
    "Operators: |",
    token_1.ASSIGN + "          | Assign",
    token_1.PLUS + "          | Plus / Addition",
    token_1.BANG + "          | Bang / Not",
    token_1.MINUS + "          | Minus",
    token_1.SLASH + "          | Slash / Divide",
    token_1.ASTERISK + "          | Asterisk / Multiply",
    token_1.MOD + "          | Mod / Remainder",
    token_1.LT + "          | Less Than",
    token_1.GT + "          | Greater Than",
    token_1.EQ + "         | Is Equal To",
    token_1.NOT_EQ + "         | Is Not Equal",
    "typeof" + "     | Name Of Type",
    token_1.AND + "         | And",
    token_1.OR + "         | Or",
    token_1.SOURCE + "         | Source (coming soon)",
    token_1.SINK + "         | Sink (coming soon)",
    token_1.INSERTION + "         | Insert (coming soon)",
    token_1.EXTRACTION + "         | Extract (coming soon)"
]);
docs.set("builtins", [
    "╠═════════════════════════════════════════════════════════════════════",
    "║ Builtin Classes:  ",
    "╠═════════════════════════════════════════════════════════════════════",
    "",
    "║ Component                 │ // A piece of an entity in the world.",
    "║ Make one:                 │ new Component(initialAttributes: object) ",
    "║ Get the current component:│ let component = Component();",
    "╠═══════════════════════════╩",
    "║ Component.events     │ Component.events(): {",
    "║                      │                       [attributeName]: {",
    "║                      │                         [observable]: fn(stream | fn(event){}){}",
    "║                      │                       }",
    "║                      │                     }",
    "╠══════════════════════╩══════════════════════════════════════════════════",
    "║ Component.actions    │ Component.actions(): {",
    "║                      │                        [attributeName]: {",
    "║                      │                          [eventSink]: fn(stream | literal){}",
    "║                      │                        }",
    "║                      │                      }",
    "╠══════════════════════╩══════════════════════════════════════════════════",
    "║ Component.attrs      │ object",
    "╠══════════════════════╩══════════════════════════════════════════════════",
    "║ Component.components │ Component[]",
    "╠══════════════════════╩══════════════════════════════════════════════════",
    "║ Component.position   | float[3] // offset in 3d space, from parent entity",
    "╠══════════════════════╩══════════════════════════════════════════════════",
    "║ Component.quaternion | float[3]",
    "╠══════════════════════╩══════════════════════════════════════════════════",
    "║ Component.class      | string // ",
    "╠══════════════════════╩══════════════════════════════════════════════════",
    "║ Component.tags       | string[]",
    "╠══════════════════════╩══════════════════════════════════════════════════",
    "",
    "║ Entity                  │ // A group of components and their children.",
    "║ Make one:               │ new Entity({ components: [{}, ...], ...})",
    "║ Get the current entity: | Entity();",
    "║ Get entity by name:     | let theEntity = Entity('withSomeName');",
    "║ Get an entity by path:  | let entity = Entity([0, 0, 1, 22]);",
    "╠═════════════════════════╩",
    "║ Entity.components  │ Component[]",
    "╠════════════════════╩══════════════════════════════════════════════════",
    "║ Entity.position    │ float[3]",
    "╠════════════════════╩══════════════════════════════════════════════════",
    "║ Entity.quaternion  │ float[4]",
    "╠════════════════════╩══════════════════════════════════════════════════",
    "",
    "╠═══════════════════════════════════════════════════════════════════════",
    "║ RegExp           │",
    "╠══════════════════╩",
    "║ RegExp.test      │",
    "║                  │",
    "╠══════════════════╩══════════════════════════════════════════════════",
    "",
    "╠═════════════════════════════════════════════════════════════════════",
    "║ JSON             │",
    "╠══════════════════╩",
    "║ JSON.parse       │ JSON.parse( string ): object | array",
    "║                  │   returns de-serialized object or array",
    "╠══════════════════╩══════════════════════════════════════════════════",
    "║ JSON.stringify   │ JSON.stringify( object | array ): string",
    "║                  │   returns JSON-serialized representation of object ",
    "╠══════════════════╩══════════════════════════════════════════════════",
    "",
    "╠══════════════════╩══════════════════════════════════════════════════",
    "║ Math             │ The static class Math is much like",
    "║                  │ the Math object from javascript.",
    "║                  │ It has the following methods:",
    "╠══════════════════╩",
    "╠══════════════════╩══════════════════════════════════════════════════",
    "║ Math.floor       │ Math.floor( float ): int",
    "║                  │   returns number rounded down",
    "╠══════════════════╩══════════════════════════════════════════════════",
    "║ Math.ceil        │ Math.ceil( float ): int",
    "║                  │   returns number rounded up",
    "╠══════════════════╩══════════════════════════════════════════════════",
    "║ Math.fract       │ Math.fract( float ): float",
    "║                  │   returns fractional portion of number",
    "╠══════════════════╩══════════════════════════════════════════════════",
    "║ Math.abs         │ Math.abs( float | int ): float | int",
    "║                  │   returns absolute value of number",
    "╠══════════════════╩══════════════════════════════════════════════════",
    "║ Math.sqrt        │ Math.sqrt( float | int ): float | int",
    "║                  │   returns square root of number",
    "╠══════════════════╩══════════════════════════════════════════════════",
    "║ Math.PI          │ Math.PI(): float",
    "║                  │   returns constant pi",
    "╠══════════════════╩══════════════════════════════════════════════════",
    "║ Math.sin         │ Math.sin( float ): float",
    "║                  │   returns sine of number in radians",
    "╠══════════════════╩══════════════════════════════════════════════════",
    "║ Math.cos         │ Math.cos( float ): float",
    "║                  │   returns cosine of number in radians",
    "╠══════════════════╩══════════════════════════════════════════════════",
    "║ Math.tan         │ Math.tan( float ): float",
    "║                  │   returns tangent of number in radians",
    "╠══════════════════╩══════════════════════════════════════════════════",
    "║ Math.atan2       │ Math.atan2( float, float ): float",
    "║                  │   returns atan2 of number in radians",
    "╠══════════════════╩══════════════════════════════════════════════════",
    "",
    "╠═════════════════════════════════════════════════════════════════════",
    "║                  |",
    "║ Matrix           │",
    "╠══════════════════╩",
    "║ Matrix.multiply  │ multiply(otherColumns: int, otherData: float[]): float[]",
    "║                  │   returns result of multiplication without mutating state",
    "╠══════════════════╩══════════════════════════════════════════════════",
    "║ Matrix.get2DArray│ get2DArray(): float[][]",
    "║                  │   returns list of lists, according to number of columns ",
    "╠══════════════════╩══════════════════════════════════════════════════",
    "",
    "║",
    "║ Builtin Functions:   ",
    "╠═════════════════════════════════════════════════════════════════════",
    "║ print            │ print( <string> value ): null",
    "║                  │   prints string to console",
    "╠══════════════════╩══════════════════════════════════════════════════",
    "║ len              │ len( array | string ): int",
    "║                  │   return length of array or string",
    "╠══════════════════╩══════════════════════════════════════════════════",
    "║ push             │ push( array, any ): array",
    "║                  │   adds item to end of array",
    "╠══════════════════╩══════════════════════════════════════════════════",
    "║ pop              │ pop( array, any ): array",
    "║                  │   remove and return item from end of array",
    "╠══════════════════╩══════════════════════════════════════════════════",
    "║ shift            │ shift( array, any ): any",
    "║                  │   remove and return first item in an array",
    "╠══════════════════╩══════════════════════════════════════════════════",
    "║ splice           │ splice( array, int, int ): any",
    "║                  │   remove and return the range of elements from the",
    "║                  │   starting index specified by the second argument",
    "║                  │   to that index plus the third argument.",
    "╠══════════════════╩══════════════════════════════════════════════════",
    "║ join             │ join( string[] | number[], string ): string",
    "║                  │   concatenate an array of strings or numbers into one string.",
    "║                  │   second argument is the string to splice in between",
    "╠══════════════════╩══════════════════════════════════════════════════",
    "║ join (object)    │ join( hash | object, string, string ): string",
    "║                  │   strings the key value pairs in the first argument",
    "║                  │   The second argument separates pairs",
    "║                  │   The third argument separates each key and value.",
    "╠══════════════════╩══════════════════════════════════════════════════",
    "║ split            │ split( string, string ): string[]",
    "║                  │   return an array of pieces of the first argument,",
    "║                  │   separated at every instance of the second argument",
    "╠══════════════════╩══════════════════════════════════════════════════",
    "║ split (object)   │ split( hash | object, string, string ): | hash",
    "║                  │   return a hashmap of key value pairs from a string,",
    "║                  │   Second argument splits each pair out of the string.",
    "║                  │   Third argument splits each key and value.",
    "╠══════════════════╩══════════════════════════════════════════════════",
    "║ toFloat          │ toFloat( int ): float",
    "║                  │   returns the floating point version of the int argument",
    "╠══════════════════╩══════════════════════════════════════════════════",
    "║ time             │ time(): int",
    "║                  │   returns current time in milliseconds",
    "╠══════════════════╩══════════════════════════════════════════════════",
]);
docs.set("print", "builtins");
docs.set("push", "builtins");
docs.set("shift", "builtins");
docs.set("pop", "builtins");
docs.set("len", "builtins");
docs.set("join", "builtins");
docs.set("split", "builtins");
docs.set("float", "builtins");
docs.set("Math.floor", "builtins");
docs.set("Math.ceil", "builtins");
docs.set("Math.fract", "builtins");
docs.set("Math.abs", "builtins");
docs.set("Math.sqrt", "builtins");
docs.set("Math.PI", "builtins");
docs.set("Math.sin", "builtins");
docs.set("Math.cos", "builtins");
docs.set("Math.tan", "builtins");
docs.set("Math.atan2", "builtins");
docs.set("Math.time", "builtins");
docs.set("graphics", [
    "graphics",
    "{",
    "   makeCanvas: builtin(dimensions: number[], format?: string, data?: string[]|number[][]|number[][][])",
    "   draw: builtin",
    "   fill: builtin",
    "   dot: builtin",
    "   line: builtin",
    "   circle: builtin",
    "   procedure: builtin",
    "}"
]);
docs.set("io", [
    "io",
    "{",
    "   readLine: builtin(): string /* reads line from stdin or modal dialog in browser */",
    "   File: builtin:(): File",
    "}"
]);
docs.set("JSON", [
    "JSON",
    "{",
    "   parse: builtin(data: string): hashmap",
    "   stringify: builtin(information: hashmap): string",
    "}"
]);
docs.set("io.File", [
    "class File(filename: string)",
    "{",
    "   writeLine: builtin(data: string): null",
    "   readAll: builtin(callback: fn(data: string){ }: null): null",
    "}"
]);
docs.set("terminal", [
    "Terminal",
    "{",
    "   getDimensions: builtin(): number[]",
    "   hasColorSupport: builtin(): boolean",
    "   has3dSupport: builtin(): boolean",
    "   beep: builtin(): void",
    "}"
]);
var getHelp = function (docName) {
    var helpResults = docs.get(docName);
    if (typeof helpResults == "string") {
        helpResults = docs.get(helpResults);
    }
    return helpResults ? helpResults : [];
};
exports.getHelp = getHelp;
var listAllDocs = function () {
    var keys = [], key = "";
    var foundDocs = docs.keys();
    while (key = foundDocs.next().value) {
        if (typeof docs.get(key) != "string") {
            keys.push(key);
        }
    }
    return keys;
};
exports.listAllDocs = listAllDocs;
