"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var object = require("../object/object");
var object_1 = require("../object/object");
var help_1 = require("./help");
var help_2 = require("./help");
var io_1 = require("./io");
var maths_1 = require("./maths");
var json_1 = require("./io/json");
var regex_1 = require("./regex");
var graphics_1 = require("./graphics");
var terminal_1 = require("./terminal");
var component_1 = require("./convolvr/component");
var entity_1 = require("./convolvr/entity");
var util_1 = require("../util");
var obj_util_1 = require("../object/obj-util");
var error_1 = require("../object/error");
var matrix_1 = require("./matrix");
var vector_1 = require("./vector");
var builtin_util_1 = require("./builtin-util");
var color_1 = require("../ubiquitous/color");
exports.builtins = {
    "gl": graphics_1.Graphics,
    "io": io_1.io,
    "terminal": terminal_1.terminal,
    "Component": component_1.Component,
    "Entity": entity_1.Entity,
    "Image": graphics_1.ImageObject,
    "Math": maths_1.ECS_MATHS,
    "Matrix": matrix_1.Matrix,
    "Vector": vector_1.Vector,
    "JSON": json_1.ECS_JSON,
    "RegExp": regex_1.ECS_RegExp,
    "toFloat": new object.Builtin("toFloat", [], function (_, _2, _3, val) {
        if (!val || val.Type() != object.INTEGER_OBJ) {
            return error_1.newError("argument to `toFloat` must be int, got %s", val ? val.Type() : "null");
        }
        return new object.Float((val.Value));
    }, null, null, null, true),
    "time": new object.Builtin("time", [], function () {
        return new object.Integer(Date.now());
    }, null, null, null, true),
    "print": new object.Builtin("print", [], function (context, scope, _3) {
        var args = [];
        for (var _i = 3; _i < arguments.length; _i++) {
            args[_i - 3] = arguments[_i];
        }
        util_1.println(context, scope, null, args);
        return object_1.NULL;
    }, null, null, null, true),
    "colorize": new object.Builtin("colorize", [object_1.ARRAY_OBJ, object_1.STRING_OBJ], function (_, _2, _3, color, str) {
        return color_1.systemColorRenderer(color, str);
    }),
    "len": new object.Builtin("len", [], function (_, _2, _3, obj) {
        var arg = obj, argType = arg.Type();
        switch (argType) {
            case object_1.ARRAY_OBJ:
                return new object.Integer(arg.Elements.length);
            case object_1.STRING_OBJ:
                return new object.Integer(arg.Value.length);
            default:
                return error_1.newError("argument to `len` not supported, got %s", obj.Type());
        }
    }, null, null, null, true),
    "push": new object.Builtin("push", [], function (_, _2, _3, list, item) {
        var err = builtin_util_1.assertBuiltinArgs([list, item], 2, null, 'push', [object.ARRAY_OBJ]);
        if (err) {
            return err;
        }
        list.Elements.push(item);
        return list;
    }, null, null, null, true),
    "pop": new object.Builtin("pop", [], function (_, _2, _3, list) {
        var err = builtin_util_1.assertBuiltinArgs([list], 1, null, 'pop', [object_1.ARRAY_OBJ]);
        if (err) {
            return err;
        }
        return list.Elements.pop();
    }, null, null, null, true),
    "shift": new object.Builtin("shift", [], function (_, _2, _3, list) {
        var err = builtin_util_1.assertBuiltinArgs([list], 1, null, 'shift', [object_1.ARRAY_OBJ]);
        if (err) {
            return err;
        }
        return list.Elements.shift();
    }, null, null, null, true),
    "splice": new object.Builtin("splice", [], function (_, _2, _3) {
        var args = [];
        for (var _i = 3; _i < arguments.length; _i++) {
            args[_i - 3] = arguments[_i];
        }
        var argErr = builtin_util_1.assertBuiltinArgs(args, 3);
        if (argErr) {
            return argErr;
        }
        if (args[0].Type() != object.ARRAY_OBJ || args[1].Type() != object.INTEGER_OBJ || args[2].Type() != object.INTEGER_OBJ) {
            return error_1.newError("arguments to `splice` must be ARRAY, INTEGER, INTEGER got %s %s", args[0].Type(), args[1].Type());
        }
        var elems = args[0].Elements.splice(args[1].Inspect(), args[2].Inspect());
        return new object_1.ArrayObject(elems);
    }, null, null, null, true),
    "concat": new object.Builtin("concat", [object_1.ARRAY_OBJ, object_1.ARRAY_OBJ], function (_, scope, jsScope, listOne, listTwo) {
        if (scope === void 0) { scope = null; }
        if (jsScope === void 0) { jsScope = null; }
        var err = builtin_util_1.assertBuiltinArgs([listOne, listTwo], 1, null, 'concat', [object_1.ARRAY_OBJ, object_1.ARRAY_OBJ]);
        if (err) {
            return err;
        }
        return new object_1.ArrayObject(listOne.Elements.concat(obj_util_1.copyListElements(listTwo)));
    }, null, null, null, true),
    "join": new object.Builtin("join", [], function (_, _2, _3) {
        var args = [];
        for (var _i = 3; _i < arguments.length; _i++) {
            args[_i - 3] = arguments[_i];
        }
        var argErr = builtin_util_1.assertBuiltinArgs(args, 2, 3);
        if (argErr) {
            return argErr;
        }
        var arg0Type = args[0].Type(), arg1Type = args[1].Type();
        if (arg1Type != object_1.STRING_OBJ) {
            return error_1.newError("second argument to `join` must be STRING, got %s", args[0].Type());
        }
        var outStr = "";
        var strArray = [];
        if (arg0Type == object.HASH_OBJ) {
            if (arg1Type != object_1.STRING_OBJ || args.length < 3 || args[2].Type() != object_1.STRING_OBJ) {
                return error_1.newError("first argument to `join(hashmap, string, string)` must be Hash, got join(%s, %s, %s) ", args[0].Type(), args[1] ? args[1].Type() : "null", args[2] ? args[2].Type() : "null");
            }
            var struct = args[0], pairs = struct.Pairs;
            for (var key in pairs) {
                strArray.push(key + args[2].Inspect() + pairs[key].Value.Inspect());
            }
        }
        else {
            if (arg0Type != object_1.ARRAY_OBJ) {
                return error_1.newError("first argument to `join(array, string)` must be ARRAY, got %s", args[0].Type());
            }
            var arr = args[0];
            for (var si = 0, l = arr.Elements.length; si < l; si++) {
                var element = arr.Elements[si];
                var s = "";
                s = element.Inspect();
                strArray.push(s);
            }
        }
        outStr = strArray.join(args[1].Value);
        return new object_1.StringObject(outStr);
    }, null, null, null, true),
    "split": new object.Builtin("split", [], function (_, _2, _3) {
        var args = [];
        for (var _i = 3; _i < arguments.length; _i++) {
            args[_i - 3] = arguments[_i];
        }
        var argErr = builtin_util_1.assertBuiltinArgs(args, 2, 3);
        if (argErr) {
            return argErr;
        }
        if (args[0].Type() != object_1.STRING_OBJ) {
            return error_1.newError("first argument to `split` must be STRING or HASH, got %s", args[0].Type());
        }
        if (args[1].Type() != object_1.STRING_OBJ) {
            return error_1.newError("second argument to `split` must be STRING, got %s", args[0].Type());
        }
        if (args.length === 3) {
            if (args[2].Type() != object_1.STRING_OBJ) {
                return error_1.newError("third argument to `split` must be STRING, got %s", args[0].Type());
            }
            var pairs = {}, kvs = args[2].Inspect() + "", ps = args[1].Inspect() + "", parsedPairs = (args[0].Inspect() + "").split(ps);
            for (var p in parsedPairs) {
                var keyValue = parsedPairs[p].split(kvs);
                pairs[keyValue[0]] = keyValue[1];
            }
            return obj_util_1.nativeObjToMap(pairs);
        }
        else {
            var str = args[0].Value + "", elems = str.split(args[1].Value), ecsElems = [];
            for (var i = 0, l = elems.length; i < l; i++) {
                var element = elems[i];
                ecsElems.push(new object_1.StringObject(element));
            }
            return new object_1.ArrayObject(ecsElems);
        }
    }, null, null, null, true),
    "help": new object.Builtin("help", [], function (_, _2, _3) {
        var args = [];
        for (var _i = 3; _i < arguments.length; _i++) {
            args[_i - 3] = arguments[_i];
        }
        var helpElems = [], helpStrs = help_1.listAllDocs();
        if (args.length > 1) {
            return error_1.newError("wrong number of arguments. got=%d, want=0 or 1", args.length);
        }
        if (args.length === 1) {
            man(args);
            return new object_1.StringObject("");
        }
        else {
            util_1.printNativeString(null, null, "");
            util_1.printNativeString(null, null, "╔═══════════════════════════════════════════════╗");
            util_1.printNativeString(null, null, "║ Help Topics             💡 help(\"topicName\"); ║");
            util_1.printNativeString(null, null, "╚═══════════════════════════════════════════════╝");
            helpStrs.forEach(function (helpStr) {
                helpElems.push(new object_1.StringObject(helpStr));
            });
            man([new object_1.StringObject("builtins")]);
            return new object_1.ArrayObject(helpElems);
        }
    }, null, null, null, true)
};
function man(args) {
    if (args[0].Type() != object.STRING_OBJ) {
        return error_1.newError("argument to `man` must be STRING, got %s", args[0].Type());
    }
    var helpKey = args[0].Value;
    var helpElems = [], helpStrs = help_2.getHelp(helpKey);
    var leftBorder = helpKey != "builtins";
    util_1.printNativeString(null, null, "");
    util_1.printNativeString(null, null, "╔════════════════════════════════════════════════════════════════════════════════════════════════════════════╗");
    helpStrs.forEach(function (helpStr) {
        util_1.printNativeString(null, null, (leftBorder ? "║ " : "") + helpStr);
    });
    util_1.printNativeString(null, null, "╚════════════════════════════════════════════════════════════════════════════════════════════════════════════╝");
}
