"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var util_1 = require("../../util");
var util_2 = require("../../util");
var object_1 = require("../../object/object");
var object = require("../../object/object");
var node_objects_1 = require("../../repl/node-objects");
var error_1 = require("../../object/error");
exports.readWholeFile = function (path, fs, relativeToScript, context) {
    if (relativeToScript === void 0) { relativeToScript = true; }
    return util_1.platformSpecificCall(context, null, function () { return readWholeFileNode(path, fs); }, function () { return readFileFromLocalStorage(path); }, function () { return util_2.workerNativeCall(context, "readFile", [path], false); }, [path]);
};
function readWholeFileNode(path, fs) {
    if (fs) {
        return new Promise(function (resolve, reject) {
            if (fs.readFile) {
                fs.readFile(path, "utf8", function (err, data) {
                    if (err)
                        reject();
                    resolve(data);
                });
            }
            else {
                resolve("not implemented");
            }
        });
    }
}
function writeLineToFile(src, data, fs) {
    fs.appendFile(src, data, function (err) {
        if (err)
            throw err;
    });
}
exports.writeLineToFile = writeLineToFile;
function readFileFromLocalStorage(filename) {
    return util_1.getWindow().localStorage.getItem(filename);
}
exports.readFileFromLocalStorage = readFileFromLocalStorage;
function writeToLocalStorage(filename, data) {
    return util_1.getWindow().localStorage.setItem(filename, data);
}
exports.writeToLocalStorage = writeToLocalStorage;
function writeLineToLocalStorage(filename, data) {
    var storage = util_1.getWindow().localStorage;
    return storage.setItem(filename, storage.getItem(filename) + "\n" + data);
}
exports.writeLineToLocalStorage = writeLineToLocalStorage;
function FileState(resource) {
    this.resource = resource;
}
exports.fileConstructor = new object.Builtin("File", [], function (context, scope, jsScope, name) {
    var file = new FileState(name);
    if (scope) {
        scope.builtins = file;
        return scope;
    }
    return file;
}, "object"), exports.fileOpen = new object.Builtin("open", [], function (context, scope, jsScope, resource) {
    jsScope.resource = resource;
    return object_1.FALSE;
}, "file"), exports.fileReadAll = new object.Builtin("readAll", [], function (context, scope, jsScope, callback) {
    if (!callback) {
        return error_1.newError("argument to readAll must be callback function");
    }
    return util_1.platformSpecificCall(context, scope, function () {
        exports.readWholeFile(jsScope.resource + "", node_objects_1.nodeObjects.fs).then(function (data) {
            callback(data);
        });
        return object.NULL;
    }, function () {
        ;
        callback(readFileFromLocalStorage(jsScope.resource) || "");
        return object.NULL;
    }, function () { return util_2.workerNativeCall(context, "readAll", [jsScope.resource], false); }, []);
}, "object"), exports.fileWriteLine = new object.Builtin("writeLine", [], function (context, scope, jsScope) {
    var args = [];
    for (var _i = 3; _i < arguments.length; _i++) {
        args[_i - 3] = arguments[_i];
    }
    if (args.length != 1 || args[0].Type() != object_1.STRING_OBJ) {
        return error_1.newError("argument to writeLine must be a string");
    }
    return util_1.platformSpecificCall(context, scope, function () {
        var filename = scope.Pairs.resource.Value.Inspect();
        writeLineToFile(filename + "", args[0].Inspect() + "\n", node_objects_1.nodeObjects.fs);
        return object.NULL;
    }, function () {
        var filename = scope.Pairs.resource.Value.Inspect();
        writeLineToLocalStorage(filename + "", args[0].Inspect() + "\n");
        return object.NULL;
    }, function () { return util_2.workerNativeCall(context, "writeLine", args, false); }, args);
}, "file"), exports.fileReadLine = new object.Builtin("readLine", [], function (context, scope, jsScope) {
    var args = [];
    for (var _i = 3; _i < arguments.length; _i++) {
        args[_i - 3] = arguments[_i];
    }
    return new object_1.StringObject("not implemented");
}, "file"), exports.fileHasNextLine = new object.Builtin("hasNextLine", [], function (context, scope, jsScope) {
    var args = [];
    for (var _i = 3; _i < arguments.length; _i++) {
        args[_i - 3] = arguments[_i];
    }
    return new object_1.StringObject("not implemented");
}, "object");
