"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var object_1 = require("../object/object");
var builtin_util_1 = require("./builtin-util");
exports.ECS_MATHS = builtin_util_1.makeBuiltinInterface([
    [
        "PI",
        new object_1.Builtin("PI", [], function (context, scope, jsScope) {
            return Math.PI;
        }, null, null, null, true)
    ], [
        "sin",
        new object_1.Builtin("sin", [object_1.FLOAT_OBJ], function (context, scope, jsScope, angle) {
            return new object_1.Float(Math.sin(angle.Value));
        }, null, null, null, true)
    ], [
        "cos",
        new object_1.Builtin("cos", [object_1.FLOAT_OBJ], function (context, scope, jsScope, angle) {
            return new object_1.Float(Math.cos(angle.Value));
        }, null, null, null, true)
    ], [
        "tan",
        new object_1.Builtin("tan", [object_1.FLOAT_OBJ], function (context, scope, jsScope, angle) {
            return new object_1.Float(Math.tan(angle.Value));
        }, null, null, null, true)
    ], [
        "atan2",
        new object_1.Builtin("atan2", [object_1.FLOAT_OBJ, object_1.FLOAT_OBJ], function (context, scope, jsScope, a, b) {
            return new object_1.Float(Math.atan2(a.Value, b.Value));
        }, null, null, null, true)
    ], [
        "ceil",
        new object_1.Builtin("ceil", [object_1.FLOAT_OBJ], function (context, scope, jsScope, value) {
            return new object_1.Float(Math.ceil(value.Value));
        }, null, null, null, true),
    ], [
        "floor",
        new object_1.Builtin("floor", [object_1.FLOAT_OBJ], function (context, scope, jsScope, value) {
            return new object_1.Integer(Math.floor(value.Value));
        }, null, null, null, true)
    ], [
        "fract",
        new object_1.Builtin("fract", [object_1.FLOAT_OBJ], function (context, scope, jsScope, value) {
            return value - Math.floor(value);
        }, null, null, null, false) // no native JS implementation of this
    ], [
        "sqrt",
        new object_1.Builtin("sqrt", [[object_1.FLOAT_OBJ, object_1.INTEGER_OBJ]], function (context, scope, jsScope, value) {
            return new object_1.Float(Math.sqrt(value.Value));
        }, null, null, null, true)
    ], [
        "pow",
        new object_1.Builtin("pow", [object_1.FLOAT_OBJ, object_1.INTEGER_OBJ], function (context, scope, jsScope, value, exponent) {
            return new object_1.Float(Math.pow(value.Value, exponent.Value));
        }, null, null, null, true)
    ], [
        "abs",
        new object_1.Builtin("abs", [[object_1.FLOAT_OBJ, object_1.INTEGER_OBJ]], function (context, scope, jsScope, value) {
            return new object_1.Float(Math.abs(value.Value));
        }, null, null, null, true)
    ], [
        "random",
        new object_1.Builtin("random", [], function (context, scope, jsScope, scalar) {
            var isInteger = scalar.Value - Math.floor(scalar.Value) == 0, type;
            return new object_1.Float(isInteger
                ? Math.floor(Math.random() * (typeof scalar.Value == "number" ? scalar.Value : 1))
                : Math.random() * (typeof scalar.Value == "number" ? scalar.Value : 1));
        }, null, null, null, true)
    ]
]);
