"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var obj_util_1 = require("../object/obj-util");
var object = require("../object/object");
var vector_1 = require("./vector");
var builtin_util_1 = require("./builtin-util");
function initMatrix(cols, rows) {
    var elements = [];
    for (var xy = 0, len = rows * cols; xy < len; xy++) {
        elements.push(((xy % cols == Math.floor(xy / rows)) ? 1 : 0));
    }
    return elements;
}
function get2DArray(scope) {
    var cols = scope.columns;
    var data = scope.data;
    var out = [];
    var col = 0;
    var row = [];
    for (var el = 0, l = data.length; el < l; el++) {
        row.push(data[el]);
        col++;
        if (col >= cols) {
            out.push(row);
            row = [];
            col = 0;
        }
    }
    return out;
}
var multiply = function (matColumns, matrix, argCols, vector) {
    var matRows = matrix.length / matColumns, argSize = vector.length, vecRows = argSize / argCols;
    var out = initMatrix(matRows, argCols);
    if (vecRows != matColumns) {
        return new object.ErrorObject("Argument rows: " + vecRows + " do not match matrix columns: " + matColumns + " ...");
    }
    var selfRowI = 0;
    while (selfRowI < matRows) {
        var argColJ = 0;
        while (argColJ < argCols) {
            var selfColumnK = 0;
            var acc = 0;
            while (selfColumnK < matColumns) {
                acc += (matrix[(selfRowI * matColumns) + selfColumnK]
                    *
                        vector[(selfColumnK * argCols) + argColJ]);
                selfColumnK++;
            }
            out[(selfRowI * argCols) + argColJ] = acc;
            argColJ++;
        }
        selfRowI++;
    }
    return out;
};
function MatrixState(data, columns) {
    this.data = data;
    this.columns = columns;
}
exports.Matrix = builtin_util_1.makeBuiltinClass("Matrix", [
    // ["data", new object.ArrayObject([])], // TODO: mark this as a js value
    // ["columns", new object.Integer(4)], // ALSO TODO: generate getters and or setters for native members based on access modifiers
    ["Matrix", new object.Builtin("Matrix", [object.INTEGER_OBJ], function (context, scope, jsScope, columns, data) {
            var cols = columns ? columns : 4;
            var matrix = new MatrixState(data || initMatrix(cols, cols), cols);
            if (scope) {
                scope.builtins = matrix;
                return scope;
            }
            return matrix;
        })],
    ["set", new object.Builtin("set", [object.ARRAY_OBJ], function (context, scope, jsScope, data) {
            if (!jsScope) {
                return new object.ErrorObject(obj_util_1.instanceMethodError);
            }
            jsScope.data = data;
            return data;
        }, null, null, ["data"])],
    [/*static*/ "multiply", new object.Builtin("multiply", [object.INTEGER_OBJ, object.ARRAY_OBJ, object.INTEGER_OBJ, object.ARRAY_OBJ], function (context, scope, jsScope, numColsA, matrixA, numColsB, matrixB) {
            if (!jsScope) {
                return new object.ErrorObject(obj_util_1.instanceMethodError);
            }
            return multiply(numColsA, matrixA, numColsB, matrixB);
        })],
    ["applySelf", new object.Builtin("applySelf", [object.INTEGER_OBJ, object.ARRAY_OBJ], function (context, scope, jsScope, vectorColumns, vector) {
            if (!jsScope) {
                return new object.ErrorObject(obj_util_1.instanceMethodError);
            }
            return multiply(jsScope.columns, jsScope.data, vectorColumns, vector);
        }, null, null, ["data"])],
    ["multiplySelf", new object.Builtin("multiplySelf", [object.INTEGER_OBJ, object.ARRAY_OBJ], function (context, scope, jsScope, vectorColumns, vector) {
            if (!jsScope) {
                return new object.ErrorObject(obj_util_1.instanceMethodError);
            }
            return jsScope.data = multiply(jsScope.columns, jsScope.data, vectorColumns, vector);
        }, null, null, ["data"])],
    ["multiplyScalar", vector_1.builtin_MultiplyScalar],
    ["rotateX", new object.Builtin("rotateX", [object.FLOAT_OBJ], function (context, scope, jsScope, angle) {
            if (!jsScope) {
                return new object.ErrorObject(obj_util_1.instanceMethodError);
            }
            var sine = Math.sin(angle), cosine = Math.cos(angle);
            // This should be a JS Array instead of ecs ArrayObject
            var transform = [
                1, 0, 0, 0,
                0, cosine, -sine, 0,
                0, sine, cosine, 0,
                0, 0, 0, 1
            ];
            // scope needs to hold js objects..
            // scope could possibly use 2 halfs // one half for js // one half for ecs
            jsScope.data = multiply(jsScope.columns, jsScope.data, 4, transform);
            return scope;
        }, null, null, ["data"])],
    ["rotateY", new object.Builtin("rotateY", [object.FLOAT_OBJ], function (context, scope, jsScope, angle) {
            if (!jsScope) {
                return new object.ErrorObject(obj_util_1.instanceMethodError);
            }
            var sine = Math.sin(angle), cosine = Math.cos(angle);
            var transform = [
                cosine, 0, -sine, 0,
                0, 1, 0, 0,
                sine, 0, cosine, 0,
                0, 0, 0, 1
            ];
            jsScope.data = multiply(jsScope.columns, jsScope.data, 4, transform);
            return scope;
        }, null, null, ["data"])],
    ["rotateZ", new object.Builtin("rotateZ", [object.FLOAT_OBJ], function (context, scope, jsScope, angle) {
            if (!jsScope) {
                return new object.ErrorObject(obj_util_1.instanceMethodError);
            }
            var sine = Math.sin(angle), cosine = Math.cos(angle);
            var transform = [
                cosine, -sine, 0, 0,
                sine, cosine, 0, 0,
                0, 0, 1, 0,
                0, 0, 0, 1
            ];
            jsScope.data = multiply(jsScope.columns, jsScope.data, 4, transform); // TODO: move matrix error handling up a level
            return scope;
        }, null, null, ["data"])],
    ["scaleSelf", new object.Builtin("scaleSelf", [object.ARRAY_OBJ], function (context, scope, jsScope, vec) {
            if (!jsScope) {
                return new object.ErrorObject(obj_util_1.instanceMethodError);
            }
            var transform = [
                vec[0], 0, 0, 0,
                0, vec[1], 0, 0,
                0, 0, vec[2], 0,
                0, 0, 0, 1
            ];
            jsScope.data = multiply(jsScope.columns, jsScope.data, 4, transform);
            return scope;
        }, null, null, ["data"])],
    ["translate", new object.Builtin("translate", [object.ARRAY_OBJ], function (context, scope, jsScope, translation) {
            if (!jsScope) {
                return new object.ErrorObject(obj_util_1.instanceMethodError);
            }
            var transform = [
                1, 0, 0, 0,
                0, 1, 0, 0,
                0, 0, 1, 0,
                translation[0], translation[1], translation[2], 1
            ];
            jsScope.data = multiply(jsScope.columns, jsScope.data, 4, transform);
            return scope;
        }, null, null, ["data"])],
    ["get2DArray", new object.Builtin("get2DArray", [], function (context, scope, jsScope) {
            if (!jsScope) {
                return new object.ErrorObject(obj_util_1.instanceMethodError);
            }
            return get2DArray(jsScope);
        })
    ],
    ["List", new object.Builtin("List", [], function (context, scope, jsScope) {
            if (!jsScope) {
                return new object.ErrorObject(obj_util_1.instanceMethodError);
            }
            return jsScope.data;
        })
    ],
    ["String", new object.Builtin("String", [], function (context, scope, jsScope) {
            if (!jsScope) {
                return new object.ErrorObject(obj_util_1.instanceMethodError);
            }
            return get2DArray(jsScope).join("\n");
        })
    ]
]);
