"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var object = require("../object/object");
var obj_util_1 = require("../object/obj-util");
var builtin_util_1 = require("./builtin-util");
function VectorState(data) {
    this.data = data;
}
exports.multiplyScalar = function (context, scope, jsScope, scalar) {
    var matrix = jsScope.data, elements = [];
    for (var i = 0, elems = matrix.length; i < elems; i++) {
        elements.push(matrix[i] * scalar);
    }
    return elements;
};
exports.builtin_MultiplyScalar = new object.Builtin('multiplyScalar', [[object.FLOAT_OBJ, object.INTEGER_OBJ]], function (context, scope, jsScope, scalar) {
    return exports.multiplyScalar(null, scope, jsScope, scalar);
});
function addVectors(into, argEls, sign) {
    into[0] += argEls[0] * sign;
    into[1] += argEls[1] * sign;
    switch (argEls.length) {
        case 4:
            into[2] += argEls[2] * sign;
            into[3] += argEls[3] * sign;
            return into;
        case 2:
            return into;
        default:
            for (var i = 2, l = into.length; i < l; i++) {
                into[i] += argEls[i] * sign;
            }
    }
    return into;
}
exports.addVectors = addVectors;
function crossProduct(vector, other) {
    var ax = vector[0], ay = vector[1], az = vector[2];
    var bx = other[0], by = other[1], bz = other[2];
    return [
        ay * bz - az * by,
        az * bx - ax * bz,
        ax * by - ay * bx
    ];
}
exports.Vector = builtin_util_1.makeBuiltinClass("Vector", [
    //["data", new object.ArrayObject([])],
    ["Vector", new object.Builtin("Vector", [], function (context, scope, jsScope, data) {
            var vector = new VectorState(data || [0, 0, 0, 1]);
            if (scope) {
                scope.builtins = vector;
                return scope;
            }
            return vector;
        })],
    [/* static */ "add", new object.Builtin("add", [object.ARRAY_OBJ], function (context, scope, jsScope, vector, other) {
            if (!jsScope) {
                return new object.ErrorObject(obj_util_1.instanceMethodError);
            }
            return addVectors(vector, other, 1);
        })],
    [/* static */ "sub", new object.Builtin("sub", [object.ARRAY_OBJ], function (context, scope, jsScope, vector, other) {
            if (!jsScope) {
                return new object.ErrorObject(obj_util_1.instanceMethodError);
            }
            return addVectors(vector, other, -1);
        })],
    ["addSelf", new object.Builtin("addSelf", [object.ARRAY_OBJ], function (context, scope, jsScope, other) {
            if (!jsScope) {
                return new object.ErrorObject(obj_util_1.instanceMethodError);
            }
            jsScope.data = addVectors(jsScope.data, other, 1);
            return jsScope.data;
        }, null, null, ["data"])],
    ["subSelf", new object.Builtin("subSelf", [object.ARRAY_OBJ], function (context, scope, jsScope, other) {
            if (!jsScope) {
                return new object.ErrorObject(obj_util_1.instanceMethodError);
            }
            jsScope.data = addVectors(jsScope.data, other, -1);
            return jsScope.data;
        }, null, null, ["data"])],
    ["dot", new object.Builtin("dot", [object.ARRAY_OBJ], function (context, scope, jsScope, other) {
            if (!jsScope) {
                return new object.ErrorObject(obj_util_1.instanceMethodError);
            }
            var data = jsScope.data;
            if (data.length == 3) {
                return data[0] * other[0] + data[1] * other[1] + data[2] * other[2];
            }
            var sum = 0;
            for (var e = 0, l = data.length; e < l; e++) {
                sum += data[e] * other[e];
            }
            return sum;
        })],
    [/* static */ "cross", new object.Builtin("cross", [object.ARRAY_OBJ, object.ARRAY_OBJ], function (context, scope, jsScope, vector, other) {
            if (!jsScope) {
                return new object.ErrorObject(obj_util_1.instanceMethodError);
            }
            return crossProduct(vector, other);
        })],
    ["crossSelf", new object.Builtin("crossSelf", [object.ARRAY_OBJ], function (context, scope, jsScope, other) {
            if (!jsScope) {
                return new object.ErrorObject(obj_util_1.instanceMethodError);
            }
            return jsScope.data = crossProduct(jsScope.data, other);
        }, null, null, ["data"])],
    ["multiplyScalar", exports.builtin_MultiplyScalar],
    ["String", new object.Builtin("String", [], function (context, scope, jsScope) {
            if (!jsScope) {
                return new object.ErrorObject(obj_util_1.instanceMethodError);
            }
            return jsScope.data.map(function (el) { return el + ""; }).join(", ");
        })],
    ["List", new object.Builtin("List", [], function (context, scope, jsScope) {
            if (!jsScope) {
                return new object.ErrorObject(obj_util_1.instanceMethodError);
            }
            return jsScope.data.concat([]);
        })]
]);
