"use strict";
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
Object.defineProperty(exports, "__esModule", { value: true });
var object = require("./object/object");
var environment = require("./object/environment");
var builtins_1 = require("./builtins");
var util = require("./util");
var objUtil = require("./object/obj-util");
var file_1 = require("./builtins/io/file");
var object_1 = require("./object/object");
var util_1 = require("./util");
var node_objects_1 = require("./repl/node-objects");
var token_1 = require("./token");
var optimizer_1 = require("./runtime/optimizer");
var javascript_1 = require("./transpiler/javascript");
var error_1 = require("./object/error");
var analyzer_1 = require("./transpiler/analyzer");
var runtime_1 = require("./runtime");
var embeddedInterpreter;
var optimizer = new optimizer_1.Optimizer(new javascript_1.JSTranspiler(), applyFunction, Eval);
var analyzer = new analyzer_1.Analyzer();
function nativeBoolToBooleanEObject(input) {
    if (input) {
        return object.TRUE;
    }
    return object.FALSE;
}
function Eval(node, env, objectContext, analysis) {
    if (!!!node) {
        return;
    }
    var left = null, right = null, index = null, val = null;
    switch (node.NodeName) {
        // Statements
        case "Program":
            // console.log(JSON.stringify(node, null, 2));
            return optimizer.optimizedEvaluate(analysis, node, env, function () { return evalProgram(node, env); });
        //evalProgram(node, env);
        case "BlockStatement":
            return evalBlockStatement(node, env, objectContext);
        case "ReturnStatement":
            val = Eval(node.ReturnValue, env, objectContext);
            if (isError(val)) {
                return val;
            }
            return new object.ReturnValue(val);
        case "ClassStatement":
            val = Eval(node.Value, env, objectContext);
            if (isError(val)) {
                return val;
            }
            var pair = val.Pairs[node.Name.Value], constructor = pair ? pair.Value : null;
            if (constructor != null) {
                val.Constructor = constructor;
            }
            env.set(node.Name.Value, val);
            break;
        case "LetStatement":
            if (node.Vars) {
                for (var v in node.Vars) {
                    var variable = node.Vars[v];
                    val = Eval(variable[1], env, objectContext);
                    if (isError(val)) {
                        return val;
                    }
                    env.set(variable[0].Value, val);
                }
            }
            else {
                val = Eval(node.Value, env, objectContext);
                if (isError(val)) {
                    return val;
                }
                env.set(node.Name.Value, val);
            }
            break;
        case "AssignmentStatement":
            val = Eval(node.Value, env, objectContext);
            if (isError(val)) {
                return val;
            }
            env.set(node.Name.Value, val);
            break;
        case "ExpressionStatement":
            return Eval(node.Expression, env, objectContext);
        // Expressions
        case "PrefixExpression":
            right = Eval(node.Right, env, objectContext);
            if (isError(right)) {
                return right;
            }
            return evalPrefixExpression(node.Operator, right);
        case "InfixExpression":
            left = Eval(node.Left, env, objectContext);
            if (isError(left)) {
                return left;
            }
            right = Eval(node.Right, env, objectContext);
            if (isError(right)) {
                return right;
            }
            return evalInfixExpression(node.Operator, left, right);
        case "CallExpression":
            var fun = Eval(node.Function, env); //, node.Function.ObjectContext); // objectContext);
            if (isError(fun)) {
                return fun;
            }
            var args = evalExpressions(node.Arguments, env, objectContext);
            if (args.length == 1 && isError(args[0])) {
                return args[0];
            }
            return applyFunction(fun, env, args, objectContext);
        case "IndexExpression":
            left = Eval(node.Left, env, objectContext);
            if (isError(left)) {
                return left;
            }
            index = Eval(node.Index, env, objectContext);
            if (isError(index)) {
                return index;
            }
            return evalIndexExpression(left, index, (left.Constructor ? left : objectContext));
        case "IndexAssignmentExpression":
            left = Eval(node.Left, env, objectContext);
            if (isError(left)) {
                return left;
            }
            index = Eval(node.Index, env, objectContext);
            if (isError(index)) {
                return index;
            }
            var assignment = Eval(node.Assignment, env, objectContext);
            if (isError(assignment)) {
                return assignment;
            }
            return evalIndexAssignmentExpression(left, index, assignment, objectContext);
        case "Boolean":
            return nativeBoolToBooleanEObject(node.Value);
        case "IntegerLiteral":
            return new object.Integer(node.Value);
        case "FloatLiteral":
            return new object.Float(node.Value);
        case "FunctionLiteral":
            return new object_1.LambdaFunction(node.Parameters, node.Body, env, null, objectContext, node.ReturnType, node.ParameterTypes);
        case "StringLiteral":
            return new object_1.StringObject(node.Value);
        case "ArrayLiteral":
            var elements = evalExpressions(node.Elements, env);
            if (elements.length == 1 && isError(elements[0])) {
                return elements[0];
            }
            return new object.ArrayObject(elements);
        case "HashLiteral":
            return evalHashLiteral(node, env);
        case "StreamExpression":
            return evalStreamExpression(node, env, objectContext);
        case "IfStatement":
            return evalIfStatement(node, env, objectContext);
        case "ForStatement":
            return evalForStatement(node, env, objectContext);
        case "WhileStatement":
            return evalWhileStatement(node, env, objectContext);
        case "SleepStatement":
            return evalSleepStatement(node, env, objectContext);
        case "NewExpression":
            return evalNewExpression(node, env, objectContext);
        case "ExecExpression":
            return evalExecExpression(node, env, objectContext);
        case "Identifier":
            return evalIdentifier(node, env, objectContext);
    }
    return null;
}
exports.Eval = Eval;
embeddedInterpreter = runtime_1.makeRuntimeEnvironment(Eval, evalProgram);
function optimizedEvalProgram(program, env) {
    var analysis = analyzer.analyzeParseTree(program);
    return optimizer.optimizedEvaluate(analysis, program, env, function () { return evalProgram(program, env); });
}
exports.optimizedEvalProgram = optimizedEvalProgram;
function evalProgram(program, env) {
    var result;
    var statements = program.Statements;
    for (var s = 0, l = statements.length; s < l; s++) {
        var stmt = statements[s];
        result = Eval(stmt, env);
        if (!!result && result.Message) {
            return result;
        }
    }
    return result;
}
exports.evalProgram = evalProgram;
function evalBlockStatement(block, env, objectContext) {
    var result, statements = block.Statements;
    for (var bs = 0, l = statements.length; bs < l; bs++) {
        var stmt = statements[bs];
        // block.Statements.forEach( statement => {
        result = Eval(stmt, env, objectContext);
        if (!!result) {
            var rt = result && result.Type ? result.Type() : null;
            if (rt == object.RETURN_VALUE_OBJ || rt == object.ERROR_OBJ) {
                return result;
            }
        }
    }
    return result;
}
function evalExpressions(exps, env, objectContext) {
    var result = [];
    exps.forEach(function (e) {
        var evaluated = Eval(e, env, objectContext);
        if (isError(evaluated)) {
            return Array();
        }
        result.push(evaluated);
    });
    return result;
}
function evalPrefixExpression(operator, right) {
    switch (operator) {
        case "!":
            return evalBangOperatorExpression(right);
        case "-":
            return evalMinusPrefixOperatorExpression(right);
        case "typeof":
            return evalTypeofExpression(right);
        default:
            return error_1.newError("unknown operator: %s%s", operator, right.Type());
    }
}
function evalInfixExpression(operator, left, right) {
    var leftType = left.Type(), rightType = right.Type();
    if (operator == "==") {
        return nativeBoolToBooleanEObject(left.Value == right.Value);
    }
    else if (operator == "!=") {
        return nativeBoolToBooleanEObject(left.Value != right.Value);
    }
    else if (leftType == object_1.INTEGER_OBJ && rightType == object_1.INTEGER_OBJ) {
        return evalIntegerInfixExpression(operator, left, right);
    }
    else if (rightType == object_1.FLOAT_OBJ || leftType == object_1.FLOAT_OBJ) {
        return evalFloatInfixExpression(operator, left, right);
    }
    else if (leftType == object.BOOLEAN_OBJ || leftType == object_1.NULL_OBJ) {
        return evalBooleanInfixExpression(operator, left, right);
    }
    else if (rightType == object.BOOLEAN_OBJ || rightType == object_1.NULL_OBJ) {
        return evalBooleanInfixExpression(operator, left, right);
    }
    else if (leftType == object.STRING_OBJ && rightType == object.STRING_OBJ) {
        return evalStringInfixExpression(operator, left, right);
    }
    else if (leftType != rightType) {
        if (leftType == object.STRING_OBJ) {
            if (operator == "*") {
                return evalStringRepeatExpression(left, right);
            }
            else {
                return evalStringConcatExpression(left, right, true);
            }
        }
        else if (rightType == object.STRING_OBJ) { // leftType == INTEGER_OBJ &&
            if (operator == "*") {
                return evalStringRepeatExpression(right, left);
            }
            else {
                return evalStringConcatExpression(left, right, false);
            }
        }
        return error_1.newError("type mismatch: %s %s %s", leftType, operator, rightType);
    }
    else {
        return error_1.newError("unknown operator: %s %s %s", leftType, operator, rightType);
    }
}
function evalTypeofExpression(right) {
    if (right.Type() == object.HASH_OBJ && right.className != null) {
        return new object_1.StringObject(right.className);
    }
    else {
        return new object_1.StringObject(right.Type());
    }
}
function evalStringConcatExpression(left, right, stringFirst) {
    if (stringFirst) {
        return new object_1.StringObject(left.Value + right.Inspect());
    }
    else {
        return new object_1.StringObject(left.Inspect() + right.Value);
    }
}
function evalBooleanInfixExpression(operator, left, right) {
    var leftVal = left.Value, rightVal = right.Value;
    if (leftVal === undefined) {
        leftVal = true;
    }
    if (rightVal === undefined) {
        rightVal = true;
    }
    switch (operator) {
        case "&&":
            return new object_1.BooleanObject(leftVal && rightVal);
        case "||":
            return new object_1.BooleanObject(leftVal || rightVal);
        default:
            return error_1.newError("unknown operator: %s %s %s", left.Type(), operator, right.Type());
    }
}
function evalIntegerInfixExpression(operator, left, right) {
    var leftVal = left.Value, rightVal = right.Value;
    switch (operator) {
        case "+":
            return new object.Integer(leftVal + rightVal);
        case "-":
            return new object.Integer(leftVal - rightVal);
        case "*":
            return new object.Integer(leftVal * rightVal);
        case "/":
            return new object.Integer(leftVal / rightVal);
        case "%":
            return new object.Integer(leftVal % rightVal);
        case "<":
            return nativeBoolToBooleanEObject(leftVal < rightVal);
        case ">":
            return nativeBoolToBooleanEObject(leftVal > rightVal);
        case "==":
            return nativeBoolToBooleanEObject(leftVal == rightVal);
        case "!=":
            return nativeBoolToBooleanEObject(leftVal != rightVal);
        default:
            return error_1.newError("unknown operator: %s %s %s", left.Type(), operator, right.Type());
    }
}
function evalFloatInfixExpression(operator, left, right) {
    var leftVal = left.Value, rightVal = right.Value;
    switch (operator) {
        case "+":
            return new object.Float(leftVal + rightVal);
        case "-":
            return new object.Float(leftVal - rightVal);
        case "*":
            return new object.Float(leftVal * rightVal);
        case "/":
            return new object.Float(leftVal / rightVal);
        case "%":
            return new object.Float(leftVal % rightVal);
        case "<":
            return nativeBoolToBooleanEObject(leftVal < rightVal);
        case ">":
            return nativeBoolToBooleanEObject(leftVal > rightVal);
        case "==":
            return nativeBoolToBooleanEObject(leftVal == rightVal);
        case "!=":
            return nativeBoolToBooleanEObject(leftVal != rightVal);
        default:
            return error_1.newError("unknown operator: %s %s %s", left.Type(), operator, right.Type());
    }
}
function evalStringRepeatExpression(left, right) {
    var leftVal = left.Value, rightVal = right.Value;
    return new object_1.StringObject(leftVal.repeat(rightVal));
}
function evalStringInfixExpression(operator, left, right) {
    var leftVal = left.Value, rightVal = right.Value;
    switch (operator) {
        case "+":
            return new object_1.StringObject(leftVal + rightVal);
        case "==":
            return nativeBoolToBooleanEObject(leftVal == rightVal);
        case "!=":
            return nativeBoolToBooleanEObject(leftVal != rightVal);
        case "<":
            return nativeBoolToBooleanEObject(leftVal < rightVal);
        case ">":
            return nativeBoolToBooleanEObject(leftVal > rightVal);
        default:
            return error_1.newError("unknown operator: %s %s %s", left.Type(), operator, right.Type());
    }
}
function evalNewExpression(ne, env, objectContext) {
    var classData = evalIdentifier(ne.Name, env, objectContext);
    var classType = classData.Type();
    if (classType == object.HASH_OBJ) {
        var instance = objUtil.copyHashMap(classData);
        if (instance.Constructor == null && instance.Pairs[ne.Name.Value] != null) {
            instance.Constructor = instance.Pairs[ne.Name.Value].Value;
            instance.className = ne.Name.Value;
            bindContextToMethods(instance);
        }
        return instance;
    }
    else if (classType == object.ARRAY_OBJ) {
        return objUtil.copyList(classData);
    }
    else {
        return error_1.newError("new operator can only be used with Class or Hashmap. Invalid type: " + classData.Type());
    }
}
function bindContextToMethods(instance) {
    var pairs = instance.Pairs;
    for (var p in pairs) {
        var pairType = pairs[p].Value.Type();
        if (pairType == object.BUILTIN_OBJ || pairType == object.FUNCTION_OBJ) {
            // console.log("setting object context", instance);
            pairs[p].Value.ObjectContext = instance;
        }
    }
}
function evalExecExpression(se, env, objectContext) {
    var files = se.Name.Elements;
    var path = files ? files[0].Value : "", idx = 0;
    if (path.length == 0) {
        return error_1.newError("path can't be empty");
    }
    var all = [];
    while (idx < files.length) {
        path = files[idx] ? files[idx].Value : "";
        all.push(util.evaluateSourceFile(env, embeddedInterpreter, path, file_1.readWholeFile, node_objects_1.nodeObjects.fs));
        idx++;
    }
    Promise.all(all).then(function () {
        evalBlockStatement(se.Callback, env, objectContext);
    }).catch(function (err) {
        util_1.printNativeString(null, null, "Error executing file: " + err);
    });
    return object_1.NULL;
}
function evalIfStatement(ie, env, objectContext) {
    var condition = Eval(ie.Condition, env, objectContext);
    if (isError(condition)) {
        return condition;
    }
    if (isTruthy(condition)) {
        return Eval(ie.Consequence, env, objectContext);
    }
    else if (ie.Alternative != null) {
        return Eval(ie.Alternative, env, objectContext);
    }
    else {
        return object_1.NULL;
    }
}
function evalForStatement(fl, env, objectContext) {
    var range = Eval(fl.Range, env, objectContext), element = fl.Element.Value, rangeType, index = 0, indexObj = new object.Integer(index), length = 0;
    if (isError(range)) {
        return range;
    }
    rangeType = range.Type();
    var err, result;
    if (rangeType == object_1.INTEGER_OBJ) {
        length = (range).Value;
        while (index < length) {
            indexObj.Value = index;
            env.set(element, indexObj);
            result = Eval(fl.Consequence, env, objectContext);
            err = err || isError(result) ? result : null;
            index += 1;
        }
    }
    else if (rangeType == object.ARRAY_OBJ) {
        length = (range).Elements.length;
        while (index < length) {
            indexObj.Value = index;
            env.set(element, indexObj);
            result = Eval(fl.Consequence, env, objectContext);
            err = err || isError(result) ? result : null;
            index += 1;
        }
    }
    else if (rangeType == object.STRING_OBJ) {
        length = (range).Value.length;
        while (index < length) {
            indexObj.Value = index;
            env.set(element, indexObj);
            result = Eval(fl.Consequence, env, objectContext);
            err = err || isError(result) ? result : null;
            index += 1;
        }
    }
    else if (rangeType == object.HASH_OBJ) {
        var keys = Object.keys(range.Pairs);
        length = keys.length;
        while (index < length) {
            env.set(element, new object_1.StringObject(keys[index]));
            result = Eval(fl.Consequence, env, objectContext);
            err = err || isError(result) ? result : null;
            index += 1;
        }
        if (err != null) {
            return error_1.newError("error in for loop %s", JSON.stringify(err));
        }
    }
    else {
        return error_1.newError("unknown range type in for loop: %s", range.Type());
    }
    return object_1.NULL;
}
function evalWhileStatement(ie, env, objectContext) {
    var condition = Eval(ie.Condition, env);
    if (isError(condition)) {
        return condition;
    }
    while (isTruthy(condition)) {
        Eval(ie.Consequence, env, objectContext);
        condition = Eval(ie.Condition, env, objectContext);
    }
    return object_1.NULL;
}
function evalSleepStatement(se, env, objectContext) {
    var duration = Eval(se.Duration, env);
    if (isError(duration)) {
        return duration;
    }
    setTimeout(function () {
        Eval(se.Consequence, env, objectContext);
    }, duration.Inspect());
    return object_1.NULL;
}
function evalStreamExpression(se, env, objectContext) {
    var transforms = [];
    if (se.Transforms.length) {
        var expTransforms = se.Transforms;
        for (var t = 0, l = expTransforms.length; t < l; t++) {
            transforms.push(Eval(expTransforms[t], env, objectContext));
        }
    }
    var source = Eval(se.Source, env, objectContext);
    var sink = Eval(se.Sink, env, objectContext);
    var stream = new object.Stream(se.Direction, transforms, source, sink);
    // if the source is a builtin,
    if (source.Type() == "BUILTIN" && se.Direction == token_1.STREAM_DIRECTION.READ) { // a host event handler has to be added
        applyBuiltinFunction(source, [
            transforms.length
                ? transforms[0] // if there is a stream transform chain, pass the event data to it.
                : sink // if there are no transformations, connect the source and sink directly with an event handler.
        ], env);
    }
    // now the stream is created.. 
    /*

                     | Source is builtin     |
                     |_______________________|________________________|_____________________|_____________________
                     | call it               |
                     | to add event listener |
_____________________|_______________________|
                    
    */
    return stream;
}
function evalIndexExpression(left, index, objectContext) {
    var indexExpType = left.Type() == object.ARRAY_OBJ && index.Type() == object_1.INTEGER_OBJ
        ? "arrayIndex"
        : "default";
    indexExpType = left.Type() == object.HASH_OBJ ? "hashIndex" : indexExpType;
    indexExpType = left.Type() == object.STRING_OBJ ? "stringIndex" : indexExpType;
    switch (indexExpType) {
        case "arrayIndex":
            return evalArrayIndexExpression(left, index);
        case "hashIndex":
            return evalHashIndexExpression(left, index, objectContext);
        case "stringIndex":
            return evalStringIndexExpression(left, index);
        default:
            return error_1.newError("index operator not supported: %s", left.Type());
    }
}
function evalArrayIndexExpression(array, index) {
    var arrayEObject = array, idx = index.Value, max = arrayEObject.Elements.length - 1;
    if (idx < 0 || idx > max) {
        return object_1.NULL;
    }
    return arrayEObject.Elements[idx];
}
function evalHashIndexExpression(hash, index, objectContext) {
    var hashObject = hash, ok = index.HashKey, key = null;
    if (!ok) {
        return error_1.newError("unusable as hash key: %s", index.Type());
    }
    else {
        key = index.HashKey();
    }
    var pair = hashObject.Pairs[key];
    ok = pair != null;
    if (!ok) {
        return object_1.NULL;
    }
    if (!objectContext && pair.modifiers && pair.modifiers.indexOf(0) > -1) {
        return error_1.newError("cannot access private field: %s", index.Inspect());
    }
    return pair.Value;
}
function evalStringIndexExpression(array, index) {
    var stringObject = array, idx = index.Value, max = stringObject.Value.length - 1;
    if (idx < 0 || idx > max) {
        return object_1.NULL;
    }
    return new object_1.StringObject(stringObject.Value[idx]);
}
function evalIndexAssignmentExpression(left, index, assignment, objectContext) {
    var indexExpType = left.Type() == object.ARRAY_OBJ && index.Type() == object_1.INTEGER_OBJ
        ? "arrayIndex"
        : "default";
    indexExpType = left.Type() == object.HASH_OBJ ? "hashIndex" : indexExpType;
    indexExpType = left.Type() == object.STRING_OBJ ? "stringIndex" : indexExpType;
    switch (indexExpType) {
        case "arrayIndex":
            return evalArrayIndexAssignment(left, index, assignment);
        case "hashIndex":
            return evalHashIndexAssignment(left, index, assignment, objectContext);
        case "stringIndex":
            return evalStringIndexAssignment(left, index, assignment);
        default:
            return error_1.newError("index operator not supported: %s", left.Type());
    }
}
function evalStringIndexAssignment(str, index, assignment) {
    var stringObject = str, idx = index.Value, max = stringObject.Value.length - 1;
    if (idx < 0 || idx > max) {
        return object_1.NULL;
    }
    var oldStr = stringObject.Value, left = oldStr.substr(0, idx), right = oldStr.substr(idx + 1, oldStr.length - 1);
    stringObject.Value = left + assignment.Inspect() + right;
    return object_1.NULL;
}
function evalArrayIndexAssignment(array, index, assignment) {
    var arrayEObject = array, idx = index.Value, max = arrayEObject.Elements.length - 1;
    if (idx < 0 || idx > max) {
        return object_1.NULL;
    }
    arrayEObject.Elements[idx] = assignment;
    return object_1.NULL;
}
function evalHashIndexAssignment(hash, index, assignment, objectContext) {
    var hashObject = hash, ok = index.Value, key = null;
    if (!ok) {
        return error_1.newError("unusable as hash key: %s", index.Type());
    }
    else {
        key = index.Value;
    }
    var pair = hashObject.Pairs[key];
    ok = pair != null;
    if (!ok) {
        return object_1.NULL;
    }
    if (pair.modifiers) {
        if (!objectContext && pair.modifiers.indexOf(0) > -1) {
            return error_1.newError("cannot access private field: %s", index.Inspect());
        }
        if (pair.modifiers.indexOf(2) > -1) {
            return error_1.newError("cannot modify final field: %s", index.Inspect());
        }
    }
    pair.Value = assignment;
    return object_1.NULL;
}
function evalIdentifier(node, env, objectContext) {
    var name = node.Value;
    if (name == "this") {
        if (objectContext != null) {
            return objectContext;
        }
        return error_1.newError("statement has no object context");
    }
    else {
        var ident = env.get(name);
        if (ident != null) {
            return ident;
        }
        var builtin = builtins_1.builtins[name];
        if (builtin != null && builtin != undefined) {
            return builtin;
        }
        return error_1.newError("identifier not found: %s", name);
    }
}
function evalHashLiteral(node, env) {
    var hashmap = new object.Hash({}), pairs = {};
    for (var key in node.Pairs) {
        var value = void 0, hashPair = void 0;
        value = Eval(node.Pairs[key], env, hashmap);
        if (isError(value)) {
            return value;
        }
        hashPair = ({ Key: new object_1.StringObject(key), Value: value });
        if (node.Modifiers[key] != null) {
            hashPair.modifiers = node.Modifiers[key];
        }
        pairs[key] = hashPair;
    }
    hashmap.Pairs = pairs;
    return hashmap;
}
function evalBangOperatorExpression(right) {
    switch (right) {
        case object.TRUE:
            return object.FALSE;
        case object.FALSE:
            return object.TRUE;
        case object_1.NULL:
            return object.TRUE;
        default:
            return object.FALSE;
    }
}
function evalMinusPrefixOperatorExpression(right) {
    if (right.Type() != object_1.INTEGER_OBJ) {
        return error_1.newError("unknown operator: -%s", right.Type());
    }
    var value = right.Value;
    return new object.Integer(-value);
}
function applyFunction(fn, env, args, objectContext) {
    var fnType = fn.Type();
    var extendedEnv, evaluated;
    switch (fnType) {
        case "pure_function":
            evaluated = Eval(fn.Body, createPureFunctionEnv(fn.Parameters, args));
            return unwrapReturnValue(evaluated);
        case "function":
            extendedEnv = extendFunctionEnv(fn, args);
            evaluated = Eval(fn.Body, extendedEnv, fn.ObjectContext);
            return unwrapReturnValue(evaluated);
        case "hash":
            var constructor = fn.Constructor;
            if (!constructor) {
                return error_1.newError("object has no constructor");
            }
            if (constructor.Parameters) { // Function Literal constructor
                extendedEnv = extendFunctionEnv(constructor, args);
                evaluated = Eval(constructor.Body, extendedEnv, fn);
            }
            else { //                      Builtin Function Constructor
                evaluated = applyBuiltinFunction(constructor, args, env, fn);
            }
            return fn;
        case "BUILTIN":
            return applyBuiltinFunction(fn, args, env, objectContext);
        default:
            return error_1.newError("not a function: %s", fn.Type());
    }
}
exports.applyFunction = applyFunction;
function applyBuiltinFunction(fn, args, env, objectContext) {
    var contextName = fn.Context, context = null, scope = null;
    context = env.getContext();
    if (!context) {
        context = {};
    }
    context.env = env;
    var objContext = fn.ObjectContext || objectContext; // can this be simpler?
    if (objContext) {
        scope = objContext;
    }
    else if (contextName) {
        scope = context ? context.data[contextName] : null;
    }
    if (fn.ecsOnly) {
        return fn.Fn.apply(fn, __spreadArrays([context, scope, scope ? scope.builtins : null], args));
    }
    else {
        return fn.callFromECSRuntime(context, scope, optimizer, args);
    }
}
exports.applyBuiltinFunction = applyBuiltinFunction;
function createPureFunctionEnv(Parameters, args) {
    var env = new environment.Environment(), paramIdx = 0, param = null, nParams = Parameters.length;
    while (paramIdx < nParams) {
        param = Parameters[paramIdx];
        env.set(param.Value, args[paramIdx]);
        paramIdx += 1;
    }
    return env;
}
function extendFunctionEnv(fn, args) {
    var env = environment.NewEnclosedEnvironment(fn.Env), paramIdx = 0, param = null, nParams = fn.Parameters.length;
    while (paramIdx < nParams) {
        param = fn.Parameters[paramIdx];
        env.set(param.Value, args[paramIdx]);
        paramIdx += 1;
    }
    return env;
}
function unwrapReturnValue(obj) {
    if (!obj || !obj.Type) {
        return object_1.NULL;
    }
    if (obj.Type() == object.RETURN_VALUE_OBJ) {
        return obj.Value;
    }
    return obj;
}
function isError(obj) {
    if ((obj) != null) {
        return !!(obj.Message); //obj.Type() == object.ERROR_OBJ
    }
    return false;
}
function isTruthy(obj) {
    return obj && (obj.Value !== false);
}
