"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var repl_1 = require("./repl");
var environment_1 = require("./object/environment");
var evaluator_1 = require("./evaluator");
var parser_1 = require("./parser");
var lexer_1 = require("./lexer");
var runtime_1 = require("./runtime");
var translator_1 = require("./transpiler/translator");
var lang = new translator_1.Translator();
var ECS = /** @class */ (function () {
    function ECS(config) {
        this.window = this.window;
        this.config = config;
    }
    ECS.prototype.repl = function (args, makeCLIRepl, replPlugins) {
        this.env = repl_1.startRepl("cli", args, makeCLIRepl, replPlugins);
    };
    ECS.prototype.webRepl = function () {
        this.env = repl_1.startRepl("web");
    };
    ECS.prototype.getEnv = function () {
        return this.env;
    };
    ECS.makeRuntimeEnvironment = function () {
        return runtime_1.makeRuntimeEnvironment(evaluator_1.Eval, evaluator_1.evalProgram);
    };
    ECS.makeEnvironment = function () {
        return new environment_1.Environment();
    };
    ECS.parse = function (sourceCode) {
        var l = new lexer_1.Lexer(), p = new parser_1.Parser(l);
        l.setText(sourceCode);
        p.parse();
        return {
            tree: p.parseProgram(),
            analysis: p.diagnosticContext
        };
    };
    ECS.transpile = function (augmentedAST, customCompiler) {
        if (customCompiler) {
            return customCompiler.transpile(augmentedAST.tree, null, augmentedAST.analysis);
        }
        return lang.getTranspiler("js").transpile(augmentedAST.tree, null, augmentedAST.analysis);
    };
    return ECS;
}());
exports.ECS = ECS;
