"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var actions_1 = require("../builtins/convolvr/actions");
var obj_util_1 = require("./obj-util");
var events_1 = require("../runtime/events");
var Environment = /** @class */ (function () {
    function Environment(outer, context) {
        var _this = this;
        this.events = {};
        this.getContext = function () {
            return _this.context || (_this.outer ? _this.outer.getContext() : makeSynchronousEventContext(["0.0.0", 0, 0]));
        };
        this.store = {}; //new Map<string, any>();
        this.outer = outer;
        if (context) {
            if (this.context) {
                context.data = __assign(__assign({}, this.context.data), context.data);
            }
            this.context = context;
        }
    }
    Environment.prototype.get = function (name) {
        var value = this.store[name];
        if (value == null && this.outer != null) {
            value = this.outer.get(name);
        }
        return value;
    };
    Environment.prototype.set = function (name, value) {
        this.store[name] = value; //.set(name, value);
        return value;
    };
    Environment.prototype.setContext = function (name, value) {
        this.context.data[name] = value;
    };
    /***
     * addEventHandler is called from builtin functions to register external observables
     */
    Environment.prototype.addEventHandler = function (system, eventName, callback) {
        if (!this.hasEvents(system)) {
            this.initEvents(system);
        }
        if (!this.events[system][eventName]) {
            this.events[system][eventName] = [];
        }
        this.events[system][eventName].push(callback);
    };
    /***
     * onEvent is called when the web-worker gets an `event` message from outside
     * This is the hot path 🔥🔥🔥
     */
    Environment.prototype.onEvent = function (system, eventName, data, interpreter, env, synchronousContext) {
        if (this.hasEvents(system)) {
            var handlers = this.events[system][eventName], objectArgs = obj_util_1.nativeListToListOfObjects(data);
            for (var h in handlers) {
                interpreter.applyFunction(handlers[h], env, objectArgs); // what could go wrong
                // TODO: 
            }
        }
    };
    Environment.prototype.hasEvents = function (system) {
        return !!this.events[system];
    };
    Environment.prototype.initEvents = function (system) {
        var templateEvents = actions_1.events[system];
        var handlers = {};
        for (var e in templateEvents) {
            var eventName = templateEvents[e];
            handlers[eventName] = [];
        }
        this.events[system] = handlers;
    };
    return Environment;
}());
exports.Environment = Environment;
function NewEnclosedEnvironment(outer) {
    return new Environment(outer);
}
exports.NewEnclosedEnvironment = NewEnclosedEnvironment;
function makeSynchronousEventContext(path, data) {
    return {
        data: data || {},
        postMessage: function (command, data) {
            // there must be a better way
            events_1.SynchronousEventContext.postMessage(command, data);
        }
    };
}
exports.makeSynchronousEventContext = makeSynchronousEventContext;
