"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var object_1 = require("./object");
exports.instanceMethodError = "non-static, instance methods must be called on an instance";
// things
function copyListElements(list) {
    var newElements = [], elements = list.Elements;
    for (var x = 0, l = elements.length; x < l; x++) {
        newElements.push(copyObject(elements[x]));
    }
    return newElements;
}
exports.copyListElements = copyListElements;
exports.copyList = function (list) {
    return new object_1.ArrayObject(copyListElements(list));
};
// stuff
function copyHashMap(data) {
    var objectContext = {};
    var allKeys = Object.keys(data.Pairs), pairData = data.Pairs;
    var pairs = {};
    for (var pk in allKeys) {
        var pairKey = allKeys[pk], pair = pairData[pairKey], valueNode = pair.Value, keyNode = pair.Key, isStatic = pair.modifiers && pair.modifiers.indexOf(1) > -1;
        var newPair = void 0;
        if (isStatic) {
            pairs[keyNode.Value] = pair;
        }
        else {
            newPair = { Key: copyObject(keyNode), Value: copyObject(valueNode, objectContext) };
            if (pair.modifiers) {
                newPair.modifiers = pair.modifiers;
            }
            pairs[keyNode.Value] = newPair;
        }
    }
    return (objectContext = new object_1.Hash(pairs));
}
exports.copyHashMap = copyHashMap;
// junk
function copyObject(valueNode, objectContext) {
    switch (valueNode.Type()) {
        case "boolean":
            return new object_1.BooleanObject(valueNode.Value);
        case "int":
            return new object_1.Integer(valueNode.Value);
        case "float":
            return new object_1.Float(valueNode.Value);
        case "string":
            return new object_1.StringObject(valueNode.Value);
        case "array":
            return exports.copyList(valueNode);
        case "hash":
            return copyHashMap(valueNode);
        case "function":
            return copyFunction(valueNode, objectContext);
        case "BUILTIN":
            return valueNode;
        default:
            return object_1.NULL;
    }
}
exports.copyObject = copyObject;
function copyFunction(fn, objectContext) {
    return new object_1.LambdaFunction(fn.Parameters, fn.Body, fn.Env, fn.Fn, objectContext, fn.ReturnType, fn.ParameterTypes);
}
exports.copyFunction = copyFunction;
function assignHashPairs(from, to) {
    var fromPairs = from.Pairs, toPairs = to.Pairs, keys = Object.keys(fromPairs);
    for (var x = 0, l = keys.length; x < l; x++) {
        var key = keys[x];
        toPairs[key] = { Key: new String(key), Value: copyObject(fromPairs[key].Value) };
    }
    return to;
}
exports.assignHashPairs = assignHashPairs;
exports.nativeValueToECSValue = function (element) {
    if (element === null || element === undefined) {
        return object_1.NULL;
    }
    switch (typeof element) {
        case "string":
            return new object_1.StringObject(element);
        case "number":
            return element % 1 == 0 ? new object_1.Integer(element) : new object_1.Float(element);
        case "boolean":
            return new object_1.BooleanObject(element);
        case "object":
            if (typeof element.length == "number") {
                return exports.nativeListToArray(element);
            }
            else {
                return exports.nativeObjToMap(element);
            }
        case "function":
            // this could work actually
            return object_1.NULL;
        default:
            return object_1.NULL;
    }
};
exports.nativeListToArray = function (obj) {
    return new object_1.ArrayObject(nativeListToListOfObjects(obj));
};
function listOfObjectsToNativeList(obj, opt) {
    var out = [];
    for (var elem = 0, len = obj.length; elem < len; elem++) {
        out[elem] = exports.ecsObjectToNativeObject(obj[elem], opt);
    }
    return out;
}
exports.listOfObjectsToNativeList = listOfObjectsToNativeList;
function nativeListToListOfObjects(obj) {
    var out = [];
    for (var elem = 0, len = obj.length; elem < len; elem++) {
        out[elem] = exports.nativeValueToECSValue(obj[elem]);
    }
    return out;
}
exports.nativeListToListOfObjects = nativeListToListOfObjects;
function numberArrayToFloatArray(numbers) {
    var elems = [], arr = new object_1.ArrayObject(elems);
    for (var n = 0, len = numbers.length; n < len; n++) {
        elems.push(new object_1.Float(numbers[n]));
    }
    return arr;
}
exports.numberArrayToFloatArray = numberArrayToFloatArray;
exports.nativeObjToMap = function (obj) {
    if (obj === void 0) { obj = {}; }
    var map = new object_1.Hash({});
    for (var objectKey in obj) {
        var value = exports.nativeValueToECSValue(obj[objectKey]);
        map.Pairs[objectKey] = {
            Key: new object_1.StringObject(objectKey),
            Value: value
        };
    }
    return map;
};
exports.assignNativeProperties = function (to, from) {
    var keys = Object.keys(from);
    for (var x = 0, l = keys.length; x < l; x++) {
        var key = keys[x];
        to.Pairs[key] = { Key: new object_1.StringObject(key), Value: exports.nativeValueToECSValue(from[key]) };
    }
};
exports.assignNativePropertiesToJSObj = function (to, from) {
    var keys = Object.keys(from);
    for (var x = 0, l = keys.length; x < l; x++) {
        var key = keys[x];
        to.Pairs[key] = from[key];
    }
};
exports.ecsObjectToNativeObject = function (obj, opt, env) {
    switch (obj.Type()) {
        case object_1.STRING_OBJ:
        case object_1.INTEGER_OBJ:
        case object_1.FLOAT_OBJ:
        case object_1.BOOLEAN_OBJ:
            return obj.Inspect();
        case object_1.HASH_OBJ:
            return exports.objectToNativeObject(obj);
        case object_1.ARRAY_OBJ:
            return exports.arrayToNativeList(obj, opt, env);
        case "function":
            return opt ? opt.compileFunction(obj, env) : null;
        case object_1.ERROR_OBJ:
            return obj.Inspect();
        default:
    }
};
exports.objectToNativeObject = function (map) {
    var obj = {}, pairs = map.Pairs;
    for (var objectKey in pairs) {
        var data = pairs[objectKey].Value;
        obj[objectKey] = exports.ecsObjectToNativeObject(data);
    }
    return obj;
};
exports.arrayToNativeList = function (arr, opt, env) {
    return arr.Elements.map(function (val) { return (exports.ecsObjectToNativeObject(val, opt, env)); });
};
exports.nativePairValue = function (hash, key) { return hash.Pairs[key].Value.Inspect(); };
exports.makeEventSource = function (prefix, event) {
    return [event, new object_1.Builtin(prefix + "." + event, [], function (context, scope, jsScope) {
            var args = [];
            for (var _i = 3; _i < arguments.length; _i++) {
                args[_i - 3] = arguments[_i];
            }
            var callback = args[0];
            context.env.addEventHandler(prefix, event, callback);
            context.postMessage("add-event-handler", { system: prefix, method: event });
            return object_1.NULL;
        }, null)];
};
exports.makeEventSink = function (prefix, event) {
    return [event, new object_1.Builtin(prefix + "." + event, [], function (context, scope, jsScope) {
            var args = [];
            for (var _i = 3; _i < arguments.length; _i++) {
                args[_i - 3] = arguments[_i];
            }
            // context.env.emit event or something??/
            context.postMessage("action", { system: prefix, method: event, args: [nativeListToListOfObjects(args)] });
            return object_1.NULL;
        }, null)];
};
