"use strict";
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
Object.defineProperty(exports, "__esModule", { value: true });
var token_1 = require("../token");
var token_2 = require("../token");
var obj_util_1 = require("./obj-util");
var builtin_util_1 = require("../builtins/builtin-util");
var INTEGER_OBJ = "int", FLOAT_OBJ = "float", BOOLEAN_OBJ = "boolean", NULL_OBJ = "null", RETURN_VALUE_OBJ = "RETURN_VALUE", ERROR_OBJ = "error", FUNCTION_OBJ = "function", PURE_FUNCTION_OBJ = "pure_function", STREAM_OBJ = "stream", STRING_OBJ = "string", BUILTIN_OBJ = "BUILTIN", ARRAY_OBJ = "array", HASH_OBJ = "hash";
exports.INTEGER_OBJ = INTEGER_OBJ;
exports.FLOAT_OBJ = FLOAT_OBJ;
exports.BOOLEAN_OBJ = BOOLEAN_OBJ;
exports.NULL_OBJ = NULL_OBJ;
exports.RETURN_VALUE_OBJ = RETURN_VALUE_OBJ;
exports.ERROR_OBJ = ERROR_OBJ;
exports.FUNCTION_OBJ = FUNCTION_OBJ;
exports.PURE_FUNCTION_OBJ = PURE_FUNCTION_OBJ;
exports.STRING_OBJ = STRING_OBJ;
exports.BUILTIN_OBJ = BUILTIN_OBJ;
exports.ARRAY_OBJ = ARRAY_OBJ;
exports.HASH_OBJ = HASH_OBJ;
var Null = /** @class */ (function () {
    function Null() {
        this.Value = null;
    }
    Null.prototype.Type = function () { return NULL_OBJ; };
    Null.prototype.Inspect = function () { return null; };
    return Null;
}());
exports.Null = Null;
var Integer = /** @class */ (function () {
    function Integer(Value) {
        this.Value = Value;
    }
    Integer.prototype.Inspect = function () { return this.Value; };
    Integer.prototype.Type = function () { return INTEGER_OBJ; };
    Integer.prototype.HashKey = function () { return "" + this.Value; };
    return Integer;
}());
exports.Integer = Integer;
var Float = /** @class */ (function () {
    function Float(Value) {
        this.Value = Value;
    }
    Float.prototype.Inspect = function () { return this.Value; };
    Float.prototype.Type = function () { return FLOAT_OBJ; };
    Float.prototype.HashKey = function () { return "" + this.Value; };
    return Float;
}());
exports.Float = Float;
var BooleanObject = /** @class */ (function () {
    function BooleanObject(Value) {
        this.Value = Value;
    }
    BooleanObject.prototype.Type = function () { return BOOLEAN_OBJ; };
    BooleanObject.prototype.Inspect = function () { return this.Value; };
    BooleanObject.prototype.HashKey = function () { return "" + (this.Value ? 1 : 0); };
    return BooleanObject;
}());
exports.BooleanObject = BooleanObject;
var ReturnValue = /** @class */ (function () {
    function ReturnValue(Value) {
        this.Value = Value;
    }
    ReturnValue.prototype.Type = function () { return RETURN_VALUE_OBJ; };
    ReturnValue.prototype.Inspect = function () { return this.Value.Inspect(); };
    return ReturnValue;
}());
exports.ReturnValue = ReturnValue;
var LambdaFunction = /** @class */ (function () {
    function LambdaFunction(Parameters, Body, Env, Fn, ObjectContext, ReturnType, ParameterTypes, stateful) {
        if (stateful === void 0) { stateful = null; }
        this.Parameters = Parameters;
        this.Body = Body;
        this.Env = Env;
        this.Fn = Fn;
        this.ObjectContext = ObjectContext;
        this.ReturnType = ReturnType;
        this.ParameterTypes = ParameterTypes;
        this.stateful = stateful;
    }
    LambdaFunction.prototype.Type = function () { return FUNCTION_OBJ; };
    LambdaFunction.prototype.Inspect = function (indentLevel) {
        if (indentLevel === void 0) { indentLevel = 1; }
        var out = "", params = [];
        for (var _i = 0, _a = this.Parameters; _i < _a.length; _i++) {
            var p = _a[_i];
            params.push(p.String());
        }
        if (this.ObjectContext && this.ObjectContext.className) {
            out += "{" + this.ObjectContext.className + "} ";
        }
        var indentation = "    ".repeat(indentLevel), outerIndentation = "    ".repeat(indentLevel - 1);
        var strParams = [];
        if (this.ParameterTypes && this.ParameterTypes.length > 0) {
            for (var p = 0, l = this.ParameterTypes.length; p < l; p++) {
                strParams.push(this.ParameterTypes[p] + " " + params[p]);
            }
        }
        else {
            strParams = params;
        }
        var joinedParams = strParams.join(", ");
        out += ((this.ReturnType ? this.ReturnType + " " : "") + ("fn(" + joinedParams + ")") + (this.ReturnType ? this.ReturnType : ""));
        var body = this.Body.String()
            .split("\n")
            .map(function (v) { return indentation + v; })
            .join("\n");
        out += this.Body.Statements.length > 0 ? "\n" + outerIndentation + "{\n" + body + "\n" + outerIndentation + "}" : "{}";
        return out;
    };
    return LambdaFunction;
}());
exports.LambdaFunction = LambdaFunction;
var PureFunction = /** @class */ (function () {
    function PureFunction(Parameters, Body, Env, Fn, ReturnType, ParameterTypes) {
        this.Parameters = Parameters;
        this.Body = Body;
        this.Env = Env;
        this.Fn = Fn;
        this.ReturnType = ReturnType;
        this.ParameterTypes = ParameterTypes;
    }
    PureFunction.prototype.Type = function () { return PURE_FUNCTION_OBJ; };
    PureFunction.prototype.Inspect = function (indentLevel) {
        if (indentLevel === void 0) { indentLevel = 1; }
        var out = "", params = [];
        for (var _i = 0, _a = this.Parameters; _i < _a.length; _i++) {
            var p = _a[_i];
            params.push(p.String());
        }
        var indentation = "    ".repeat(indentLevel), outerIndentation = "    ".repeat(indentLevel - 1);
        var strParams = [];
        if (this.ParameterTypes && this.ParameterTypes.length > 0) {
            for (var p = 0, l = this.ParameterTypes.length; p < l; p++) {
                strParams.push(this.ParameterTypes[p] + " " + params[p]);
            }
        }
        else {
            strParams = params;
        }
        var joinedParams = strParams.join(", ");
        out += (("fn(" + joinedParams + ")") + (this.ReturnType ? this.ReturnType : ""));
        var body = this.Body.String()
            .split("\n")
            .map(function (v) { return indentation + v; })
            .join("\n");
        out += this.Body.Statements.length > 0 ? "\n" + outerIndentation + "{\n" + body + "\n" + outerIndentation + "}" : "{}";
        return out;
    };
    return PureFunction;
}());
exports.PureFunction = PureFunction;
var Builtin = /** @class */ (function () {
    function Builtin(name, signature, Fn, Context, ObjectContext, stateful, ecsOnly // for builtin classes like Math
    ) {
        if (ecsOnly === void 0) { ecsOnly = false; }
        this.name = name;
        this.signature = signature;
        this.Fn = Fn;
        this.Context = Context;
        this.ObjectContext = ObjectContext;
        this.stateful = stateful;
        this.ecsOnly = ecsOnly;
        this.builtin = true;
    }
    Builtin.prototype.callFromECSRuntime = function (context, scope, optimizer, params) {
        var err = builtin_util_1.assertBuiltinArgs(params, this.signature.length, null, this.name, this.signature);
        if (err) {
            return err;
        }
        var result = this.Fn.apply(this, __spreadArrays([
            // Runtime Context
            context,
            // Object context
            scope,
            // JS Builtins (maybe it could be combined with `context` )
            (scope && scope.builtins) ? scope.builtins : null], (obj_util_1.listOfObjectsToNativeList(params, optimizer))));
        if (result && typeof result.Type === "function") {
            return result;
        }
        return result ? obj_util_1.nativeValueToECSValue(result) : exports.NULL;
    };
    Builtin.prototype.Type = function () { return BUILTIN_OBJ; };
    Builtin.prototype.Inspect = function (indentLevel) {
        var functionSig = this.Fn.toString().split("\n")[0].split("jsScope, ")[1];
        return 'builtin fn' + (indentLevel
            ? "(" + (functionSig
                ? this.decorateSignature(functionSig.split(")")[0])
                : "")
                + ")"
            : "(" + this.Fn.toString() + ")");
    };
    Builtin.prototype.decorateSignature = function (sig) {
        var splitStr = sig.split(",");
        if (splitStr.length > 1) {
            var out = "";
            for (var idx = 0, len = splitStr.length; idx < len; idx++) {
                var paramType = (this.signature && this.signature[idx]) ? this.signature[idx] + " " : "";
                out += paramType.replace(",", " | ") + splitStr[idx];
                if (idx < len - 1) {
                    out += ", ";
                }
            }
            return out;
        }
        else {
            return ((this.signature && this.signature.length) ? (this.signature[0] + " ").replace(",", " | ") : "") + splitStr[0];
        }
    };
    return Builtin;
}());
exports.Builtin = Builtin;
var Stream = /** @class */ (function () {
    function Stream(Direction, Transforms, Source, Sink, ReturnType) {
        if (Transforms === void 0) { Transforms = []; }
        this.Direction = Direction;
        this.Transforms = Transforms;
        this.Source = Source;
        this.Sink = Sink;
        this.ReturnType = ReturnType;
    }
    Stream.prototype.Type = function () { return STREAM_OBJ; };
    Stream.prototype.Inspect = function (indentLevel) {
        if (indentLevel === void 0) { indentLevel = 1; }
        var out = (this.ReturnType ? this.ReturnType : "");
        out += ((this.Direction + ("" + this.Source.Inspect())) +
            this.Transforms.map(function (t) { return t.Inspect(); }).join(this.Direction == token_1.STREAM_DIRECTION.READ ? token_2.SOURCE : token_2.SINK));
        return out + " " + this.Sink.Inspect();
    };
    return Stream;
}());
exports.Stream = Stream;
var StringObject = /** @class */ (function () {
    function StringObject(Value) {
        this.Value = Value;
    }
    StringObject.prototype.Type = function () { return STRING_OBJ; };
    StringObject.prototype.Inspect = function () { return this.Value; };
    StringObject.prototype.HashKey = function () { return "" + this.Value.substr(0, 16); };
    return StringObject;
}());
exports.StringObject = StringObject;
var ArrayObject = /** @class */ (function () {
    function ArrayObject(Elements) {
        this.Elements = Elements;
    }
    ArrayObject.prototype.Type = function () { return ARRAY_OBJ; };
    ArrayObject.prototype.Inspect = function () {
        var out = "", elements = [];
        this.Elements.forEach(function (e) {
            elements.push(e.Inspect ? e.Inspect() : null);
        });
        var tableMode = this.Elements.length > 0 && this.Elements[0].Type() == STRING_OBJ && this.Elements[0].Value.length > 16;
        out += "[" + (tableMode ? "\n" : "");
        out += (tableMode ? "     " : "") + elements.join(tableMode ? ",\n     " : ", ");
        out += (tableMode ? "\n" : "") + "]";
        return out;
    };
    return ArrayObject;
}());
exports.ArrayObject = ArrayObject;
var ErrorObject = /** @class */ (function () {
    function ErrorObject(Message) {
        this.Message = Message;
    }
    ErrorObject.prototype.Type = function () { return ERROR_OBJ; };
    ErrorObject.prototype.Inspect = function () { return "💀  RUNTIME ERROR: " + this.Message; };
    return ErrorObject;
}());
exports.ErrorObject = ErrorObject;
var Hash = /** @class */ (function () {
    function Hash(Pairs, Constructor, className, builtins) {
        if (builtins === void 0) { builtins = {}; }
        this.Pairs = Pairs;
        this.Constructor = Constructor;
        this.className = className;
        this.builtins = builtins;
    }
    Hash.prototype.Type = function () { return HASH_OBJ; };
    Hash.prototype.Inspect = function (indentLevel) {
        var _this = this;
        if (indentLevel === void 0) { indentLevel = 1; }
        var out = "";
        out += this.className ? this.className + " " : "";
        out += "{\n";
        var allPairs = Object.keys(this.Pairs);
        var len = allPairs.length;
        allPairs.forEach(function (key, idx) {
            var pair = _this.Pairs[key];
            var modifierStr = "";
            if (pair.modifiers) {
                for (var m = 0, l = pair.modifiers.length; m < l; m++) {
                    modifierStr += token_1.modifierNames[m] + " ";
                }
            }
            out += "    ".repeat(indentLevel) + (modifierStr) + '"' + key + '": ';
            var isString = false;
            if (pair.Value.Type() == STRING_OBJ) {
                out += '"';
                isString = true;
            }
            var comma = idx < len - 1 ? "," : "";
            out += pair.Value.Inspect(indentLevel + 1) + comma;
            if (isString) {
                out += '"';
            }
            out += "\n";
        });
        out += "}";
        return out;
    };
    return Hash;
}());
exports.Hash = Hash;
exports.strBuiltin = new StringObject("builtin");
exports.TRUE = new BooleanObject(true);
exports.FALSE = new BooleanObject(false);
exports.NULL = new Null();
