import * as token from "./token";
import * as lexer from "./lexer";
import * as ast from "./ast";
import * as statement from "./ast/statement";
import * as expressions from "./ast/expression";
export interface ParseTreeAnalysis {
    hasLoops: boolean;
    hasRecursion: boolean;
    declaredVariables: {
        [identName: string]: string;
    };
    undeclaredVariables: {
        [identName: string]: boolean;
    };
}
export interface ParseResult {
    tree: ast.Program;
    analysis: ParseTreeAnalysis;
}
export declare type prefixParseFn = (exp?: expressions.Expression, classMap?: boolean, returnType?: string) => expressions.Expression;
export declare type infixParseFn = (exp?: expressions.Expression) => expressions.Expression;
export declare type AnyStatement = ast.Statement | statement.AssignmentStatement | statement.LetStatement | statement.ReturnStatement | statement.ExpressionStatement;
export declare const LOWEST = 0, LOGICAL = 1, EQUALS = 2, LESSGREATER = 3, SUM = 4, PRODUCT = 5, PREFIX = 6, CALL = 7, STREAM = 8, INDEX = 9;
export declare const precedences: {
    [tokenType: string]: number;
};
export declare class Parser {
    diagnosticContext: ParseTreeAnalysis;
    private l;
    private curToken;
    private peekToken;
    private errors;
    private prefixParseFns;
    private infixParseFns;
    constructor(l: lexer.Lexer);
    private resetDiagnosticContext;
    parseProgram(): ast.Program;
    parse(): void;
    private curTokenIs;
    private peekTokenIs;
    private peekPrecedence;
    private curPrecedence;
    nextToken(): void;
    private registerPrefix;
    private registerInfix;
    /**
     * parseExpression
     * @argument {number} precedence
     * @argument {boolean} identAsString
     * @returns {expressions.Expression}
     */
    private parseExpression;
    private parseDeclaration;
    /**
     * parseExpressionWithModifiers
     * @argument {number} precedence
     *
     * handles prefixes private, static and final
     */
    private parseExpressionWithModifiers;
    private parseDataType;
    private parsePrefixExpression;
    private parseInfixExpression;
    private parseIfStatement;
    private parseForStatement;
    private parseWhileStatement;
    private parseSleepStatement;
    parseGroupedExpression: () => expressions.Expression;
    private parseCallExpression;
    private parseReadStreamExpression;
    private parseWriteStreamExpression;
    private parseIndexExpression;
    private parseDotIndexExpression;
    private parseCommentBlock;
    parseStatement: () => AnyStatement;
    parseExpressionStatement: () => statement.ExpressionStatement;
    parseNewExpression: () => expressions.Expression;
    parseExecExpression: () => expressions.Expression;
    parseAssignmentStatement: () => statement.AssignmentStatement;
    parseLetStatement: (_?: any, _2?: any, pure?: boolean) => statement.LetStatement;
    private handleMultipleVarDeclaration;
    parseClassStatement: () => statement.ClassStatement;
    parseConceptStatement: () => statement.ConceptStatement;
    parseReturnStatement: () => statement.ReturnStatement;
    parseBlockStatement: (checkForReturnStatement?: boolean) => statement.BlockStatement;
    private parseIdentifier;
    private parseBoolean;
    private parseIntegerLiteral;
    private parseFloatLiteral;
    private parseFunctionLiteral;
    private parsePureFunctionLiteral;
    private parseHashLiteral;
    private parseFunctionParameters;
    private parseStringLiteral;
    private parseArrayLiteral;
    private parseExpressionList;
    private expectPeek;
    private checkDataType;
    private isDataType;
    private isCustomDataType;
    private lineNumber;
    Errors(): string[];
    private noPrefixParseFnError;
    peekError(expectedType: token.TokenType): void;
    private parseError;
}
