"use strict";
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
var token = require("./token");
var token_1 = require("./token");
var ast = require("./ast");
var statement = require("./ast/statement");
var expressions = require("./ast/expression");
var literal = require("./ast/literal");
var token_2 = require("./token");
var util_1 = require("./ubiquitous/util");
var util_2 = require("./ast/util");
var util_3 = require("./ast/util");
exports.LOWEST = 0, exports.LOGICAL = 1, exports.EQUALS = 2, exports.LESSGREATER = 3, exports.SUM = 4, exports.PRODUCT = 5, exports.PREFIX = 6, exports.CALL = 7, exports.STREAM = 8, exports.INDEX = 9;
exports.precedences = (_a = {},
    _a[token.EQ] = exports.EQUALS,
    _a[token.NOT_EQ] = exports.EQUALS,
    _a[token.AND] = exports.LOGICAL,
    _a[token.OR] = exports.LOGICAL,
    _a[token.TYPEOF] = exports.LESSGREATER,
    _a[token.LT] = exports.LESSGREATER,
    _a[token.GT] = exports.LESSGREATER,
    _a[token.PLUS] = exports.SUM,
    _a[token.MINUS] = exports.SUM,
    _a[token.SLASH] = exports.PRODUCT,
    _a[token.ASTERISK] = exports.PRODUCT,
    _a[token.MOD] = exports.PRODUCT,
    _a[token.DOT] = exports.INDEX,
    _a[token.LPAREN] = exports.CALL,
    _a[token.SOURCE] = exports.STREAM,
    _a[token.SINK] = exports.STREAM,
    _a[token.LBRACKET] = exports.INDEX,
    _a);
var streamTransformErr = " Parse Error; invalid element in stream transform";
var Parser = /** @class */ (function () {
    function Parser(l) {
        var _this = this;
        this.curToken = null;
        this.peekToken = null;
        this.prefixParseFns = {};
        this.infixParseFns = {};
        /**
         * parseExpression
         * @argument {number} precedence
         * @argument {boolean} identAsString
         * @returns {expressions.Expression}
         */
        this.parseExpression = function (precedence, declaration) {
            var curTokenType = _this.curToken.Type;
            var prefix = _this.prefixParseFns[curTokenType], leftExp;
            if (declaration) {
                leftExp = _this.parseDeclaration(curTokenType);
            }
            else {
                leftExp = prefix && prefix();
            }
            while (!_this.peekTokenIs(token.SEMICOLON) && precedence < _this.peekPrecedence()) {
                var infix = _this.infixParseFns[_this.peekToken.Type];
                if (!infix) {
                    if (!leftExp) {
                        _this.noPrefixParseFnError(curTokenType);
                        return null;
                    }
                    return leftExp;
                }
                _this.nextToken();
                leftExp = infix(leftExp);
            }
            return leftExp;
        };
        this.parseDeclaration = function (curTokenType) {
            var pure = false;
            if (curTokenType === token.PURE) {
                var dataType = _this.checkDataType();
                pure = true;
                if (_this.peekTokenIs(token.FUNCTION) || dataType) {
                    if (dataType) {
                        if (!_this.peekTokenIs(token.FUNCTION)) {
                            _this.noPrefixParseFnError(_this.peekToken.Literal);
                        }
                        _this.nextToken();
                    }
                }
                return _this.parseFunctionLiteral(null, null, dataType, pure);
            }
        };
        /**
         * parseExpressionWithModifiers
         * @argument {number} precedence
         *
         * handles prefixes private, static and final
         */
        this.parseExpressionWithModifiers = function (precedence) {
            var prefix, leftExp, modifiers = [], modifier = -1;
            modifier = token_1.modifierTypes.indexOf(_this.curToken.Type);
            while (modifier > -1) {
                modifiers.push(modifier);
                _this.nextToken();
                modifier = token_1.modifierTypes.indexOf(_this.curToken.Type);
            }
            _this.parseDataType(false);
            var curTokenType = _this.curToken.Type;
            prefix = _this.prefixParseFns[curTokenType];
            if (prefix == null) {
                _this.noPrefixParseFnError(curTokenType);
                return null;
            }
            if (curTokenType === "IDENT") {
                prefix = _this.parseStringLiteral;
                leftExp = prefix();
                if (modifiers.length > 0) {
                    leftExp.modifiers = modifiers;
                }
            }
            else {
                leftExp = prefix();
            }
            while (!_this.peekTokenIs(token.SEMICOLON) && precedence < _this.peekPrecedence()) {
                var infix = _this.infixParseFns[_this.peekToken.Type];
                if (infix == null) {
                    return leftExp;
                }
                _this.nextToken();
                leftExp = infix(leftExp);
            }
            return leftExp;
        };
        this.parsePrefixExpression = function () {
            var expression = new expressions.PrefixExpression(_this.curToken, _this.curToken.Literal);
            _this.nextToken();
            expression.Right = _this.parseExpression(exports.PREFIX);
            return expression;
        };
        this.parseInfixExpression = function (left) {
            var expression = new expressions.InfixExpression(_this.curToken, left, _this.curToken.Literal), precedence = _this.curPrecedence();
            _this.nextToken();
            expression.Right = _this.parseExpression(precedence);
            return expression;
        };
        this.parseIfStatement = function () {
            var expression = new statement.IfStatement(_this.curToken);
            if (!_this.expectPeek(token.LPAREN)) {
                return null;
            }
            _this.nextToken();
            expression.Condition = _this.parseExpression(exports.LOWEST);
            if (!_this.expectPeek(token.RPAREN)) {
                return null;
            }
            if (!_this.expectPeek(token.LBRACE)) {
                return null;
            }
            expression.Consequence = _this.parseBlockStatement();
            if (_this.peekTokenIs(token.ELSE)) {
                _this.nextToken();
                if (!_this.expectPeek(token.LBRACE)) {
                    return null;
                }
                expression.Alternative = _this.parseBlockStatement();
            }
            return expression;
        };
        this.parseForStatement = function () {
            _this.diagnosticContext.hasLoops = true;
            var expression = new statement.ForStatement(_this.curToken);
            if (!_this.expectPeek(token.LPAREN)) {
                return null;
            }
            if (!_this.expectPeek(token.IDENT)) {
                return null;
            }
            expression.Element = new literal.Identifier(_this.curToken, _this.curToken.Literal);
            var declaredVars = _this.diagnosticContext.declaredVariables;
            if (!declaredVars[_this.curToken.Literal]) {
                declaredVars[_this.curToken.Literal] = "int"; // todo, check range type here
            }
            if (!_this.expectPeek(token.COMMA)) {
                return null;
            }
            _this.nextToken();
            expression.Range = _this.parseExpression(exports.LOWEST);
            if (expression.Range.NodeName == "Identifier") {
                if (!_this.diagnosticContext.declaredVariables[expression.Range.Value]) {
                    _this.diagnosticContext.undeclaredVariables[expression.Range.Value] = true;
                }
            }
            if (!_this.expectPeek(token.RPAREN)) {
                return null;
            }
            if (!_this.expectPeek(token.LBRACE)) {
                return null;
            }
            expression.Consequence = _this.parseBlockStatement();
            return expression;
        };
        this.parseWhileStatement = function () {
            _this.diagnosticContext.hasLoops = true;
            var expression = new statement.WhileStatement(_this.curToken);
            if (!_this.expectPeek(token.LPAREN)) {
                return null;
            }
            _this.nextToken();
            expression.Condition = _this.parseExpression(exports.LOWEST);
            if (!_this.expectPeek(token.RPAREN)) {
                return null;
            }
            if (!_this.expectPeek(token.LBRACE)) {
                return null;
            }
            expression.Consequence = _this.parseBlockStatement();
            return expression;
        };
        this.parseSleepStatement = function () {
            var expression = new statement.SleepStatement(_this.curToken);
            if (!_this.expectPeek(token.LPAREN)) {
                return null;
            }
            _this.nextToken();
            expression.Duration = _this.parseExpression(exports.LOWEST);
            if (!_this.expectPeek(token.RPAREN)) {
                return null;
            }
            if (!_this.expectPeek(token.LBRACE)) {
                return null;
            }
            expression.Consequence = _this.parseBlockStatement();
            return expression;
        };
        this.parseGroupedExpression = function () {
            _this.nextToken();
            var exp = _this.parseExpression(exports.LOWEST);
            if (!_this.expectPeek(token.RPAREN)) {
                return null;
            }
            return exp;
        };
        this.parseCallExpression = function (fun) {
            var curToken = _this.curToken, exp = new expressions.CallExpression(curToken, fun);
            exp.Arguments = _this.parseExpressionList(token.RPAREN);
            return exp;
        };
        this.parseReadStreamExpression = function (source) {
            var curToken = _this.curToken, exp = new expressions.StreamExpression(curToken, token_2.STREAM_DIRECTION.READ);
            exp.Source = source;
            while (_this.curToken.Type == token.SOURCE) {
                _this.nextToken();
                if (!_this.curTokenIs(token.IDENT) && !_this.curTokenIs(token.FUNCTION)) {
                    _this.parseError(streamTransformErr);
                    return null;
                }
                exp.Transforms.push(_this.parseExpression(exports.LOWEST));
                _this.nextToken();
            }
            exp.Sink = exp.Transforms.pop();
            return exp;
        };
        this.parseWriteStreamExpression = function (sink) {
            var curToken = _this.curToken, exp = new expressions.StreamExpression(curToken, token_2.STREAM_DIRECTION.WRITE);
            exp.Sink = sink;
            while (_this.curToken.Type == token.SINK) {
                _this.nextToken();
                if (!_this.curTokenIs(token.IDENT) && !_this.curTokenIs(token.FUNCTION)) {
                    _this.parseError(streamTransformErr);
                    return null;
                }
                exp.Transforms.push(_this.parseExpression(exports.LOWEST));
                _this.nextToken();
            }
            exp.Source = exp.Transforms.pop();
            return exp;
        };
        this.parseIndexExpression = function (left) {
            var exp = null;
            var bracketAndLeft = [_this.curToken, left];
            _this.nextToken();
            var Index = _this.parseExpression(exports.LOWEST);
            if (!_this.expectPeek(token.RBRACKET)) {
                return null;
            }
            if (!_this.peekTokenIs(token.ASSIGN)) {
                exp = new expressions.IndexExpression(bracketAndLeft[0], bracketAndLeft[1]);
            }
            else {
                exp = new expressions.IndexAssignmentExpression(bracketAndLeft[0], bracketAndLeft[1]);
                _this.nextToken();
                _this.nextToken();
                exp.Assignment = _this.parseExpression(exports.LOWEST);
            }
            exp.Index = Index;
            return exp;
        };
        this.parseDotIndexExpression = function (left) {
            var exp = null, Index = null;
            var bracketAndLeft = [_this.curToken, left];
            if (_this.peekTokenIs(token.IDENT)) {
                _this.nextToken();
                var identValue = _this.curToken.Literal;
                Index = new literal.StringLiteral({ Type: token.STRING, Literal: identValue }, identValue);
            }
            else {
                Index = _this.parseExpression(exports.LOWEST);
            }
            if (!_this.peekTokenIs(token.ASSIGN)) {
                exp = new expressions.IndexExpression(bracketAndLeft[0], bracketAndLeft[1]);
            }
            else {
                exp = new expressions.IndexAssignmentExpression(bracketAndLeft[0], bracketAndLeft[1]);
                _this.nextToken();
                _this.nextToken();
                exp.Assignment = _this.parseExpression(exports.LOWEST);
            }
            exp.Index = Index;
            return exp;
        };
        this.parseCommentBlock = function () {
            var endComment = false;
            while (!endComment) {
                _this.nextToken();
                endComment = _this.curToken.Type === token.RCOMMENT;
            }
        };
        this.parseStatement = function () {
            if (_this.curToken.Type == token.IDENT && _this.peekTokenIs(token.ASSIGN)) {
                return _this.parseAssignmentStatement();
            }
            if (_this.isCustomDataType() && _this.peekTokenIs(token.IDENT)) {
                return _this.parseLetStatement();
            }
            else {
                switch (_this.curToken.Type) {
                    case token.RETURN:
                        return _this.parseReturnStatement();
                    case token.CONST:
                    case token.LET:
                    case token.LET_BOOL:
                    case token.LET_INT:
                    case token.LET_FLOAT:
                    case token.LET_STRING:
                    case token.LET_ARRAY:
                    case token.LET_OBJECT:
                    case token.LET_FUNCTION:
                        return _this.parseLetStatement();
                    case token.PURE:
                        return _this.parseLetStatement(null, null, true);
                    case token.CONCEPT:
                        return _this.parseConceptStatement();
                    case token.CLASS:
                        return _this.parseClassStatement();
                    case token.IF:
                        return _this.parseIfStatement();
                    case token.WHILE:
                        return _this.parseWhileStatement();
                    case token.FOR:
                        return _this.parseForStatement();
                    case token.SLEEP:
                        return _this.parseSleepStatement();
                    default:
                        return _this.parseExpressionStatement();
                }
            }
        };
        this.parseExpressionStatement = function () {
            var curToken = _this.curToken, stmt = new statement.ExpressionStatement(curToken);
            stmt.Expression = _this.parseExpression(exports.LOWEST);
            if (!stmt.Expression) {
                return null;
            }
            if (_this.peekTokenIs(token.SEMICOLON)) {
                _this.nextToken();
            }
            return stmt;
        };
        this.parseNewExpression = function () {
            var exp = new expressions.NewExpression(_this.curToken);
            if (!_this.expectPeek(token.IDENT)) {
                return null;
            }
            exp.Name = new literal.Identifier(_this.curToken, _this.curToken.Literal);
            return exp;
        };
        this.parseExecExpression = function () {
            var exp = new expressions.ExecExpression(_this.curToken);
            if (!_this.expectPeek(token.LBRACKET)) {
                return null;
            }
            exp.Name = _this.parseArrayLiteral();
            if (!_this.expectPeek(token.LBRACE)) {
                return null;
            }
            exp.Callback = _this.parseBlockStatement();
            return exp;
        };
        this.parseAssignmentStatement = function () {
            var stmt = new statement.AssignmentStatement(new literal.Identifier(_this.curToken, _this.curToken.Literal));
            if (!_this.expectPeek(token.ASSIGN)) {
                return null;
            }
            _this.nextToken();
            stmt.Value = _this.parseExpression(exports.LOWEST);
            if (_this.peekTokenIs(token.SEMICOLON)) {
                _this.nextToken();
            }
            return stmt;
        };
        this.parseLetStatement = function (_, _2, pure) {
            var stmt = new statement.LetStatement(_this.curToken);
            stmt.dataType = _this.checkDataType(false);
            if (!_this.expectPeek(token.IDENT)) {
                return null;
            }
            stmt.Name = new literal.Identifier(_this.curToken, _this.curToken.Literal);
            if (!_this.peekTokenIs(token.ASSIGN)) {
                stmt.Value = util_3.getDefaultValueNodeForDataType(stmt.dataType);
            }
            else {
                _this.nextToken();
                _this.nextToken();
                _this.diagnosticContext.declaredVariables[stmt.Name.Value] = "any";
                stmt.Value = _this.parseExpression(exports.LOWEST, pure);
            }
            _this.diagnosticContext.declaredVariables[stmt.Name.Value] = stmt.dataType
                || util_2.getDataTypeByNodeName(stmt.Value);
            if (_this.peekTokenIs(token.SEMICOLON)) {
                _this.nextToken();
            }
            if (_this.peekTokenIs(token.COMMA)) {
                return _this.handleMultipleVarDeclaration(stmt, pure);
            }
            return stmt;
        };
        this.parseClassStatement = function () {
            var stmt = new statement.ClassStatement(_this.curToken);
            if (!_this.expectPeek(token.IDENT)) {
                return null;
            }
            stmt.Name = new literal.Identifier(_this.curToken, _this.curToken.Literal);
            _this.diagnosticContext.declaredVariables[stmt.Name.Value] = stmt.Name.Value;
            _this.nextToken();
            stmt.Value = _this.parseHashLiteral(true);
            if (_this.peekTokenIs(token.SEMICOLON)) {
                _this.nextToken();
            }
            return stmt;
        };
        this.parseConceptStatement = function () {
            var conceptType;
            var stmt = new statement.ConceptStatement(_this.curToken, conceptType);
            if (!_this.expectPeek(token.IDENT)) {
                return null;
            }
            //
            stmt.Name = new literal.Identifier(_this.curToken, _this.curToken.Literal);
            _this.diagnosticContext.declaredVariables[stmt.Name.Value] = stmt.Name.Value;
            _this.nextToken();
            stmt.Value = _this.parseHashLiteral(true);
            if (_this.peekTokenIs(token.SEMICOLON)) {
                _this.nextToken();
            }
            return stmt;
        };
        this.parseReturnStatement = function () {
            var stmt = new statement.ReturnStatement(_this.curToken);
            _this.nextToken();
            stmt.ReturnValue = _this.parseExpression(exports.LOWEST);
            if (_this.peekTokenIs(token.SEMICOLON)) {
                _this.nextToken();
            }
            return stmt;
        };
        this.parseBlockStatement = function (checkForReturnStatement) {
            if (checkForReturnStatement === void 0) { checkForReturnStatement = false; }
            var block = new statement.BlockStatement(_this.curToken), hasReturnStatement = false;
            block.Statements = [];
            _this.nextToken();
            while (!_this.curTokenIs(token.RBRACE) && !_this.curTokenIs(token.EOF)) {
                var stmt = _this.parseStatement();
                if (stmt.NodeName == "ExpressionStatement") {
                    if (stmt.Expression == null) {
                        continue;
                    }
                }
                if (checkForReturnStatement) {
                    if (stmt.Token && stmt.Token.Literal == "this") {
                        _this.errors.push("use of this / object-context not allowed in pure functions");
                        return null;
                    }
                    else if (stmt.Expression && stmt.Expression.NodeName == "IndexAssignmentExpression") {
                        if (stmt.Expression.Left.TokenLiteral() == "this") {
                            _this.errors.push("use of this / object-context not allowed in pure functions");
                            return null;
                        }
                    }
                    if (stmt.Token.Type === "RETURN") {
                        switch (stmt.ReturnValue.NodeName) {
                            case "InfixExpression":
                                //TODO: check if the return statement has side-effects
                                break;
                            case "IndexExpression":
                                //TODO: check if the return statement has side-effects
                                break;
                            case "IndexAssignmentExpression":
                                //TODO: check if the return statement has side-effects
                                break;
                        }
                        hasReturnStatement = true;
                    }
                }
                if (stmt != null) {
                    block.Statements.push(stmt);
                }
                _this.nextToken();
            }
            if (checkForReturnStatement && !hasReturnStatement) {
                _this.errors.push("pure function must have a return statement");
                return null;
            }
            return block;
        };
        this.parseIdentifier = function () {
            if (!_this.diagnosticContext.declaredVariables[_this.curToken.Literal]) {
                _this.diagnosticContext.undeclaredVariables[_this.curToken.Literal] = true;
            }
            return new literal.Identifier(_this.curToken, _this.curToken.Literal);
        };
        this.parseBoolean = function () {
            return new literal.BooleanLiteral(_this.curToken, _this.curTokenIs(token.TRUE));
        };
        this.parseIntegerLiteral = function () {
            var curToken = _this.curToken, lit = new literal.IntegerLiteral(curToken), value = parseInt(_this.curToken.Literal);
            if (value == null || value == NaN) {
                _this.errors.push("could not parse " + _this.curToken.Literal + " as integer");
                return null;
            }
            lit.Value = value;
            return lit;
        };
        this.parseFloatLiteral = function () {
            var curToken = _this.curToken, lit = new literal.FloatLiteral(curToken), value = parseFloat(_this.curToken.Literal);
            if (value == null || value == NaN) {
                _this.errors.push("could not parse " + _this.curToken.Literal + " as float");
                return null;
            }
            lit.Value = value;
            return lit;
        };
        this.parseFunctionLiteral = function (_, _2, returnType, pure) {
            var curToken = _this.curToken, lit = pure
                ? new literal.PureFunctionLiteral(curToken)
                : new literal.FunctionLiteral(curToken);
            if (!_this.expectPeek(token.LPAREN)) {
                return null;
            }
            var params = _this.parseFunctionParameters();
            if (!params) {
                return null;
            }
            lit.Parameters = params[0];
            lit.ParameterTypes = params[1];
            lit.ReturnType = returnType;
            if (!_this.expectPeek(token.LBRACE)) {
                return null;
            }
            lit.Body = _this.parseBlockStatement(pure);
            return lit;
        };
        this.parsePureFunctionLiteral = function (_, _2, returnType) {
            returnType = _this.parseDataType(true);
            _this.nextToken();
            return _this.parseFunctionLiteral(null, null, returnType, true);
        };
        this.parseHashLiteral = function (exp, classMap) {
            if (classMap === void 0) { classMap = false; }
            var value = null, key = null, hash = new literal.HashLiteral(_this.curToken);
            hash.Pairs = {};
            while (!_this.peekTokenIs(token.RBRACE)) {
                _this.nextToken();
                // pass flag to parseExpression to parse identifiers as strings
                key = _this.parseExpressionWithModifiers(exports.LOWEST);
                if (!_this.expectPeek(token.COLON)) {
                    return null;
                }
                _this.nextToken();
                value = _this.parseExpression(exports.LOWEST);
                hash.Modifiers[key.Value] = key.modifiers;
                hash.Pairs[key.Value] = value;
                if (!_this.peekTokenIs(token.RBRACE) && !_this.expectPeek(token.COMMA)) {
                    return null;
                }
            }
            if (!_this.expectPeek(token.RBRACE)) {
                return null;
            }
            return hash;
        };
        this.parseFunctionParameters = function () {
            var identifiers = [], types = [];
            if (_this.peekTokenIs(token.RPAREN)) {
                _this.nextToken();
                return [identifiers, types];
            }
            _this.nextToken();
            if (_this.isDataType()) {
                types.push(_this.curToken.Literal);
                _this.nextToken();
            }
            var ident = new literal.Identifier(_this.curToken, _this.curToken.Literal);
            identifiers.push(ident);
            _this.diagnosticContext.declaredVariables[ident.Value] = types[0] || "any";
            while (_this.peekTokenIs(token.COMMA)) {
                _this.nextToken();
                _this.nextToken();
                if (_this.isDataType()) {
                    types.push(_this.curToken.Literal);
                    _this.nextToken();
                }
                ident = new literal.Identifier(_this.curToken, _this.curToken.Literal);
                _this.diagnosticContext.declaredVariables[ident.Value] = types[types.length - 1] || "any";
                identifiers.push(ident);
            }
            if (!_this.expectPeek(token.RPAREN)) {
                return null;
            }
            return [identifiers, types];
        };
        this.parseStringLiteral = function () {
            return new literal.StringLiteral(_this.curToken, _this.curToken.Literal);
        };
        this.parseArrayLiteral = function () {
            var array = new literal.ArrayLiteral(_this.curToken);
            array.Elements = _this.parseExpressionList(token.RBRACKET);
            return array;
        };
        this.parseExpressionList = function (end, delimiter) {
            if (delimiter === void 0) { delimiter = token.COMMA; }
            var list = [];
            if (_this.peekTokenIs(end)) {
                _this.nextToken();
                return list;
            }
            _this.nextToken();
            list.push(_this.parseExpression(exports.LOWEST));
            while (_this.peekTokenIs(delimiter)) {
                _this.nextToken();
                _this.nextToken();
                list.push(_this.parseExpression(exports.LOWEST));
            }
            if (!_this.expectPeek(end)) {
                return null;
            }
            return list;
        };
        this.resetDiagnosticContext();
        this.l = l;
        this.errors = [];
        /* Infix expressions */
        this.registerInfix(token.PLUS, this.parseInfixExpression);
        this.registerInfix(token.MINUS, this.parseInfixExpression);
        this.registerInfix(token.SLASH, this.parseInfixExpression);
        this.registerInfix(token.ASTERISK, this.parseInfixExpression);
        this.registerInfix(token.MOD, this.parseInfixExpression);
        this.registerInfix(token.EQ, this.parseInfixExpression);
        this.registerInfix(token.NOT_EQ, this.parseInfixExpression);
        this.registerInfix(token.AND, this.parseInfixExpression);
        this.registerInfix(token.OR, this.parseInfixExpression);
        this.registerInfix(token.LT, this.parseInfixExpression);
        this.registerInfix(token.GT, this.parseInfixExpression);
        this.registerInfix(token.LPAREN, this.parseCallExpression);
        this.registerInfix(token.SOURCE, this.parseReadStreamExpression);
        this.registerInfix(token.SINK, this.parseWriteStreamExpression);
        this.registerInfix(token.LBRACKET, this.parseIndexExpression);
        this.registerInfix(token.DOT, this.parseDotIndexExpression);
        /* Prefix expressions */
        this.registerPrefix(token.IDENT, this.parseIdentifier);
        this.registerPrefix(token.INT, this.parseIntegerLiteral);
        this.registerPrefix(token.FLOAT, this.parseFloatLiteral);
        this.registerPrefix(token.BANG, this.parsePrefixExpression);
        this.registerPrefix(token.MINUS, this.parsePrefixExpression);
        this.registerPrefix(token.TYPEOF, this.parsePrefixExpression);
        this.registerPrefix(token.LPAREN, this.parseGroupedExpression);
        this.registerPrefix(token.TRUE, this.parseBoolean);
        this.registerPrefix(token.FALSE, this.parseBoolean);
        this.registerPrefix(token.PURE, this.parsePureFunctionLiteral);
        this.registerPrefix(token.FUNCTION, this.parseFunctionLiteral);
        this.registerPrefix(token.STRING, this.parseStringLiteral);
        this.registerPrefix(token.LBRACKET, this.parseArrayLiteral);
        this.registerPrefix(token.LBRACE, this.parseHashLiteral);
        this.registerPrefix(token.NEW, this.parseNewExpression);
        this.registerPrefix(token.EXEC, this.parseExecExpression);
        this.registerPrefix(token.LCOMMENT, this.parseCommentBlock);
    }
    Parser.prototype.resetDiagnosticContext = function () {
        this.diagnosticContext = {
            hasLoops: false,
            hasRecursion: false,
            undeclaredVariables: {},
            declaredVariables: {}
        };
    };
    Parser.prototype.parseProgram = function () {
        var Statements = [], program = new ast.Program(Statements);
        this.nextToken();
        while (!this.curTokenIs(token.EOF)) {
            var stmt = this.parseStatement();
            if (stmt != null) {
                if (stmt.NodeName == "ExpressionStatement") {
                    if (stmt.Expression == null) {
                        continue;
                    }
                }
                program.Statements.push(stmt);
            }
            this.nextToken();
        }
        // console.log(JSON.stringify(this.diagnosticContext, null, 2));
        // console.log(JSON.stringify(program, null, 2));
        return program;
    };
    Parser.prototype.parse = function () {
        this.errors = [];
        this.curToken = null;
        this.peekToken = null;
        this.resetDiagnosticContext();
        this.nextToken();
        this.nextToken();
    };
    Parser.prototype.curTokenIs = function (t) {
        return this.curToken.Type == t;
    };
    Parser.prototype.peekTokenIs = function (t) {
        return this.peekToken.Type == t;
    };
    Parser.prototype.peekPrecedence = function () {
        var p = exports.precedences[this.peekToken.Type];
        if (typeof p == "number") {
            return p;
        }
        return exports.LOWEST;
    };
    Parser.prototype.curPrecedence = function () {
        var p = exports.precedences[this.curToken.Type];
        if (typeof p == "number") {
            return p;
        }
        return exports.LOWEST;
    };
    Parser.prototype.nextToken = function () {
        this.curToken = this.peekToken;
        this.peekToken = this.l.NextToken();
    };
    Parser.prototype.registerPrefix = function (tokenType, fn) {
        this.prefixParseFns[tokenType] = fn;
    };
    Parser.prototype.registerInfix = function (tokenType, fn) {
        this.infixParseFns[tokenType] = fn;
    };
    Parser.prototype.parseDataType = function (peek) {
        if (this.isDataType(peek)) {
            if (!this.peekTokenIs(token.COLON)) {
                this.nextToken();
            }
            // this.nextToken();
            return this.curToken;
        }
    };
    Parser.prototype.handleMultipleVarDeclaration = function (stmt, pure) {
        var Name, Value;
        stmt.Vars = [[stmt.Name, stmt.Value]];
        this.nextToken();
        var currentDataType = this.checkDataType();
        stmt.Types.push(currentDataType || stmt.dataType);
        if (!this.expectPeek(token.IDENT)) {
            return null;
        }
        Name = new literal.Identifier(this.curToken, this.curToken.Literal);
        if (!this.peekTokenIs(token.ASSIGN)) {
            Value = util_3.getDefaultValueNodeForDataType(currentDataType || stmt.dataType);
        }
        else {
            this.nextToken();
            this.nextToken();
            Value = this.parseExpression(exports.LOWEST, pure);
        }
        this.diagnosticContext.declaredVariables[Name.Value] = (stmt.Types[stmt.Types.length - 1])
            || util_2.getDataTypeByNodeName(stmt.Value);
        stmt.Vars.push([Name, Value]);
        while (this.peekTokenIs(token.COMMA)) {
            this.nextToken();
            currentDataType = this.checkDataType();
            stmt.Types.push(currentDataType || stmt.dataType);
            if (!this.expectPeek(token.IDENT)) {
                return null;
            }
            Name = new literal.Identifier(this.curToken, this.curToken.Literal);
            if (!this.peekTokenIs(token.ASSIGN)) {
                Value = util_3.getDefaultValueNodeForDataType(currentDataType || stmt.dataType);
            }
            else {
                this.nextToken();
                this.nextToken();
                Value = this.parseExpression(exports.LOWEST, pure);
            }
            this.diagnosticContext.declaredVariables[Name.Value] = (stmt.Types[stmt.Types.length - 1])
                || util_2.getDataTypeByNodeName(Value);
            stmt.Vars.push([Name, Value]);
        }
        if (this.peekTokenIs(token.SEMICOLON)) {
            this.nextToken();
        }
        return stmt;
    };
    Parser.prototype.expectPeek = function (t) {
        if (this.peekTokenIs(t)) {
            this.nextToken();
            return true;
        }
        else {
            this.peekError(t);
            return false;
        }
    };
    Parser.prototype.checkDataType = function (peek) {
        if (peek === void 0) { peek = true; }
        if (this.isDataType(peek)) {
            if (peek) {
                this.nextToken();
            }
            return this.curToken.Literal;
        }
        return null;
    };
    Parser.prototype.isDataType = function (peek) {
        var _this = this;
        if (peek === void 0) { peek = false; }
        var detector = peek ? function (t) { return _this.peekTokenIs(t); } : function (t) { return _this.curTokenIs(t); };
        return detector(token_1.LET_BOOL)
            || detector(token_1.LET_INT) || detector(token_1.LET_FLOAT)
            || detector(token_1.LET_STRING) || detector(token_1.LET_ARRAY)
            || detector(token.LET_OBJECT) || detector(token.LET_FUNCTION)
            || this.isCustomDataType();
    };
    Parser.prototype.isCustomDataType = function () {
        var typeStr = this.curToken.Literal, firstChar = typeStr.charCodeAt(0);
        return (firstChar > 64 && firstChar < 91)
            || firstChar > 122; // anything except special characters and lower-case
    };
    Parser.prototype.lineNumber = function () {
        return "line: " + this.l.lineNumber + ": ";
    };
    Parser.prototype.Errors = function () {
        return this.errors;
    };
    Parser.prototype.noPrefixParseFnError = function (t) {
        var msg = "no prefix parse function for " + t + " found";
        this.errors.push(this.lineNumber() + msg);
    };
    Parser.prototype.peekError = function (expectedType) {
        var msg = util_1.sprintf("expected next token to be %s, got %s instead", expectedType, this.peekToken.Type);
        this.parseError(msg);
    };
    Parser.prototype.parseError = function (msg) {
        this.errors.push(this.lineNumber() + msg);
    };
    return Parser;
}());
exports.Parser = Parser;
