"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var repl_util_1 = require("./repl-util");
var file_1 = require("../builtins/io/file");
var util_1 = require("../util");
var evaluator = require("../evaluator");
var util_2 = require("../ubiquitous/util");
var node_objects_1 = require("./node-objects");
var translator_1 = require("../transpiler/translator");
var fs = require("fs");
function done() {
    console.log('Exiting..');
    process.exit();
}
var lang = new translator_1.Translator();
function makeCLIRepl(l, p, env, replPlugins) {
    node_objects_1.nodeObjects.fs = fs;
    node_objects_1.nodeObjects.Buffer = Buffer;
    node_objects_1.nodeObjects.process = process;
    var makeHandleInput = function (onInput) {
        return function (text) {
            if (text === 'quit\n' || text === 'exit\n') {
                done();
            }
            text = (text.search(/\n/) > -1 ? text.replace(/\n/g, "") : text);
            if (text[text.length - 2] != ";") {
                text = text + ";";
            }
            onInput(text);
        };
    };
    var localEvaluate = function (text, transpile) {
        l.setText(text);
        p.parse();
        var program = p.parseProgram(), evaluated;
        if (p.Errors().length != 0) {
            repl_util_1.printParserErrors(p.Errors());
            return;
        }
        if (transpile) {
            if (replPlugins && replPlugins.transpilers) {
                var unparsers = replPlugins.transpilers;
                for (var tPlugin in unparsers) {
                    lang.loadTranspiler(tPlugin, unparsers[tPlugin]);
                }
            }
            var unparser = lang.getTranspiler(transpile);
            if (unparser) {
                console.log(unparser.transpile(program, null, p.diagnosticContext));
            }
            else {
                console.error("Missing compiler language support plugin: " + transpile);
            }
        }
        else {
            evaluated = evaluator.Eval(program, env, null, p.diagnosticContext);
            if (evaluated != null) {
                if (evaluated.Inspect) {
                    console.log(evaluated.Inspect());
                    console.log("");
                }
            }
        }
    };
    return function (args) {
        var transpile;
        if (args && args.length >= 1 && args[0]) {
            if (args[0] == "-t" || args[0] == "--transpile") {
                if (args.length < 3) {
                    console.log("not enough arguments to transpile: ecs -t js filename.ecs");
                }
                transpile = args[1];
                args = args.slice(2);
            }
            util_1.getSourceFile(args[0], file_1.readWholeFile, fs).then(function (data) {
                localEvaluate(util_2.forceSingleLine(data), transpile);
            }).catch(function (err) { console.log(err); });
        }
        else {
            repl_util_1.introMessage();
            process.stdin.resume();
            process.stdin.setEncoding('utf8');
            process.stdin.on('data', makeHandleInput(localEvaluate));
        }
    };
}
exports.makeCLIRepl = makeCLIRepl;
