import * as environment from '../object/environment';
import { ECSRuntime } from '../runtime';
import { AnyObject } from '../util';
import { WorkerContext } from '../runtime/events';
declare type VoxelEnv = {
    key: string;
    data?: {
        [key: string]: any;
    };
    env?: environment.Environment;
    entities: EntityEnv[];
};
declare type EntityEnv = {
    id: any;
    data?: {
        [key: string]: any;
    };
    components: ComponentEnv[];
    env?: environment.Environment;
};
declare type ComponentEnv = {
    id: any;
    path: any[];
    data?: {
        [key: string]: any;
    };
    components: ComponentEnv[];
    env: environment.Environment;
};
interface ConvolvrEvent {
    namespace: string;
    event: string;
    data: any[];
    env: [string, number, number];
}
export declare class ECSWorker {
    interpreter: ECSRuntime;
    worker: any;
    updateLoop: any;
    environments: {
        [coords: string]: VoxelEnv;
    };
    constructor(webWorker: {
        postMessage: Function;
        onmessage: Function;
    });
    onCommand(data: any): void;
    onNativeReturn(data: any): void;
    onEvent(data: ConvolvrEvent): void;
    getEnv: (voxelCoords: string, entityId?: any, componentIdx?: any, componentData?: {
        [_: string]: any;
    }, entityData?: {
        [_: string]: any;
    }) => ComponentEnv;
    private getOrMakeComponentEnv;
    onAddVoxel: (data: any) => void;
    onAddEntity: (data: any) => any;
    onAddComponent: (data: any) => any;
    onRefreshAttrs: (data: AnyObject) => void;
    onRefreshState: (data: AnyObject) => void;
    onRefreshBindings: (data: AnyObject) => void;
    onGetVoxel: (data: any) => void;
    onGetEntity: (data: any) => any;
    onGetComponent: (data: any) => any;
    onRemoveVoxel: (data: any) => void;
    onRemoveEntity: (data: any) => any;
    onRemoveComponent: (data: any) => any;
    onUpdateVoxel: (data: any) => void;
    onUpdateTelemetry: (data: any) => any;
    onUpdateEntity: (data: any) => any;
    onUpdateComponent: (data: any) => any;
    stop: () => void;
    clear: () => void;
    eval: (data: any) => void;
    parse: (data: any) => void;
    evalProgram: (data: any) => void;
    private onError;
    private makeVoxelEnv;
    private makeEntityEnv;
    private makeComponentEnv;
    private getById;
    private getEntityEnv;
    private getComponentEnv;
    createWorkerContext: (data: AnyObject, path: string) => WorkerContext;
}
export {};
