"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var evaluator = require("../evaluator");
var environment = require("../object/environment");
var runtime_1 = require("../runtime");
var ECSWorker = /** @class */ (function () {
    function ECSWorker(webWorker) {
        var _this = this;
        this.environments = {};
        this.getEnv = function (voxelCoords, entityId, componentIdx, componentData, entityData) {
            if (entityId === void 0) { entityId = null; }
            if (componentIdx === void 0) { componentIdx = null; }
            var voxel = _this.environments[voxelCoords], entity = null, component = null;
            if (voxel == null) {
                voxel = _this.environments[voxelCoords] = _this.makeVoxelEnv(voxelCoords, entityId == null);
            }
            if (entityId) {
                entity = _this.getById(voxel.entities, entityId);
                if (entity == null) {
                    entity = _this.makeEntityEnv(entityId, componentIdx == null, (componentIdx == null && entityData)
                        ? entityData
                        : {}, [voxelCoords, entityId]);
                    voxel.entities.push(entity);
                }
                if (componentIdx !== undefined) {
                    return _this.getOrMakeComponentEnv(entity, componentData, [voxelCoords, entityId, componentIdx]);
                }
                else {
                    return entity;
                }
            }
            else {
                return voxel;
            }
        };
        this.getOrMakeComponentEnv = function (entity, componentData, path) {
            var component = _this.getById(entity.components, path[2]);
            if (component == null) {
                component = _this.makeComponentEnv(path[2], componentData || {}, path);
                entity.components.push(component);
            }
            return component;
        };
        this.onAddVoxel = function (data) {
            _this.environments[data.voxelCoords] = data.voxel;
        };
        this.onAddEntity = function (data) {
            var voxel = _this.environments[data.voxelCoords];
            if (voxel == null) {
                return null;
            }
            var id = data.entity.id, path = [data.voxelCoords, id];
            voxel.entities.push(_this.makeEntityEnv(id, true, data.entity, path));
        };
        this.onAddComponent = function (data) {
            var voxel = _this.environments[data.voxelCoords];
            if (voxel == null) {
                return null;
            }
            var entity = _this.getEntityEnv(voxel, data.entityId);
            if (entity == null) {
                return null;
            }
            var id = data.component.id, path = [data.voxelCoords, data.entity.id, id];
            entity.components.push(_this.makeComponentEnv(id, data.component, path));
        };
        this.onRefreshAttrs = function (data) {
        };
        this.onRefreshState = function (data) {
        };
        this.onRefreshBindings = function (data) {
        };
        this.onGetVoxel = function (data) {
            var voxel;
            _this.worker.postMessage(JSON.stringify({
                command: "get-voxel",
                data: voxel
            }));
        };
        this.onGetEntity = function (data) {
            var voxel = _this.environments[data.voxelCoords];
            if (voxel == null) {
                return null;
            }
            var entity = _this.getEntityEnv(voxel, data.entityId);
            _this.worker.postMessage(JSON.stringify({
                command: "get-entity",
                data: entity
            }));
        };
        this.onGetComponent = function (data) {
            var voxel = _this.environments[data.voxelCoords];
            if (voxel == null) {
                return null;
            }
            var entity = _this.getEntityEnv(voxel, data.entityId);
            if (entity == null) {
                return null;
            }
            var component = _this.getComponentEnv(entity, data.componentId);
            _this.worker.postMessage(JSON.stringify({
                command: "get-component",
                data: component
            }));
        };
        this.onRemoveVoxel = function (data) {
            _this.environments[data.voxelCoords] = null;
        };
        this.onRemoveEntity = function (data) {
            var voxel = _this.environments[data.voxelCoords];
            if (voxel == null) {
                return null;
            }
            for (var _i = 0, _a = voxel.entities; _i < _a.length; _i++) {
                var e = _a[_i];
                if (e.id == data.entityId) {
                    voxel.entities.splice(voxel.entities.indexOf(e), 1);
                }
            }
        };
        this.onRemoveComponent = function (data) {
            var voxel = _this.environments[data.voxelCoords], entity = null;
            if (voxel == null) {
                return null;
            }
            entity = _this.getEntityEnv(voxel, data.entityId);
            if (entity == null) {
                return null;
            }
            for (var _i = 0, _a = entity.components; _i < _a.length; _i++) {
                var c = _a[_i];
                if (c.id == data.componentId) {
                    entity.components.splice(entity.components.indexOf(c), 1);
                }
            }
        };
        this.onUpdateVoxel = function (data) {
            _this.environments[data.voxelCoords].data = data.voxelData;
        };
        this.onUpdateTelemetry = function (data) {
            var voxel = _this.environments[data.voxelCoords], entity = null;
            if (voxel == null) {
                return null;
            }
            entity = _this.getEntityEnv(voxel, data.entityId);
            if (entity == null) {
                return null;
            }
            if (data.componentId) {
                for (var c in entity.components) {
                    var cmp = entity.components[c];
                    if (cmp.id == data.componentId) {
                        if (data.position) {
                            cmp.data.position = data.position;
                        }
                        if (data.rotation) {
                            cmp.data.rotation = data.rotation;
                        }
                    }
                }
            }
            else {
                if (data.position) {
                    entity.data.position = data.position;
                }
                if (data.rotation) {
                    entity.data.rotation = data.rotation;
                }
            }
        };
        this.onUpdateEntity = function (data) {
            var voxel = _this.environments[data.voxelCoords];
            if (voxel == null) {
                return null;
            }
            for (var _i = 0, _a = voxel.entities; _i < _a.length; _i++) {
                var e = _a[_i];
                if (e.id == data.entityId) {
                    e.data = data.entityData;
                }
            }
        };
        this.onUpdateComponent = function (data) {
            var voxel = _this.environments[data.voxelCoords], entity = null;
            if (voxel == null) {
                return null;
            }
            entity = _this.getEntityEnv(voxel, data.entityId);
            if (entity == null) {
                return null;
            }
            for (var _i = 0, _a = entity.components; _i < _a.length; _i++) {
                var c = _a[_i];
                if (c.id == data.componentId) {
                    c.data = data.componentData;
                }
            }
        };
        this.stop = function () {
            clearTimeout(_this.updateLoop);
        };
        this.clear = function () {
            _this.environments = {};
        };
        this.eval = function (data) {
            var environmentData;
            if (data.envData) {
                if (data.env.length == 2) {
                    environmentData = _this.getEnv(data.env[0], data.env[1], "", data.envData);
                }
                else {
                    environmentData = _this.getEnv(data.env[0], data.env[1], data.env[2], data.envData);
                }
            }
            else {
                environmentData = _this.getEnv(data.env[0], data.env[1], data.env[2]);
            }
            _this.worker.postMessage(JSON.stringify({
                command: "return-value",
                env: data.env.join(","),
                data: _this.interpreter.parseAndEvaluate(data.code, environmentData.env, function (error) {
                    _this.onError(error, data.env);
                })
            }));
        };
        this.parse = function (data) {
            _this.worker.postMessage(JSON.stringify({
                command: "parse-tree",
                env: data.env.join(","),
                data: _this.interpreter.parse(data.code, function (error) {
                    _this.onError(error, data.env);
                })
            }));
        };
        this.evalProgram = function (data) {
            _this.worker.postMessage(JSON.stringify({
                command: "return-value",
                env: data.env.join(","),
                data: _this.interpreter.evaluateProgram(data.program, _this.getEnv(data.env[0], data.env[1], data.env[2]).env, function (error) {
                    _this.onError(error, data.env);
                })
            }));
        };
        this.onError = function (error, env) {
            _this.worker.postMessage(JSON.stringify({
                command: "internal-error",
                env: env.join(","),
                data: error
            }));
        };
        this.makeVoxelEnv = function (key, isScope, data) {
            if (isScope === void 0) { isScope = false; }
            return {
                key: key,
                env: isScope ? new environment.Environment(null, _this.createWorkerContext({ voxel: data || { data: {} } }, key)) : null,
                entities: []
            };
        };
        this.makeEntityEnv = function (id, isScope, data, path) {
            if (isScope === void 0) { isScope = false; }
            return {
                id: id,
                data: data,
                path: path,
                env: new environment.Environment(null, _this.createWorkerContext({ entity: data || {} }, path.join(","))),
                components: []
            };
        };
        this.makeComponentEnv = function (id, data, path) {
            return {
                id: id,
                data: data,
                path: path,
                env: new environment.Environment(null, _this.createWorkerContext({ component: data || {} }, path.join(","))),
                components: []
            };
        };
        this.getById = function (entities, id) {
            for (var _i = 0, entities_1 = entities; _i < entities_1.length; _i++) {
                var e = entities_1[_i];
                if (e.id == id) {
                    return e;
                }
            }
            return null;
        };
        this.getEntityEnv = function (voxel, entityId) {
            for (var _i = 0, _a = voxel.entities; _i < _a.length; _i++) {
                var e = _a[_i];
                if (e.id == entityId) {
                    return e;
                }
            }
            return null;
        };
        this.getComponentEnv = function (entity, componentId) {
            for (var _i = 0, _a = entity.components; _i < _a.length; _i++) {
                var c = _a[_i];
                if (c.id == componentId) {
                    return c;
                }
            }
            return null;
        };
        this.createWorkerContext = function (data, path) {
            return {
                data: data,
                postMessage: function (command, data) { return _this.worker.postMessage(JSON.stringify({
                    command: command,
                    data: data,
                    path: path
                })); }
            };
        };
        this.interpreter = runtime_1.makeRuntimeEnvironment(evaluator.Eval, evaluator.optimizedEvalProgram, evaluator.applyFunction); // avoid circular dependency
        this.worker = webWorker;
        this.worker.onmessage = function (event) {
            var message = JSON.parse(event.data), data = message.data;
            switch (message.command) {
                case "command":
                    return _this.onCommand(data);
                case "native-return":
                    return _this.onNativeReturn(data);
                case "event":
                    return _this.onEvent(data);
                // evaluate code
                case "eval":
                    return _this.eval(data);
                case "eval-program":
                    return _this.evalProgram(data);
                case "parse":
                    return _this.parse(data);
            }
        };
    }
    ECSWorker.prototype.onCommand = function (data) {
        switch (data.commandName) {
            // add environments
            case "add-voxel":
                this.onAddVoxel(data);
                break;
            case "add-entity":
                this.onAddEntity(data);
                break;
            case "add-component":
                this.onAddComponent(data);
                break;
            // get environments
            case "get-voxel":
                this.onGetVoxel(data);
                break;
            case "get-entity":
                this.onGetEntity(data);
                break;
            case "get-component":
                this.onGetComponent(data);
                break;
            // update environments
            case "update-voxel":
                this.onUpdateVoxel(data);
                break;
            case "update-entity":
                this.onUpdateEntity(data);
                break;
            case "update-component":
                this.onUpdateComponent(data);
                break;
            case "refresh-attrs":
                this.onUpdateVoxel(data);
                break;
            case "refresh-state":
                this.onUpdateEntity(data);
                break;
            case "refresh-bindings":
                this.onUpdateComponent(data);
                break;
            // remove environments
            case "remove-voxel":
                this.onRemoveVoxel(data);
                break;
            case "remove-entity":
                this.onRemoveEntity(data);
                break;
            case "remove-component":
                this.onRemoveComponent(data);
                break;
            // remove all environments
            case "clear":
                this.clear();
                break;
        }
    };
    ECSWorker.prototype.onNativeReturn = function (data) {
        // implement
        // handle the result of os.lshw, entity.getPosition, component.listComponents, etc
    }; // Maybe this is related to `onEvent` below:
    ECSWorker.prototype.onEvent = function (data) {
        var path = data.env, component = this.getEnv(path[0], path[1], path[2]), env = component ? component.env : null;
        if (!env) {
            console.warn("External Event: environment not initialized", path);
            return;
        }
        env.onEvent(data.namespace, data.event, data.data, this.interpreter, env);
    };
    return ECSWorker;
}());
exports.ECSWorker = ECSWorker;
;
if (this['se' + 'lf']) { // identity crisis
    self.worker = new ECSWorker(self);
}
