import { CallExpression, IndexAssignmentExpression, IndexExpression } from "../ast/expression";
import { Identifier } from "../ast/literal";
import { AssignmentStatement, ExpressionStatement, LetStatement } from "../ast/statement";
import { ParseTreeAnalysis } from "../parser";
import { Transpiler } from "../transpiler";
import { UnParserPlugin } from "../transpiler/unparser";
import { Flags } from "../ubiquitous/util";
/**
 * Context plugin
 * provides access to ecs variables and functions from compiled js code
 */
export declare class JSECSContextPlugin extends UnParserPlugin {
    protected unparser: Transpiler;
    private getterIndex;
    private setterIndex;
    private callIndex;
    private callBuiltinIndex;
    private callMethodIndex;
    private getBuiltinObjectIndex;
    private isomorphicBuiltins;
    affectedIdentifiers: Flags;
    private updateExternalIdentifiers;
    private dataTypesByIdent;
    private builtinFunctionPolyfill;
    private ecsNativeClasses;
    private mutatorIdentifiers;
    private ecsNativeFunctions;
    private ecsNativeAPIs;
    private ecsNativeObjects;
    constructor();
    setParserAnalysis(analysis: ParseTreeAnalysis, builtinFunctionPolyfill: string[]): this;
    setUnParser(unparser: Transpiler): void;
    nodeTransforms: {
        "Identifier": (node: Identifier) => (string | Identifier)[];
        "LetStatement": (node: LetStatement, topScope?: boolean) => (string | LetStatement)[];
        "AssignmentStatement": (node: AssignmentStatement) => (string | AssignmentStatement)[];
        "IndexExpression": (node: IndexExpression) => [CallExpression, string, string] | (string | IndexExpression)[];
        "IndexAssignmentExpression": (node: IndexAssignmentExpression) => (string | IndexAssignmentExpression)[];
        "CallExpression": (node: CallExpression) => (string | CallExpression)[];
        "ExpressionStatement": (node: ExpressionStatement) => any;
    };
    private updateExternalObject;
    private routeBuiltinThroughContext;
    private routeBuiltinMethodThroughContext;
    private routeBuiltinObjectFieldThroughContext;
    private routeExternalIdentThroughContext;
    private routeAssignmentThroughContext;
    private routeCallExpressionThroughContext;
}
