"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var literal_1 = require("../ast/literal");
var statement_1 = require("../ast/statement");
var unparser_1 = require("../transpiler/unparser");
var util_1 = require("../ubiquitous/util");
/**
 * Context plugin
 * provides access to ecs variables and functions from compiled js code
 */
var JSECSContextPlugin = /** @class */ (function (_super) {
    __extends(JSECSContextPlugin, _super);
    function JSECSContextPlugin() {
        var _this = _super.call(this) || this;
        _this.nodeTransforms = {
            "Identifier": function (node) {
                var nodeValue = node.Value;
                if (_this.affectedIdentifiers[nodeValue] &&
                    _this.builtinFunctionPolyfill.indexOf(nodeValue) == -1 &&
                    !_this.resolvedIdentifiers[nodeValue] &&
                    !_this.isomorphicBuiltins[nodeValue]) {
                    _this.resolvedIdentifiers[node.Value] = true;
                    return [node, _this.unparser.unParse(_this.routeExternalIdentThroughContext(node), { noPlugins: true }) + ";\n", ""];
                }
                else {
                    return [node, "", ""];
                }
            },
            "LetStatement": function (node, topScope) {
                /**
                 *  [node: Node, prependedCode: string, appendedCode: string]
                 */
                var operations = [node, "", ""];
                if (topScope) {
                    if (node.Vars) {
                        for (var v in node.Vars) {
                            var identAndExp = node.Vars[v];
                            // update external context to existence of this variable
                            operations[2] = operations[2] + _this.unparser.unParse(_this.routeAssignmentThroughContext(identAndExp[0]), { noPlugins: true }) + "\n";
                        }
                    }
                    else {
                        // update external context to existence of this variable
                        operations[2] = operations[2] + _this.unparser.unParse(_this.routeAssignmentThroughContext(node.Name), { noPlugins: true }) + "\n";
                    }
                }
                return operations;
            },
            "AssignmentStatement": function (node) {
                /**
                 *  [node: Node, prependedCode: string, appendedCode: string]
                 */
                var operations = [node, "", ""], identName = node.Name.Value;
                // if an external identifier is assigned
                if (_this.affectedIdentifiers[identName]) {
                    // if it hasn't already been resolved
                    if (!_this.resolvedIdentifiers[identName]) {
                        // resolve it, before the rest of the program
                        _this.resolvedIdentifiers[identName] = true;
                        operations[1] = _this.unparser.unParse(_this.routeExternalIdentThroughContext(node.Name), { noPlugins: true }) + ";\n";
                    }
                    if (!_this.updateExternalIdentifiers[identName]) {
                        _this.updateExternalIdentifiers[identName] = true;
                        // update the variable from the external context's point of view, at the end:
                        operations[2] = _this.unparser.unParse(_this.routeAssignmentThroughContext(node.Name), { noPlugins: true }) + ";\n";
                    }
                }
                return operations;
            },
            "IndexExpression": function (node) {
                /**
                *  [node: Node, prependedCode: string, appendedCode: string]
                */
                var operations = [node, "", ""];
                if (node.Left.NodeName === "Identifier") {
                    var objName = node.Left.Value;
                    if (node.Index.NodeName === "Identifier") {
                        var identName = node.Index.Value;
                        var nativeObjects = _this.ecsNativeObjects[objName];
                        if (nativeObjects) {
                            if (nativeObjects.indexOf(identName) > -1) {
                                return _this.routeBuiltinObjectFieldThroughContext(objName, identName, node);
                            }
                        }
                    }
                }
                return operations;
            },
            "IndexAssignmentExpression": function (node) {
                /**
                 *  [node: Node, prependedCode: string, appendedCode: string]
                 */
                var operations = [node, "", ""];
                if (node.Left.NodeName === "Identifier") {
                    var identName = node.Left.Value;
                    // if an external identifier is assigned
                    if (_this.affectedIdentifiers[identName]) {
                        // just in case the variable being assigned hasn't been imported into js yet
                        if (!_this.resolvedIdentifiers[identName]) {
                            _this.resolvedIdentifiers[identName] = true;
                            operations[1] = _this.unparser.unParse(// import variable
                            _this.routeExternalIdentThroughContext(node.Left), { noPlugins: true }) + ";\n";
                        }
                        // and it has not been exported to the ecs runtime yet 
                        // update external variable:
                        operations[2] = _this.updateExternalObject(node, identName, operations[2]);
                    }
                }
                return operations;
            },
            "CallExpression": function (node) {
                var isIdent = node.Function.NodeName === "Identifier";
                var identName;
                if (isIdent) {
                    identName = node.Function.Value;
                    if (_this.ecsNativeFunctions[identName]) {
                        return _this.routeBuiltinThroughContext(identName, node);
                    }
                    if (_this.mutatorIdentifiers.indexOf(identName) > -1) { // Beware of mutants
                        return [node, "", _this.updateExternalObject(node.Arguments[0], identName, "")];
                    }
                }
                else if (node.Function.NodeName === "IndexExpression") {
                    identName = node.Function.Index.Value;
                    var leftIsIdent = node.Function.Left.NodeName === "Identifier";
                    // TODO: handle nested static built in methods
                    // if (!indexIsIdent && (node. Function as IndexExpression).Left.NodeName === "IndexExpression") {
                    // } else {
                    // }
                    var objName = leftIsIdent
                        ? node.Function.Left.Value
                        : null;
                    var nativeAPI = _this.ecsNativeAPIs[identName];
                    if (nativeAPI && nativeAPI[objName]) { // make sure this is a call to the correct object
                        return _this.routeBuiltinMethodThroughContext(objName, identName, node);
                    }
                    // figure out if the method calls need to be routed through runtime context by checking the class name
                    var outerType = _this.dataTypesByIdent[objName];
                    if (_this.ecsNativeClasses.indexOf(outerType) > -1) {
                        return _this.routeBuiltinMethodThroughContext(outerType, identName, node, node.Function.Left);
                    }
                }
                else if (node.Function.NodeName === "NewExpression") {
                    identName = node.Function.Name.Value;
                    var externalNewable = _this.affectedIdentifiers[identName] && util_1.isAbstractDataType(_this.dataTypesByIdent[identName]);
                    if (_this.ecsNativeFunctions[identName] || externalNewable) {
                        return _this.routeBuiltinThroughContext(identName, node);
                    }
                }
                // handle external functions
                if (isIdent && _this.affectedIdentifiers[identName] &&
                    _this.builtinFunctionPolyfill.indexOf(identName) == -1 &&
                    !_this.ecsNativeFunctions[identName] &&
                    !_this.isomorphicBuiltins[identName]) {
                    return [_this.routeCallExpressionThroughContext(node), "", ""];
                }
                else {
                    return [node, "", ""];
                }
            },
            "ExpressionStatement": function (node) {
                if (node.Expression) {
                    var transformPlugin = _this.nodeTransforms[node.Expression.NodeName];
                    if (transformPlugin) {
                        return transformPlugin(node.Expression);
                    }
                }
                return [node, "", ""];
            }
        };
        _this.resolvedIdentifiers = {};
        _this.updateExternalIdentifiers = {};
        _this.dataTypesByIdent = {};
        _this.builtinFunctionPolyfill = [];
        _this.ecsNativeClasses = ["Matrix", "Vector", "Component", "Entity", "Image"];
        _this.ecsNativeFunctions = {
            "print": true, "colorize": true, "Matrix": true, "Vector": true, "Component": true, "Entity": true, "Image": true
        };
        _this.ecsNativeAPIs = {
            "args": { "io": true },
            "keyDown": { "io": true },
            "keyUp": { "io": true },
            "readLine": { "io": true },
            "clear": { "terminal": true },
            "getDimensions": { "terminal": true },
            "makeBuffer": { "gl": true },
            "shadeVertices": { "gl": true },
            "blit": { "gl": true },
            "rasterize": { "gl": true },
            "add": { "Vector": true },
            "sub": { "Vector": true },
            "cross": { "Vector": true },
            "multiply": { "Matrix": true }
        };
        _this.ecsNativeObjects = {
            "gl": ["ASCII", "SHAPE", "Shaders", "SURFACE_TOPOLOGY", "BLEND_MODE", "RASTER_MODE"]
        };
        return _this;
    }
    JSECSContextPlugin.prototype.setParserAnalysis = function (analysis, builtinFunctionPolyfill) {
        this.affectedIdentifiers = analysis.undeclaredVariables || {};
        this.dataTypesByIdent = analysis.declaredVariables || {};
        this.affectedIdentifiers["print"] = null;
        this.builtinFunctionPolyfill = builtinFunctionPolyfill;
        this.resolvedIdentifiers = { "this": true };
        this.updateExternalIdentifiers = { "this": true };
        return this;
    };
    JSECSContextPlugin.prototype.setUnParser = function (unparser) {
        this.unparser = unparser;
        this.callIndex = unparser.callIndex;
        this.callBuiltinIndex = unparser.callBuiltinIndex;
        this.callMethodIndex = unparser.callMethodIndex;
        this.getBuiltinObjectIndex = unparser.getBuiltinObjectIndex;
        this.getterIndex = unparser.getterIndex;
        this.setterIndex = unparser.setterIndex;
        this.isomorphicBuiltins = unparser.isomorphicBuiltins;
        this.mutatorIdentifiers = unparser.mutantFunctions;
    };
    JSECSContextPlugin.prototype.updateExternalObject = function (ident, identName, appendOperation) {
        if (!this.updateExternalIdentifiers[identName]) {
            this.updateExternalIdentifiers[identName] = true; // mark it as updated     
            // update the variable from the external context's point of view, at the end:
            return this.unparser.unParse(this.routeAssignmentThroughContext(ident), { noPlugins: true }) + ";\n";
        }
        return appendOperation;
    };
    JSECSContextPlugin.prototype.routeBuiltinThroughContext = function (identName, node) {
        return [this.unparser.routeThroughContext(this.callBuiltinIndex, [
                new literal_1.StringLiteral({ Literal: identName, Type: "string" }, identName),
                new literal_1.ArrayLiteral({ Literal: "[", Type: "[" }, node.Arguments)
            ]), "", ""];
    };
    JSECSContextPlugin.prototype.routeBuiltinMethodThroughContext = function (className, methodName, node, instance) {
        var contextArgs = [
            new literal_1.StringLiteral({ Literal: className, Type: "string" }, className),
            new literal_1.StringLiteral({ Literal: methodName, Type: "string" }, methodName),
            new literal_1.ArrayLiteral({ Literal: "[", Type: "[" }, node.Arguments)
        ];
        if (instance) {
            contextArgs.push(instance);
        }
        return [this.unparser.routeThroughContext(this.callMethodIndex, contextArgs), "", ""];
    };
    JSECSContextPlugin.prototype.routeBuiltinObjectFieldThroughContext = function (objName, fieldName, node) {
        var contextArgs = [
            new literal_1.StringLiteral({ Literal: objName, Type: "string" }, objName),
            new literal_1.StringLiteral({ Literal: fieldName, Type: "string" }, fieldName),
        ];
        return [this.unparser.routeThroughContext(this.getBuiltinObjectIndex, contextArgs), "", ""];
    };
    JSECSContextPlugin.prototype.routeExternalIdentThroughContext = function (node) {
        return new statement_1.LetStatement({ Literal: "let", Type: "LET" }, node, this.unparser.routeThroughContext(this.getterIndex, [new literal_1.StringLiteral({ Literal: node.Value, Type: '"' }, node.Value)]));
    };
    JSECSContextPlugin.prototype.routeAssignmentThroughContext = function (name) {
        // TODO: handle let statements with multiple vars
        return this.unparser.routeThroughContext(this.setterIndex, [
            new literal_1.StringLiteral({ Type: '"', Literal: name.Value }, name.Value),
            name
        ]);
    };
    JSECSContextPlugin.prototype.routeCallExpressionThroughContext = function (node) {
        return this.unparser.routeThroughContext(this.callIndex, [node.Function, new literal_1.ArrayLiteral({ Literal: "[", Type: "[" }, node.Arguments)]);
    };
    return JSECSContextPlugin;
}(unparser_1.UnParserPlugin));
exports.JSECSContextPlugin = JSECSContextPlugin;
