"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var object_1 = require("../object/object");
var obj_util_1 = require("../object/obj-util");
;
exports.SynchronousEventContext = {
    postMessage: null,
    env: null,
    data: null
};
exports.SynchronousEventContext.postMessage = function (command, data) {
    console.log("post message to Synchronous Event Context", command, data);
};
//TODO: Maybe some default event handlers for visualizing components from the REPL [?]
var ECSHostEvents = /** @class */ (function () {
    function ECSHostEvents(synchronousContext) {
        var _this = this;
        this.synchronousContext = synchronousContext;
        this.eventHandlers = {};
        synchronousContext.postMessage = function (command, data) {
            console.log("post message to Synchronous Event Context", command, data);
            var handlers = _this.eventHandlers[command];
            if (handlers) {
                for (var i = 0, l = handlers.length; i < l; i++) {
                    handlers[i](data);
                }
            }
        };
    }
    ECSHostEvents.prototype.setInterpreter = function (interpreter) {
        this.interpreter = interpreter;
    };
    /**
     * When running ECS outside of a web-worker, posted messages can be handled via this function
     */
    ECSHostEvents.prototype.handleAddEventHandler = function (commandName, handler) {
        if (!this.eventHandlers[commandName]) {
            this.eventHandlers[commandName] = [];
        }
        this.eventHandlers[commandName].push(handler);
    };
    ECSHostEvents.prototype.dispatchHostEvent = function (env, eventName, data) {
        env.onEvent("HostEvent", eventName, data, this.interpreter, env, this.synchronousContext);
    };
    ECSHostEvents.prototype.addHostEventHandler = function (env, eventName, handler) {
        env.addEventHandler("HostEvent", eventName, new object_1.Builtin(eventName, [object_1.FUNCTION_OBJ], function (context, scope, jsScope, data) {
            handler(obj_util_1.ecsObjectToNativeObject(arguments[2]));
            return object_1.NULL;
        }, "", null));
    };
    return ECSHostEvents;
}());
exports.ECSHostEvents = ECSHostEvents;
