"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var environment_1 = require("../object/environment");
var lexer = require("../lexer");
var parser = require("../parser");
var events_1 = require("./events");
var util_1 = require("../ubiquitous/util");
var ECSRuntime = /** @class */ (function () {
    function ECSRuntime(_tokenizer, _parser, evalFunction, evalProgramFunction, applyFunction) {
        var _this = this;
        this.applyFunction = applyFunction;
        this.makeEnvironment = function (outer) { return new environment_1.Environment(outer); };
        this.parseAndEvaluate = function (text, env, onErrors, joinNewLines) {
            if (joinNewLines === void 0) { joinNewLines = true; }
            text = joinNewLines ? util_1.forceSingleLine(text) : text;
            _this.tokenizer.setText(text);
            _this.parser.parse();
            var program = _this.parser.parseProgram();
            if (_this.parser.Errors().length != 0) {
                onErrors && onErrors(_this.parser.Errors());
                return;
            }
            var evaluated = _this.eval(program, env, null, _this.parser.diagnosticContext);
            if (evaluated != null) {
                if (evaluated.Inspect) {
                    return evaluated.Inspect();
                }
                else {
                    return null;
                }
            }
        };
        this.parse = function (code, onError) {
            code = /\n/.test(code) ? util_1.forceSingleLine(code) : code;
            _this.tokenizer.setText(code);
            _this.parser.parse();
            var program = _this.parser.parseProgram();
            if (_this.parser.Errors().length != 0) {
                onError && onError(_this.parser.Errors());
                return;
            }
            return program;
        };
        this.evaluateProgram = function (program, env, onErrors) {
            if (program == undefined || program.Statements == undefined) {
                return null;
            }
            var evaluated = _this.evalProgram(program, env);
            if (evaluated != null) {
                if (evaluated.Inspect) {
                    return evaluated.Inspect();
                }
                else {
                    return null;
                }
            }
        };
        this.tokenizer = _tokenizer;
        this.parser = _parser;
        this.eval = evalFunction;
        this.evalProgram = evalProgramFunction;
        this.events = new events_1.ECSHostEvents(events_1.SynchronousEventContext);
        this.events.setInterpreter(this);
    }
    return ECSRuntime;
}());
exports.ECSRuntime = ECSRuntime;
// needs either evaluator or evalFn
exports.makeRuntimeEnvironment = function (evalFunction, evalProgram, evalBodyAndApplyFunctionFn, hostEvents) {
    var tokenizer = new lexer.Lexer(), _parser = new parser.Parser(tokenizer);
    return new ECSRuntime(tokenizer, _parser, evalFunction, evalProgram, evalBodyAndApplyFunctionFn);
};
