"use strict";
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
Object.defineProperty(exports, "__esModule", { value: true });
var obj_util_1 = require("../object/obj-util");
var builtins_1 = require("../builtins");
var JSECSEvaluatorContext = /** @class */ (function () {
    function JSECSEvaluatorContext(applyFunction, evalFn) {
        this.applyFunction = applyFunction;
        this.evalFn = evalFn;
    }
    JSECSEvaluatorContext.prototype.setProgram = function (program) {
        this.program = program;
        return this;
    };
    JSECSEvaluatorContext.prototype.setEnv = function (env) {
        this.env = env;
        return this;
    };
    JSECSEvaluatorContext.prototype.getVar = function (identName) {
        var ecsValue = this.env.get(identName);
        // TODO: use a reference some how, if it's not something serializable..
        if (!ecsValue) {
            ecsValue = builtins_1.builtins[identName];
        }
        return (ecsValue === null || ecsValue === undefined) ? null : obj_util_1.ecsObjectToNativeObject(ecsValue);
    };
    //TODO: swap implementation with getBuiltinObject 
    JSECSEvaluatorContext.prototype.getBuiltin = function (identName) {
        var builtinObject = builtins_1.builtins[identName];
        return builtinObject ? obj_util_1.ecsObjectToNativeObject(builtinObject) : null;
    };
    JSECSEvaluatorContext.prototype.getBuiltinObject = function (identName, fieldName) {
        var ecsValue = this.env.get(identName);
        if (!ecsValue) {
            var builtin = builtins_1.builtins[identName];
            if (builtin) {
                var pair = builtin.Pairs[fieldName];
                if (pair) {
                    ecsValue = pair.Value;
                }
            }
        }
        // TODO: use a reference some how, if it's not something serializable..
        return (ecsValue === null || ecsValue === undefined) ? null : obj_util_1.ecsObjectToNativeObject(ecsValue);
    };
    JSECSEvaluatorContext.prototype.setVar = function (identName, value) {
        if (typeof value === "function") {
            var letStatement = this.program.Statements
                .filter(function (statement) { return statement.NodeName === "LetStatement"; })
                .find(function (statement) { return statement.Name.Value === identName; });
            //     ^^^ Maybe these can be thrown into a map keyed by identName, ahead of time 
            if (letStatement) {
                this.evalFn(letStatement, this.env);
            }
            else {
                console.error("js ECS Runtime Error: identifier not found: " + identName);
            }
        }
        else {
            this.env.set(identName, obj_util_1.nativeValueToECSValue(value));
        }
        return value;
    };
    JSECSEvaluatorContext.prototype.callMethod = function (className, methodName, args, instance) {
        var obj = builtins_1.builtins[className];
        if (obj) {
            var fn = obj.Pairs[methodName].Value;
            if (fn) {
                // Don't need to convert the return value if it's already JS
                // console.log("try to call method", fn.Fn);
                return fn.Fn.apply(fn, __spreadArrays([null, obj, instance], args));
            }
        }
    };
    JSECSEvaluatorContext.prototype.call = function (identName, args) {
        var fn = this.env.get(identName);
        if (fn) {
            return this.handleCallAndConvertResultToJS(fn, args);
        }
    };
    JSECSEvaluatorContext.prototype.callBuiltin = function (identName, args) {
        var _a;
        var fn = builtins_1.builtins[identName];
        if (fn) {
            if (fn.Fn) {
                return (_a = fn).Fn.apply(_a, __spreadArrays([null, null, null], args));
            }
            else {
                return this.handleCallBuiltinConstructor(identName, fn, args);
            }
        }
    };
    JSECSEvaluatorContext.prototype.handleCallBuiltinConstructor = function (className, functor, args) {
        var _a;
        // Once the instance is created, it too must bear a token of the class name
        var funkstructor = functor.Constructor;
        if (funkstructor) {
            return (_a = funkstructor).Fn.apply(_a, __spreadArrays([null, null, null], args));
        }
    };
    JSECSEvaluatorContext.prototype.handleCallAndConvertResultToJS = function (fn, args, objectContext) {
        var representationalScope = objectContext ? obj_util_1.nativeValueToECSValue(objectContext) : null;
        var result = this.applyFunction(fn, this.env, obj_util_1.nativeListToArray(args).Elements, representationalScope);
        if (objectContext && representationalScope) {
            if (fn.stateful) {
                var convertedScope = obj_util_1.ecsObjectToNativeObject(representationalScope);
                for (var keyIdx in fn.stateful) {
                    var key = fn.stateful[keyIdx];
                    objectContext[key] = convertedScope[key];
                }
            }
        }
        if (result) {
            return obj_util_1.ecsObjectToNativeObject(result);
        }
    };
    return JSECSEvaluatorContext;
}());
exports.JSECSEvaluatorContext = JSECSEvaluatorContext;
