import { Program } from "../ast";
import { ParseTreeAnalysis } from "../parser";
import { Transpiler } from "../transpiler";
import { Environment } from "../object/environment";
import { ApplyFunctionFunction } from "../object/function";
import { DynamicFunction, Hash } from "../object/object";
import { EvalFunction } from "./eval";
/**
 * Optimizer reads complexity metrics from Parser
 * and is used by Evaluator where it transpiles complex code.
 *
 * If code is deemed complex, it is transpiled and run as a JS function.
 * A context is created from the Environment which is passed to the Transpiler.
 * After running the code, the return value is de-serialized back into ecs.
 */
export declare class Optimizer {
    private unparser;
    private unParserPlugin;
    private analyzer;
    private jsECSContext;
    constructor(unparser: Transpiler, applyFunction: ApplyFunctionFunction, evalFn: EvalFunction);
    optimizedEvaluate(analysis: ParseTreeAnalysis, program: Program, env: Environment, fallback: Function): any;
    compileFunction(obj: DynamicFunction, env: Environment): Function;
    compileClass(obj: Hash, env: Environment): void;
    private runAsJS;
}
