"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var js_ecs_context_1 = require("./js-ecs-context");
var environment_plugin_1 = require("./environment-plugin");
var analyzer_1 = require("../transpiler/analyzer");
/**
 * Optimizer reads complexity metrics from Parser
 * and is used by Evaluator where it transpiles complex code.
 *
 * If code is deemed complex, it is transpiled and run as a JS function.
 * A context is created from the Environment which is passed to the Transpiler.
 * After running the code, the return value is de-serialized back into ecs.
 */
var Optimizer = /** @class */ (function () {
    function Optimizer(unparser, applyFunction, evalFn) {
        this.unparser = unparser;
        this.unParserPlugin = new environment_plugin_1.JSECSContextPlugin();
        this.analyzer = new analyzer_1.Analyzer();
        this.jsECSContext = new js_ecs_context_1.JSECSEvaluatorContext(applyFunction, evalFn);
        this.unParserPlugin.setUnParser(this.unparser);
    }
    Optimizer.prototype.optimizedEvaluate = function (analysis, program, env, fallback) {
        if (analysis && (analysis.hasLoops || analysis.hasRecursion)) {
            return this.runAsJS(program, env, this.unparser.transpile(program, env, analysis, this.unParserPlugin.setParserAnalysis(analysis, this.unparser.getBuiltinFunctionList())));
        }
        return fallback();
    };
    Optimizer.prototype.compileFunction = function (obj, env) {
        var auxParams = obj.Parameters.map(function (p) { return p.Value; });
        var analysis = this.analyzer.analyzeParseTree(obj.Body);
        var program = this.unparser.transpile(obj.Body, env, analysis, this.unParserPlugin.setParserAnalysis(analysis, this.unparser.getBuiltinFunctionList()));
        var hotFunction = new Function(["ctx"].concat(auxParams), program);
        var opt = this;
        return function () {
            //TODO: only pass context if function is impure
            return hotFunction.apply(null, Array.prototype.concat([
                opt.jsECSContext
                    .setProgram(obj.Body)
                    .setEnv(env)
            ], arguments));
        };
    };
    Optimizer.prototype.compileClass = function (obj, env) {
    };
    Optimizer.prototype.runAsJS = function (parseTree, env, program) {
        var hotFunction = new Function(["ctx"], program);
        //TODO: memoize
        return hotFunction(this.jsECSContext
            .setProgram(parseTree)
            .setEnv(env));
    };
    return Optimizer;
}());
exports.Optimizer = Optimizer;
