import { Node, Statement } from "../ast";
import { CallExpression, Expression } from "../ast/expression";
import { UnParser, UnParserPlugin } from "./unparser";
import { Identifier, StringLiteral } from "../ast/literal";
import { Token } from "../token";
import { Environment } from "../object/environment";
import { ParseTreeAnalysis } from "../parser";
export declare abstract class Transpiler implements UnParser {
    protected env?: Environment;
    protected analysis?: ParseTreeAnalysis;
    protected endOfStatementDelimiter: string;
    protected plugin?: UnParserPlugin;
    protected config: {
        noPlugins?: boolean;
    };
    protected delta: {
        [NodeName: string]: (node: Node, ctx: Transpiler) => string;
    };
    protected typeToTypeMapping: {
        [typeName: string]: string;
    };
    protected literalToTypeMapping: {
        [literalNodeName: string]: string;
    };
    isomorphicBuiltins: {
        [builtinClassName: string]: boolean;
    };
    protected builtinFunctionPolyfill: {
        [identifierPattern: string]: string;
    };
    protected callExpressionPolyfill: {
        [identifierName: string]: (node: CallExpression) => Expression;
    };
    protected methodCallPolyfill: {
        [builtinObjIdentifier: string]: {
            [methodName: string]: (node: CallExpression) => Expression;
        };
    };
    protected procedureDivision?: {
        conditionalTransforms: {
            [functionName: string]: {
                preProcessorDirectives: string[];
                code: string;
            };
        };
    };
    protected onRunWithoutPlugins: Function;
    protected injectedHeader: string;
    mutantFunctions: string[];
    injectedContextUpdate: string;
    unParse(node: Node, config?: {
        noPlugins?: boolean;
    }): string;
    getProgramHeader(): string;
    transpile: (node: Node, env?: Environment, analysis?: ParseTreeAnalysis, plugin?: UnParserPlugin, topScope?: boolean) => string;
    transpileCallExpression(node: CallExpression): CallExpression;
    transpileBlock: (statements: Statement[], topScope?: boolean) => string;
    getBuiltinFunctionList(): string[];
    routeThroughContext(indexIdent: Identifier, args: Expression[]): CallExpression;
    replaceBuiltinIdentifiers(identName: string): string;
    readonly callIndex: StringLiteral;
    readonly callBuiltinIndex: StringLiteral;
    readonly callMethodIndex: StringLiteral;
    readonly getBuiltinObjectIndex: StringLiteral;
    readonly getterIndex: StringLiteral;
    readonly setterIndex: StringLiteral;
    protected indexToken: Token;
    protected left: Identifier;
    protected getTypeFromIdentifierName(identName: string): string;
    protected inferTypeFromLiteral(node: Node): string;
    protected getTypeFromECSType(typeName: string, defaultType?: string): string;
    protected firstArgumentToObjectTransform(node: CallExpression, callName: string): CallExpression;
}
