"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var _1 = require(".");
var expression_1 = require("../ast/expression");
var literal_1 = require("../ast/literal");
var object_1 = require("../object/object");
/**
 * ECS to Javascript Un-parse-r
 */
var JSTranspiler = /** @class */ (function (_super) {
    __extends(JSTranspiler, _super);
    function JSTranspiler() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.builtinFunctionPolyfill = {
            "time": "Date.now",
        };
        _this.isomorphicBuiltins = {
            "Math": true,
            "RegExp": true
        };
        _this.typeToTypeMapping = {};
        _this.literalToTypeMapping = {};
        _this.onRunWithoutPlugins = function () {
            _this.injectedHeader += "var print=console.log;";
            _this.methodCallPolyfill.terminal.clear = function (node) { return JSTerminalClear(node); };
            _this.methodCallPolyfill.terminal.getDimensions = function (node) { return JSGetTerminalSize(); };
        };
        _this.methodCallPolyfill = {
            "terminal": {
                clear: null,
                getDimensions: null,
                beep: function (node) {
                    return node;
                }
            }
        };
        _this.callExpressionPolyfill = {
            "push": function (node) {
                return _this.firstArgumentToObjectTransform(node, "push");
            },
            "pop": function (node) {
                return _this.firstArgumentToObjectTransform(node, "pop");
            },
            "splice": function (node) {
                return _this.firstArgumentToObjectTransform(node, "splice");
            },
            "concat": function (node) {
                return _this.firstArgumentToObjectTransform(node, "concat");
            },
            "join": function (node) {
                return _this.firstArgumentToObjectTransform(node, "join");
            },
            "split": function (node) {
                return _this.firstArgumentToObjectTransform(node, "split");
            },
            "len": function (node) {
                return new expression_1.IndexExpression({ Literal: "[", Type: "[" }, node.Arguments[0], new literal_1.StringLiteral({ Literal: "length", Type: "string" }, "length"));
            }
        };
        _this.mutantFunctions = ["push", "pop", "splice"];
        _this.endOfStatementDelimiter = ";";
        _this.delta = {
            NewExpression: function (node) {
                if (_this.getTypeFromIdentifierName(node.Name.Value) == object_1.ARRAY_OBJ) {
                    return node.Name.Value + ".concat([])";
                }
                return node.UnParse(_this);
            },
            ArrayLiteral: function (node) {
                return "[" + node.Elements.map(function (el) { return _this.transpile(el); }).join(",") + "]";
            },
            HashLiteral: function (node) {
                var hashBody = "";
                var allKeys = Object.keys(node.Pairs);
                var nPairs = allKeys.length;
                for (var p = 0; p < nPairs; p++) {
                    var key = allKeys[p];
                    var value = node.Pairs[key];
                    hashBody += '"' + key + '": ' + _this.transpile(value) + (p + 1 < nPairs ? "," : "");
                }
                return "{" + hashBody + "}";
            },
            FunctionLiteral: function (node) {
                var params = node.Parameters.map(function (param) { return _this.transpile(param); }).join(",");
                var body = _this.transpileBlock(node.Body.Statements);
                return "function(" + params + "){" + body + "}";
            },
            PureFunctionLiteral: function (node) {
                var params = node.Parameters.map(function (param) { return _this.transpile(param); }).join(",");
                var body = _this.transpileBlock(node.Body.Statements);
                return "function(" + params + "){" + body + "}";
            },
            ExpressionStatement: function (node) {
                return _this.transpile(node.Expression);
            },
            LetStatement: function (node) {
                if (node.Vars) {
                    var out = "let ", lastIndex = node.Vars.length - 1, length = node.Vars.length;
                    for (var v = 0; v < length; v++) {
                        var variable = node.Vars[v];
                        out += _this.transpile(variable[0]) + " = " + _this.transpile(variable[1]);
                        if (v < lastIndex) {
                            out += ",\n";
                        }
                    }
                    return out + ";\n";
                }
                else {
                    return "let " + _this.transpile(node.Name) + " = " + _this.transpile(node.Value) + ";\n";
                }
            },
            ClassStatement: function (node) {
                var ecsConstructor = node.Value.Pairs[node.Name.Value] ? node.Value.Pairs[node.Name.Value] : null;
                var constructorArgs = ecsConstructor ? ecsConstructor.Parameters.map(function (param) { return _this.unParse(param); }) : [];
                var className = node.Name.Value;
                var proto = "var " + className + " = /** @class */ (function(){\n" +
                    "function " + className + "(" + constructorArgs.join(",") + ") { \n"
                    + _this.transpileBlock(ecsConstructor.Body.Statements) + "}\n";
                for (var method in node.Value.Pairs) {
                    var value = node.Value.Pairs[method];
                    proto += node.Name.Value + ".prototype." + method + " = " + _this.transpile(value) + ";\n";
                }
                proto += "\nreturn " + className + "}());";
                return proto;
            },
            IfStatement: function (node) {
                return "if (" + _this.transpile(node.Condition) + ") {" +
                    _this.transpileBlock(node.Consequence.Statements) +
                    "}" +
                    (node.Alternative ?
                        (" else {"
                            + _this.transpileBlock(node.Alternative.Statements) +
                            "}")
                        : "");
            },
            ForStatement: function (node) {
                var rangeNodeName = node.Range.NodeName;
                var range, counter = _this.transpile(node.Element), numericKeyType = true;
                if (rangeNodeName === "Identifier") {
                    var rangeTypeFromIdent = _this.getTypeFromIdentifierName(node.Range.Value);
                    if (rangeTypeFromIdent === "int") {
                        range = node.Range.Value;
                    }
                    else if (rangeTypeFromIdent === "array") {
                        range = _this.transpile(node.Range) + ".length";
                    }
                    else if (rangeTypeFromIdent === "object") {
                        numericKeyType = false;
                        range = _this.transpile(node.Range);
                    }
                    else {
                        range = _this.transpile(node.Range) + ".length";
                    }
                }
                else if (rangeNodeName == "ArrayLiteral") {
                    range = _this.transpile(node.Range) + ".length";
                }
                else if (rangeNodeName === "HashLiteral") {
                    numericKeyType = false;
                    range = _this.transpile(node.Range);
                }
                else if (rangeNodeName === "IntegerLiteral") {
                    range = node.Range.Value;
                }
                else {
                    range = _this.transpile(node.Range) + ".length";
                }
                var theLoop = "";
                if (numericKeyType) {
                    theLoop += "for(let " + counter + " = 0, length = "
                        + range + "; " + counter + " < length; " + counter + " ++)";
                }
                else {
                    theLoop = "let " + counter + "_keys = Object.keys(" + range + "), " + counter + "_n_keys = " + counter + "_keys.length;\n ";
                    theLoop += "for(let " + counter + "_idx = 0; " + counter + "_idx < " + counter + "_n_keys; " + counter + "_idx ++)";
                }
                theLoop += "{ \n";
                if (!numericKeyType) {
                    theLoop += "let " + counter + ' = ' + counter + '_keys[' + counter + '_idx]; \n';
                }
                theLoop += _this.transpileBlock(node.Consequence.Statements)
                    + " }";
                return theLoop;
            },
            WhileStatement: function (node) {
                return "while(" + _this.transpile(node.Condition) + ") { "
                    + _this.transpileBlock(node.Consequence.Statements)
                    + " }";
            },
            SleepStatement: function (node) {
                var consequence = new literal_1.FunctionLiteral({ Literal: "fn", Type: "fn" }, [], node.Consequence);
                return "setTimeout(" + _this.transpile(consequence) + "," + _this.transpile(node.Duration) + ");";
            }
        };
        return _this;
    }
    return JSTranspiler;
}(_1.Transpiler));
exports.JSTranspiler = JSTranspiler;
var IDENT_JS_CONSOLE = new literal_1.Identifier({ Type: "IDENT", Literal: "console" }, "console");
function JSTerminalClear(node) {
    // terminal.clear() // console.clear();
    node.Function.Left = IDENT_JS_CONSOLE;
    return node;
}
exports.JSTerminalClear = JSTerminalClear;
function JSGetTerminalSize() {
    return new literal_1.ArrayLiteral({ Literal: "[", Type: "[" }, [
        new expression_1.IndexExpression({ Type: "[", Literal: "[" }, new expression_1.IndexExpression({ Type: "[", Literal: "[" }, new literal_1.Identifier({ Type: "IDENT", Literal: 'process' }, "process"), new literal_1.StringLiteral({ Type: "IDENT", Literal: "stdout" }, "stdout")), new literal_1.StringLiteral({ Type: '"', Literal: 'columns' }, "columns")),
        new expression_1.IndexExpression({ Type: "[", Literal: "[" }, new expression_1.IndexExpression({ Type: "[", Literal: "[" }, new literal_1.Identifier({ Type: "IDENT", Literal: 'process' }, "process"), new literal_1.StringLiteral({ Type: "IDENT", Literal: "stdout" }, "stdout")), new literal_1.StringLiteral({ Type: '"', Literal: 'rows' }, "rows"))
    ]);
}
exports.JSGetTerminalSize = JSGetTerminalSize;
